Package: CatastRo
Title: Interface to the API 'Sede Electronica Del Catastro'
Version: 0.4.1
Authors@R: c(
    person("Ángel", "Delgado Panadero", , "delgadopanadero@gmail.com", role = c("aut", "cph"),
           comment = c(ORCID = "0000-0002-8189-9251")),
    person("Iñaki", "Ucar", role = "ctb",
           comment = c(ORCID = "0000-0001-6403-5550")),
    person("Diego", "Hernangómez", , "diego.hernangomezherrero@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8457-4658"))
  )
Description: Access public spatial data available under the 'INSPIRE'
    directive. Tools for downloading references and addresses of
    properties, as well as map images.
License: GPL-2
URL: https://ropenspain.github.io/CatastRo/,
        https://github.com/rOpenSpain/CatastRo
BugReports: https://github.com/rOpenSpain/CatastRo/issues
Depends: R (>= 3.6)
Imports: dplyr, httr2 (>= 1.0.0), mapSpain (>= 0.7.0), rappdirs (>=
        0.3.0), sf (>= 1.0.0), stringi, terra, tibble, xml2
Suggests: ggplot2, knitr, png, rmarkdown, slippymath, testthat (>=
        3.0.0), tidyterra
VignetteBuilder: knitr
Config/Needs/website: ropenspain/rostemplate, devtools, sessioninfo,
        remotes, sfheaders, rapidjsonr, jsonify, geometries
Config/testthat/edition: 3
Config/testthat/parallel: true
Copyright: © Dirección General del Catastro
        <https://www.catastro.meh.es/>
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
X-schema.org-isPartOf: https://ropenspain.es/
X-schema.org-keywords: catastro, cran, cran-r, gis, maps, r, r-package,
        ropenspain, rstats, spain, spatial, static-tiles
NeedsCompilation: no
Packaged: 2025-03-24 21:17:49 UTC; diego
Author: Ángel Delgado Panadero [aut, cph]
    (<https://orcid.org/0000-0002-8189-9251>),
  Iñaki Ucar [ctb] (<https://orcid.org/0000-0001-6403-5550>),
  Diego Hernangómez [aut, cre] (<https://orcid.org/0000-0001-8457-4658>)
Maintainer: Diego Hernangómez <diego.hernangomezherrero@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-24 21:30:01 UTC
Built: R 4.3.3; ; 2025-03-25 00:20:41 UTC; unix

Package: CatastRo
Title: Interface to the Spanish 'Catastro' Web Services
Version: 1.0.2
Authors@R: c(
    person("Ángel", "Delgado Panadero", , "delgadopanadero@gmail.com", role = c("aut", "cph"),
           comment = c(ORCID = "0000-0002-8189-9251")),
    person("Iñaki", "Ucar", role = "ctb",
           comment = c(ORCID = "0000-0001-6403-5550")),
    person("Diego", "Hernangómez", , "diego.hernangomezherrero@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8457-4658"))
  )
Description: Access public spatial data from the Spanish 'Catastro'
    through its 'INSPIRE' and related web services. Retrieve parcel,
    building, address and map image data, and convert between parcel
    references and coordinates.
License: GPL-2
URL: https://ropenspain.github.io/CatastRo/,
        https://github.com/rOpenSpain/CatastRo
BugReports: https://github.com/rOpenSpain/CatastRo/issues
Depends: R (>= 4.1.0)
Imports: cli, dplyr, httr2 (>= 1.0.0), lifecycle, mapSpain (>= 1.0.0),
        rappdirs (>= 0.3.0), sf (>= 1.0.0), terra, tibble, tools,
        utils, xml2
Suggests: ggplot2, knitr, quarto, testthat (>= 3.0.0), tidyterra, withr
VignetteBuilder: quarto
Config/Needs/website: ropenspain/rostemplate, devtools, sessioninfo,
        remotes, sfheaders, rapidjsonr, jsonify, geometries, magick
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Copyright: © Dirección General del Catastro
        <https://www.catastro.meh.es/>
Encoding: UTF-8
LazyData: true
X-schema.org-isPartOf: https://ropenspain.es/
X-schema.org-keywords: catastro, cran, cran-r, gis, maps, r, r-package,
        ropenspain, rstats, spain, spatial, static-tiles
NeedsCompilation: no
Packaged: 2026-06-03 19:01:18 UTC; diego
Author: Ángel Delgado Panadero [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8189-9251>),
  Iñaki Ucar [ctb] (ORCID: <https://orcid.org/0000-0001-6403-5550>),
  Diego Hernangómez [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8457-4658>)
Maintainer: Diego Hernangómez <diego.hernangomezherrero@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-03 19:10:02 UTC
Built: R 4.5.2; ; 2026-06-03 19:51:58 UTC; unix

Package: CatastRo
Title: Interface to the Spanish 'Catastro' Web Services
Version: 1.0.2
Authors@R: c(
    person("Ángel", "Delgado Panadero", , "delgadopanadero@gmail.com", role = c("aut", "cph"),
           comment = c(ORCID = "0000-0002-8189-9251")),
    person("Iñaki", "Ucar", role = "ctb",
           comment = c(ORCID = "0000-0001-6403-5550")),
    person("Diego", "Hernangómez", , "diego.hernangomezherrero@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8457-4658"))
  )
Description: Access public spatial data from the Spanish 'Catastro'
    through its 'INSPIRE' and related web services. Retrieve parcel,
    building, address and map image data, and convert between parcel
    references and coordinates.
License: GPL-2
URL: https://ropenspain.github.io/CatastRo/,
        https://github.com/rOpenSpain/CatastRo
BugReports: https://github.com/rOpenSpain/CatastRo/issues
Depends: R (>= 4.1.0)
Imports: cli, dplyr, httr2 (>= 1.0.0), lifecycle, mapSpain (>= 1.0.0),
        rappdirs (>= 0.3.0), sf (>= 1.0.0), terra, tibble, tools,
        utils, xml2
Suggests: ggplot2, knitr, quarto, testthat (>= 3.0.0), tidyterra, withr
VignetteBuilder: quarto
Config/Needs/website: ropenspain/rostemplate, devtools, sessioninfo,
        remotes, sfheaders, rapidjsonr, jsonify, geometries, magick
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Copyright: © Dirección General del Catastro
        <https://www.catastro.meh.es/>
Encoding: UTF-8
LazyData: true
X-schema.org-isPartOf: https://ropenspain.es/
X-schema.org-keywords: catastro, cran, cran-r, gis, maps, r, r-package,
        ropenspain, rstats, spain, spatial, static-tiles
NeedsCompilation: no
Packaged: 2026-06-03 19:01:18 UTC; diego
Author: Ángel Delgado Panadero [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8189-9251>),
  Iñaki Ucar [ctb] (ORCID: <https://orcid.org/0000-0001-6403-5550>),
  Diego Hernangómez [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8457-4658>)
Maintainer: Diego Hernangómez <diego.hernangomezherrero@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-03 19:10:02 UTC
Built: R 4.6.0; ; 2026-06-03 20:44:23 UTC; unix

# Generated by roxygen2: do not edit by hand

export(catr_atom_get_address)
export(catr_atom_get_address_db_all)
export(catr_atom_get_address_db_to)
export(catr_atom_get_buildings)
export(catr_atom_get_buildings_db_all)
export(catr_atom_get_buildings_db_to)
export(catr_atom_get_parcels)
export(catr_atom_get_parcels_db_all)
export(catr_atom_get_parcels_db_to)
export(catr_atom_search_munic)
export(catr_clear_cache)
export(catr_detect_cache_dir)
export(catr_get_code_from_coords)
export(catr_ovc_get_cod_munic)
export(catr_ovc_get_cod_provinces)
export(catr_ovc_get_cpmrc)
export(catr_ovc_get_rccoor)
export(catr_ovc_get_rccoor_distancia)
export(catr_set_cache_dir)
export(catr_wfs_get_address_bbox)
export(catr_wfs_get_address_codvia)
export(catr_wfs_get_address_postalcode)
export(catr_wfs_get_address_rc)
export(catr_wfs_get_buildings_bbox)
export(catr_wfs_get_buildings_rc)
export(catr_wfs_get_parcels_bbox)
export(catr_wfs_get_parcels_neigh_parcel)
export(catr_wfs_get_parcels_parcel)
export(catr_wfs_get_parcels_parcel_zoning)
export(catr_wfs_get_parcels_zoning)
export(catr_wms_get_layer)
export(inspire_wfs_get)
export(run_example)
importFrom(lifecycle,deprecated)
importFrom(utils,URLencode)
importFrom(utils,adist)
importFrom(utils,modifyList)
importFrom(utils,object.size)
importFrom(utils,unzip)

<!-- README.md is generated from README.qmd. Please edit that file -->

# CatastRo <a href="https://ropenspain.github.io/CatastRo/"><img src="man/figures/logo.png" alt="CatastRo website" align="right" height="139"/></a>

<!-- badges: start -->

[![rOS-badge](https://ropenspain.github.io/rostemplate/reference/figures/ropenspain-badge.svg)](https://ropenspain.es/)
[![CRAN-status](https://www.r-pkg.org/badges/version/CatastRo)](https://CRAN.R-project.org/package=CatastRo)
[![CRAN-results](https://badges.cranchecks.info/worst/CatastRo.svg)](https://cran.r-project.org/web/checks/check_results_CatastRo.html)
[![Downloads](https://cranlogs.r-pkg.org/badges/CatastRo)](https://CRAN.R-project.org/package=CatastRo)
[![r-universe](https://ropenspain.r-universe.dev/badges/CatastRo)](https://ropenspain.r-universe.dev/CatastRo)
[![R-CMD-check](https://github.com/rOpenSpain/CatastRo/actions/workflows/roscron-check-standard.yaml/badge.svg)](https://github.com/rOpenSpain/CatastRo/actions/workflows/roscron-check-standard.yaml)
[![codecov](https://codecov.io/gh/rOpenSpain/CatastRo/graph/badge.svg?token=KPPwTkZjW6)](https://app.codecov.io/gh/rOpenSpain/CatastRo)
[![DOI](https://img.shields.io/badge/DOI-10.32614/CRAN.package.CatastRo-blue)](https://doi.org/10.32614/CRAN.package.CatastRo)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)

<!-- badges: end -->

**CatastRo** provides access to services from the [Spanish
Cadastre](https://www.sedecatastro.gob.es/). With **CatastRo**, you can
download buildings, cadastral parcels, addresses and map imagery,
geocode cadastral references and look up cadastral references from
coordinates.

## Installation

Install **CatastRo** from
[**CRAN**](https://CRAN.R-project.org/package=CatastRo):

``` r
install.packages("CatastRo")
```

Check the documentation for the development version at
<https://ropenspain.github.io/CatastRo/dev/>.

You can install the development version of **CatastRo** from
[r-universe](https://ropenspain.r-universe.dev/CatastRo):

``` r
# Install CatastRo in R.
install.packages(
  "CatastRo",
  repos = c(
    "https://ropenspain.r-universe.dev",
    "https://cloud.r-project.org"
  )
)
```

Alternatively, you can install the development version of **CatastRo**
with:

``` r
pak::pak("rOpenSpain/CatastRo")
```

> [!WARNING]
>
> ### SSL issues
>
> The SSL certificate of the Spanish Cadastre may cause an error when
> using **CatastRo** (especially on macOS, see issue
> [\#40](https://github.com/rOpenSpain/CatastRo/issues/40)):
>
> In **CatastRo \>= 1.0.0**, you can try to fix it by running this line
> after loading the package:
>
> ``` r
> # Disable SSL verification
> options(catastro_ssl_verify = 0)
> ```
>
> If you wish to make this setup persistent, write the same code in your
> [`.Rprofile`](https://docs.posit.co/ide/user/ide/guide/environments/r/managing-r.html):
>
> ``` r
> # ... other options...
> options(catastro_ssl_verify = 0)
> ```
>
> Modify your `.Rprofile` with `usethis::edit_r_profile()`.

## Package API

The functions of **CatastRo** are organized by source service. The
package naming convention is `catr_*service*_*description*`.

### OVC services

OVC services cover geocoding and reverse geocoding with
[OVCCoordenadas](https://ovc.catastro.meh.es/ovcservweb/OVCSWLocalizacionRC/OVCCoordenadas.asmx),
and province and municipality code lookup with OVCCallejero.

These functions use the `catr_ovc_get_*()` prefix and return tibbles
from the **tibble** package. See
`vignette("ovcservice", package = "CatastRo")` for a detailed
description of these functions.

### INSPIRE

INSPIRE functions return spatial objects from the [Spanish Cadastre
INSPIRE](https://www.catastro.hacienda.gob.es/webinspire/index.html)
services using the **sf** or **terra** packages.

Note that these services cover 95% of the Spanish territory, excluding
the Basque Country and Navarre[^1], which have their own independent
cadastral offices.

There are three types of functions, each querying a different service:

#### ATOM service

The ATOM service downloads complete municipal datasets for different
cadastral elements. Results are returned as `sf` objects from the **sf**
package.

These functions use the `catr_atom_get_*()` prefix.

#### WFS service

The WFS service downloads vector objects for specific cadastral
elements. Results are returned as `sf` objects from the
[**sf**](https://r-spatial.github.io/sf/) package. Note that there are
restrictions on the extent and number of elements that can be queried.
For full municipal downloads, prefer the ATOM service.

These functions use the `catr_wfs_get_*()` prefix.

#### WMS service

The WMS service downloads georeferenced map images for different
cadastral elements. Results are returned as `SpatRaster` objects from
the
[**terra**](https://rspatial.github.io/terra/reference/terra-package.html)
package.

There is a single function for querying this service:
`catr_wms_get_layer()`.

#### Terms and conditions of use

Please check the service [terms of
use](https://www.catastro.hacienda.gob.es/webinspire/documentos/Licencia.pdf).

## Examples

This script highlights some features of **CatastRo**:

### Geocode a cadastral reference

``` r
library(CatastRo)

catr_ovc_get_cpmrc(rc = "13077A01800039")
#> # A tibble: 1 × 10
#>   xcoord ycoord refcat     address pc.pc1 pc.pc2 geo.xcen geo.ycen geo.srs ldt  
#>    <dbl>  <dbl> <chr>      <chr>   <chr>  <chr>  <chr>    <chr>    <chr>   <chr>
#> 1  -3.46   38.6 13077A018… DS DIS… 13077… 18000… -3.4575… 38.6184… EPSG:4… DS D…
```

### Reverse geocode coordinates to a cadastral reference

``` r
catr_ovc_get_rccoor(
  lat = 38.6196566583596,
  lon = -3.45624183836806,
  srs = "4230"
)
#> # A tibble: 1 × 8
#>   refcat         address           pc.pc1 pc.pc2 geo.xcen geo.ycen geo.srs ldt  
#>   <chr>          <chr>             <chr>  <chr>     <dbl>    <dbl> <chr>   <chr>
#> 1 13077A01800039 DS DISEMINADO  P… 13077… 18000…    -3.46     38.6 EPSG:4… DS D…
```

### Extract geometries using the ATOM service

``` r
bu <- catr_atom_get_buildings("Nava de la Asuncion", to = "Segovia")

# Map.
library(ggplot2)

ggplot(bu) +
  geom_sf(aes(fill = currentUse), col = NA) +
  coord_sf(
    xlim = c(374500, 375500),
    ylim = c(4556500, 4557500)
  ) +
  scale_fill_manual(values = hcl.colors(6, "Dark 3")) +
  theme_minimal() +
  labs(title = "Nava de la Asunción, Segovia")
```

<img src="man/figures/README-atom-1.png" style="width:100.0%"
alt="Extract buildings in Nava de la Asuncion with the ATOM service" />

### Extract geometries using the WFS service

``` r
wfs_get_buildings <- catr_wfs_get_buildings_bbox(
  c(-4.134, 40.952, -4.131, 40.953),
  srs = 4326
)

# Map.
ggplot(wfs_get_buildings) +
  geom_sf() +
  labs(title = "Alcázar of Segovia, Segovia, Spain")
```

<img src="man/figures/README-wfs-1.png" style="width:100.0%"
alt="Extract Alcázar of Segovia with the WFS service" />

## A note on caching

Some datasets and tiles may exceed 50 MB. You can set a local cache
directory using the following function:

``` r
catr_set_cache_dir("./path/to/location")
```

When this option is set, **CatastRo** looks for cached files and loads
them, which speeds up repeated queries.

## Citation

<p>

Delgado Panadero Á, Hernangómez D (2026). <em>CatastRo: Interface to the
API Sede Electrónica Del Catastro</em>.
<a href="https://doi.org/10.32614/CRAN.package.CatastRo">doi:10.32614/CRAN.package.CatastRo</a>.
<a href="https://ropenspain.github.io/CatastRo/">https://ropenspain.github.io/CatastRo/</a>.
</p>

A BibTeX entry for LaTeX users is:

    @Manual{R-CatastRo,
      title = {{CatastRo}: Interface to the {API} Sede Electrónica Del Catastro},
      author = {Ángel {Delgado Panadero} and Diego Hernangómez},
      doi = {10.32614/CRAN.package.CatastRo},
      year = {2026},
      version = {1.0.2},
      url = {https://ropenspain.github.io/CatastRo/},
      abstract = {Access public spatial data from the Spanish Catastro through its INSPIRE and related web services. Retrieve parcel, building, address and map image data, and convert between parcel references and coordinates.},
    }

## Contribute

Check the GitHub page for the [source
code](https://github.com/ropenspain/CatastRo/).

[^1]: The package
    [**CatastRoNav**](https://ropenspain.github.io/CatastRoNav/)
    provides access to the Cadastre of Navarre, with similar
    functionality to **CatastRo**.

<!-- README.md is generated from README.qmd. Please edit that file -->

# CatastRo <a href="https://ropenspain.github.io/CatastRo/"><img src="man/figures/logo.png" alt="CatastRo website" align="right" height="139"/></a>

<!-- badges: start -->

[![rOS-badge](https://ropenspain.github.io/rostemplate/reference/figures/ropenspain-badge.svg)](https://ropenspain.es/)
[![CRAN-status](https://www.r-pkg.org/badges/version/CatastRo)](https://CRAN.R-project.org/package=CatastRo)
[![CRAN-results](https://badges.cranchecks.info/worst/CatastRo.svg)](https://cran.r-project.org/web/checks/check_results_CatastRo.html)
[![Downloads](https://cranlogs.r-pkg.org/badges/CatastRo)](https://CRAN.R-project.org/package=CatastRo)
[![r-universe](https://ropenspain.r-universe.dev/badges/CatastRo)](https://ropenspain.r-universe.dev/CatastRo)
[![R-CMD-check](https://github.com/rOpenSpain/CatastRo/actions/workflows/roscron-check-standard.yaml/badge.svg)](https://github.com/rOpenSpain/CatastRo/actions/workflows/roscron-check-standard.yaml)
[![codecov](https://codecov.io/gh/rOpenSpain/CatastRo/graph/badge.svg?token=KPPwTkZjW6)](https://app.codecov.io/gh/rOpenSpain/CatastRo)
[![DOI](https://img.shields.io/badge/DOI-10.32614/CRAN.package.CatastRo-blue)](https://doi.org/10.32614/CRAN.package.CatastRo)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)

<!-- badges: end -->

**CatastRo** is a package that provides access to different API services
of the [Spanish Cadastre](https://www.sedecatastro.gob.es/). With
**CatastRo**, you can download spatial objects such as buildings,
cadastral parcels and maps, and geocode cadastral references.

## Installation

Install **CatastRo** from
[**CRAN**](https://CRAN.R-project.org/package=CatastRo):

``` r
install.packages("CatastRo")
```

Check the docs of the developing version in
<https://ropenspain.github.io/CatastRo/dev/>.

You can install the developing version of **CatastRo** using the
[r-universe](https://ropenspain.r-universe.dev/CatastRo):

``` r
# Install CatastRo in R:
install.packages(
  "CatastRo",
  repos = c(
    "https://ropenspain.r-universe.dev",
    "https://cloud.r-project.org"
  )
)
```

Alternatively, you can install the development version of **CatastRo**
with:

``` r
pak::pak("rOpenSpain/CatastRo")
```

> [!WARNING]
>
> ### SSL issues
>
> The SSL certificate of the Spanish Cadastre presents some issues that
> may cause an error when using **CatastRo** (especially on macOS, see
> issue [\#40](https://github.com/rOpenSpain/CatastRo/issues/40)):
>
> In **CatastRo \>= 1.0.0** you can try to fix it by running this line
> in your session right after you start using the package:
>
> ``` r
> # Disable SSL verification
> options(catastro_ssl_verify = 0)
> ```
>
> If you wish to make this setup persistent, write the same code in your
> [`.Rprofile`](https://docs.posit.co/ide/user/ide/guide/environments/r/managing-r.html):
>
> ``` r
> # ... other options...
> options(catastro_ssl_verify = 0)
> ```
>
> Modify your `.Rprofile` with `usethis::edit_r_profile()`.

## Package API

The functions of **CatastRo** are organised by API endpoint. The package
naming convention is `catr_*api*_*description*`.

### OVCCoordenadas

These functions allow geocoding and reverse geocoding of cadastral
references using the
[OVCCoordenadas](https://ovc.catastro.meh.es/ovcservweb/OVCSWLocalizacionRC/OVCCoordenadas.asmx)
service.

These functions are named `catr_ovc_get_*` and return a tibble, as
provided by package **tibble**. See
`vignette("ovcservice", package = "CatastRo")` where these functions are
described.

### INSPIRE

These functions return spatial objects in the formats provided by the
**sf** or **terra** packages using the [Catastro
INSPIRE](https://www.catastro.hacienda.gob.es/webinspire/index.html)
service.

Note that the coverage of this service is 95% of the Spanish territory,
excluding the Basque Country and Navarre[^1], which have their own
independent cadastral offices.

There are three types of functions, each one querying a different
service:

#### ATOM service

The ATOM service allows batch-downloading vector objects of different
cadastral elements for a specific municipality. The result is provided
as `sf` objects (see the **sf** package).

These functions are named `catr_atom_get_xxx`.

#### WFS service

The WFS service allows downloading vector objects of specific cadastral
elements. The results are provided as `sf` objects (see the
[**sf**](https://r-spatial.github.io/sf/) package). Note that there are
restrictions on the extent and number of elements that can be queried.
For batch downloading, the ATOM service is preferred.

These functions are named `catr_wfs_get_xxx`.

#### WMS service

This service allows downloading georeferenced images of different
cadastral elements. The results are provided as rasters in the format
provided by the
[**terra**](https://rspatial.github.io/terra/reference/terra-package.html)
package.

There is a single function for querying this service:
`catr_wms_get_layer()`.

#### Terms and conditions of use

Please check the service’s [downloading
provisions](https://www.catastro.hacienda.gob.es/webinspire/documentos/Licencia.pdf).

## Examples

This script highlights some features of **CatastRo**:

### Geocode a cadastral reference

``` r
library(CatastRo)

catr_ovc_get_cpmrc(rc = "13077A01800039")
#> # A tibble: 1 × 10
#>   xcoord ycoord refcat     address pc.pc1 pc.pc2 geo.xcen geo.ycen geo.srs ldt  
#>    <dbl>  <dbl> <chr>      <chr>   <chr>  <chr>  <chr>    <chr>    <chr>   <chr>
#> 1  -3.46   38.6 13077A018… DS DIS… 13077… 18000… -3.4575… 38.6184… EPSG:4… DS D…
```

### Extract a cadastral reference from a given set of coordinates

``` r
catr_ovc_get_rccoor(
  lat = 38.6196566583596,
  lon = -3.45624183836806,
  srs = "4230"
)
#> # A tibble: 1 × 8
#>   refcat         address           pc.pc1 pc.pc2 geo.xcen geo.ycen geo.srs ldt  
#>   <chr>          <chr>             <chr>  <chr>     <dbl>    <dbl> <chr>   <chr>
#> 1 13077A01800039 DS DISEMINADO  P… 13077… 18000…    -3.46     38.6 EPSG:4… DS D…
```

### Extract geometries using the ATOM service

``` r
bu <- catr_atom_get_buildings("Nava de la Asuncion", to = "Segovia")

# Map
library(ggplot2)

ggplot(bu) +
  geom_sf(aes(fill = currentUse), col = NA) +
  coord_sf(
    xlim = c(374500, 375500),
    ylim = c(4556500, 4557500)
  ) +
  scale_fill_manual(values = hcl.colors(6, "Dark 3")) +
  theme_minimal() +
  labs(title = "Nava de la Asunción, Segovia")
```

<img src="man/figures/README-atom-1.png" style="width:100.0%"
alt="Extracting buildings in Nava de la Asuncion with the ATOM service" />

### Extract geometries using the WFS service

``` r
wfs_get_buildings <- catr_wfs_get_buildings_bbox(
  c(-4.134, 40.952, -4.131, 40.953),
  srs = 4326
)

# Map
ggplot(wfs_get_buildings) +
  geom_sf() +
  labs(title = "Alcázar of Segovia, Segovia, Spain")
```

<img src="man/figures/README-wfs-1.png" style="width:100.0%"
alt="Extract Alcázar of Segovia with the WFS service" />

## A note on caching

Some datasets and tiles may exceed 50MB. You can set a local cache
directory using the following function:

``` r
catr_set_cache_dir("./path/to/location")
```

When this option is set, **CatastRo** will look for cached files and
load them, speeding up the process.

## Citation

<p>

Delgado Panadero Á, Hernangómez D (2026). <em>CatastRo: Interface to the
API Sede Electrónica Del Catastro</em>.
<a href="https://doi.org/10.32614/CRAN.package.CatastRo">doi:10.32614/CRAN.package.CatastRo</a>.
<a href="https://ropenspain.github.io/CatastRo/">https://ropenspain.github.io/CatastRo/</a>.
</p>

A BibTeX entry for LaTeX users is:

    @Manual{R-CatastRo,
      title = {{CatastRo}: Interface to the {API} Sede Electrónica Del Catastro},
      author = {Ángel {Delgado Panadero} and Diego Hernangómez},
      doi = {10.32614/CRAN.package.CatastRo},
      year = {2026},
      version = {1.0.1},
      url = {https://ropenspain.github.io/CatastRo/},
      abstract = {Access public spatial data available under the INSPIRE directive. Tools for downloading references and addresses of properties, as well as map images.},
    }

## Contribute

Check the GitHub page for the [source
code](https://github.com/ropenspain/CatastRo/).

[^1]: The package
    [**CatastRoNav**](https://ropenspain.github.io/CatastRoNav/)
    provides access to the Cadastre of Navarre, with similar
    functionalities to **CatastRo**.

#' ATOM INSPIRE: Download all cadastral parcels of a municipality
#'
#' @description
#' Retrieve the spatial data of all cadastral parcels belonging to a single
#' municipality using the INSPIRE ATOM service.
#'
#' @encoding UTF-8
#' @family INSPIRE
#' @family ATOM
#' @family parcels
#' @family spatial
#' @inheritParams catr_atom_get_address
#' @export
#'
#' @inherit catr_atom_get_address references return
#' @param what Information to load. Options are:
#'   - `"parcel"` for cadastral parcels.
#'   - `"zoning"` for cadastral zoning.
#'
#' @examplesIf run_example()
#' \donttest{
#' s <- catr_atom_get_parcels("Melque", to = "Segovia", what = "parcel")
#'
#' library(ggplot2)
#'
#' ggplot(s) +
#'   geom_sf() +
#'   labs(
#'     title = "Cadastral Parcels",
#'     subtitle = "Melque de Cercos, Segovia"
#'   )
#' }
#'
catr_atom_get_parcels <- function(
  munic,
  to = NULL,
  what = c("parcel", "zoning"),
  cache = deprecated(),
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE
) {
  if (lifecycle::is_present(cache)) {
    lifecycle::deprecate_warn(
      when = "1.0.0",
      what = "CatastRo::catr_atom_get_parcels(cache)",
      details = "Results are always cached."
    )
  }

  # Sanity checks
  what <- match_arg_pretty(what)
  munic <- validate_non_empty_arg(munic)
  to <- ensure_null(to)

  all <- catr_atom_get_parcels_db_all(
    update_cache = update_cache,
    cache_dir = cache_dir,
    verbose = FALSE
  )
  if (is.null(all)) {
    return(NULL)
  }

  if (!is.null(to)) {
    linesto <- grep(to, all$territorial_office, ignore.case = TRUE)

    # Filter by territorial office if matches found
    if (length(linesto) > 1) {
      all <- all[linesto, ]
    } else {
      if (verbose) {
        cli::cli_alert_warning(
          paste0(
            "Ignoring {.arg to} argument. No results ",
            "found with pattern {.str {munic}} in {.str {to}}."
          )
        )
      }
    }
  }

  to_loc <- ensure_null(grep(munic, all$munic, ignore.case = TRUE))

  if (is.null(to_loc)) {
    cli::cli_alert_warning(
      "No municipality found with pattern {.str {munic}}."
    )
    cli::cli_alert_info(
      paste0(
        "Check available municipalities with ",
        "{.fn CatastRo::catr_atom_get_address_db_all}."
      )
    )
    return(NULL)
  }

  # Compute string distances for municipality matching
  with_d <- data.frame(
    munic = all$munic,
    territorial_office = all$territorial_office,
    dist = as.vector(adist(munic, all$munic))
  )
  with_d <- with_d[to_loc, ]
  tb <- with_d[order(with_d$dist), ]

  if (nrow(tb) > 1) {
    cli::cli_alert_info(
      "Found {nrow(tb)} municipalities with pattern {.str {munic}}."
    )

    cli::cli_alert_success("Selecting {.str {tb[1,]$munic}}.")
    cli::cli_alert_danger("Discarding:")
    bullets <- tb[-1, ]$munic
    bullets <- paste0("{.str ", bullets, "}")
    names(bullets) <- rep(" ", length(bullets))
    cli::cli_bullets(bullets)

    tb <- tb[1, ]
  }

  make_msg(
    "info",
    verbose,
    paste0("Extracting information for {.str ", tb$munic, "}.")
  )

  municurls <- catr_atom_get_parcels_db_to(
    as.character(tb$territorial_office),
    cache = cache,
    update_cache = update_cache,
    cache_dir = cache_dir,
    verbose = FALSE
  )
  # Extract municipality code from reference string
  ref <- unlist(strsplit(tb$munic, "-"))[1]

  # Prepare download URL for municipality data
  api_entry <- municurls[
    grepl(ref, municurls$munic, ignore.case = TRUE),
  ]$url

  api_entry <- URLencode(api_entry)

  file_local <- download_url(
    url = api_entry,
    cache_dir = cache_dir,
    subdir = "atom_cp",
    update_cache = update_cache,
    verbose = verbose
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  sfobj <- read_geo_file_sf(file_local, hint = what)

  sfobj
}

#' Set your \CRANpkg{CatastRo} cache dir
#'
#' @family cache utilities
#' @seealso [tools::R_user_dir()]
#'
#' @rdname catr_set_cache_dir
#'
#' @description
#' Store your `cache_dir` path on your local machine for future sessions.
#' Type `Sys.getenv("CATASTROESP_CACHE_DIR")` to find your cached path or
#' use [catr_detect_cache_dir()].
#' @encoding UTF-8
#'
#' @param cache_dir Path to a cache directory. On `NULL`, the function
#'   stores cached files in a temporary directory (see [base::tempdir()]).
#' @param install Logical. If `TRUE`, installs the key on your local
#'   machine for use in future sessions. Defaults to `FALSE`. If `cache_dir`
#'   is `FALSE`, this argument is set to `FALSE` automatically.
#' @param overwrite Logical. If `TRUE`, overwrites an existing
#'   `CATASTROESP_CACHE_DIR` already present on your machine.
#' @param verbose Logical. If `TRUE`, displays informational messages.
#'
#' @details
#' By default, when no `cache_dir` is set, the package uses a folder inside
#' [base::tempdir()] (so files are temporary and are removed when the **R**
#' session ends). To persist a cache across **R** sessions, use
#' `catr_set_cache_dir(cache_dir, install = TRUE)` which writes the chosen
#' path to a small configuration file under
#' `tools::R_user_dir("CatastRo", "config")`.
#'
#' @return
#' `catr_set_cache_dir()` returns an (invisible) character with the path to
#' your `cache_dir`, but it is mainly called for its side effect.
#'
#' @section Caching strategies:
#'
#' Some files can be read from their online source without caching using the
#' option `cache = FALSE`. Otherwise the source file is downloaded to
#' your computer. \CRANpkg{CatastRo} implements the following caching options:
#'
#' - For occasional use, rely on the default [tempdir()]-based cache (no
#'   install).
#' - Modify the cache for a single session setting
#'   `catr_set_cache_dir(cache_dir = "a/path/here")`.
#' - For reproducible workflows, install a persistent cache with
#'   `catr_set_cache_dir(cache_dir = "a/path/here", install = TRUE)`.
#'   This cache is kept across **R** sessions.
#' - For caching specific files, use the `cache_dir` argument in the
#'   corresponding function.
#'
#' Sometimes cached files may be corrupt. In that case, try re-downloading
#' the data setting `update_cache = TRUE` in the corresponding function.
#'
#' If you experience any problem downloading, try downloading the
#' corresponding file by another method and save it in your
#' `cache_dir`. Use the option `verbose = TRUE` for debugging the API query
#' and [catr_detect_cache_dir()] to identify your cached path.
#'
#' @note
#'
#' In \CRANpkg{CatastRo} >= 1.0.0 the location of the configuration file has
#' moved from `rappdirs::user_config_dir("CatastRo", "R")` to
#' `tools::R_user_dir("CatastRo", "config")`. We have implemented a
#' functionality that migrates previous configuration files from one
#' location to another with a message. This message appears only once
#' informing of the migration.
#'
#' @examples
#'
#' # Caution! It modifies your current state
#' \dontrun{
#' my_cache <- catr_detect_cache_dir()
#'
#' # Set an example cache
#' ex <- file.path(tempdir(), "example", "cachenew")
#' catr_set_cache_dir(ex)
#'
#' catr_detect_cache_dir()
#'
#' # Restore initial cache
#' catr_set_cache_dir(my_cache)
#' identical(my_cache, catr_detect_cache_dir())
#' }
#'
#' @export
catr_set_cache_dir <- function(
  cache_dir = NULL,
  overwrite = FALSE,
  install = FALSE,
  verbose = TRUE
) {
  cache_dir <- ensure_null(cache_dir)

  # Default if not provided
  if (is.null(cache_dir)) {
    make_msg(
      "info",
      verbose,
      "Using a temporary cache dir (see {.fn base::tempdir}). ",
      "Set {.arg cache_dir} to a value to store permanently."
    )

    # Create a folder on tempdir
    cache_dir <- file.path(tempdir(), "CatastRo")
    is_temp <- TRUE
    install <- FALSE
  } else {
    is_temp <- FALSE
  }

  # Validate
  stopifnot(
    is.character(cache_dir),
    is.logical(overwrite),
    is.logical(install)
  )

  # Create and expand
  cache_dir <- create_cache_dir(cache_dir)
  msg <- paste0("{.pkg CatastRo} cache dir is {.path ", cache_dir, "}.")
  make_msg("info", verbose, msg)

  # Install path on environ var.
  # nocov start

  if (install) {
    config_dir <- tools::R_user_dir("CatastRo", "config")
    # Create cache dir if not presente
    if (!dir.exists(config_dir)) {
      dir.create(config_dir, recursive = TRUE)
    }

    catastroesp_file <- file.path(config_dir, "CATASTROESP_CACHE_DIR")

    if (!file.exists(catastroesp_file) || overwrite) {
      # Create file if it doesn't exist
      writeLines(cache_dir, con = catastroesp_file)
    } else {
      cli::cli_abort(
        c(
          "A {.arg cache_dir} path already exists.",
          "You can overwrite it with {.arg overwrite = TRUE}."
        )
      )
    }
    # nocov end
  } else {
    make_msg(
      "info",
      verbose && !is_temp,
      "To install your {.arg cache_dir} path for use in future sessions",
      "run this function with {.arg install = TRUE}."
    )
  }

  Sys.setenv(CATASTROESP_CACHE_DIR = cache_dir)
  invisible(cache_dir)
}

#' @export
#' @rdname catr_set_cache_dir
#' @return
#' `catr_detect_cache_dir()` returns the path to the `cache_dir` used in this
#' session.
#'
#' @examples
#'
#' catr_detect_cache_dir()
#'
catr_detect_cache_dir <- function() {
  cd <- detect_cache_dir_muted()
  cli::cli_alert_info("{.path {cd}}")
  cd
}

#' Clear your \CRANpkg{CatastRo} cache dir
#'
#' @rdname catr_clear_cache
#' @family cache utilities
#' @encoding UTF-8
#'
#' @return Invisible. This function is called for its side effects.
#'
#' @description
#' **Use this function with caution**. This function clears your cached data
#' and configuration, specifically:
#'
#' - Deletes the \CRANpkg{CatastRo} config directory
#'   (`tools::R_user_dir("CatastRo", "config")`).
#' - Deletes the `cache_dir` directory.
#' - Deletes the values stored on `Sys.getenv("CATASTROESP_CACHE_DIR")`.
#'
#' @param config If `TRUE`, deletes the configuration folder of
#'   \CRANpkg{CatastRo}.
#' @param cached_data If `TRUE`, deletes your `cache_dir` and all its contents.
#' @inheritParams catr_set_cache_dir
#'
#' @seealso [tools::R_user_dir()]
#'
#' @details
#' This is an overkill function that is intended to reset your status
#' as if you had never installed and/or used \CRANpkg{CatastRo}.
#'
#' @examples
#'
#' # Don't run this! It modifies your current state
#' \dontrun{
#' my_cache <- catr_detect_cache_dir()
#'
#' # Set an example cache
#' ex <- file.path(tempdir(), "example", "cache")
#' catr_set_cache_dir(ex, verbose = FALSE)
#'
#' # Restore initial cache
#' catr_clear_cache(verbose = TRUE)
#'
#' catr_set_cache_dir(my_cache)
#' identical(my_cache, catr_detect_cache_dir())
#' }
#' @export
catr_clear_cache <- function(
  config = FALSE,
  cached_data = TRUE,
  verbose = FALSE
) {
  migrate_cache()

  config_dir <- tools::R_user_dir("CatastRo", "config")
  data_dir <- detect_cache_dir_muted()

  # nocov start
  if (config && dir.exists(config_dir)) {
    unlink(config_dir, recursive = TRUE, force = TRUE)

    if (verbose) {
      cli::cli_alert_warning("{.pkg CatastRo} cache config deleted")
    }
  }
  # nocov end
  if (cached_data && dir.exists(data_dir)) {
    siz <- file.size(list.files(
      data_dir,
      recursive = TRUE,
      full.names = TRUE
    ))
    siz <- sum(siz, na.rm = TRUE)
    class(siz) <- class(object.size("a"))

    siz <- format(siz, unit = "auto")

    unlink(data_dir, recursive = TRUE, force = TRUE)
    if (verbose) {
      cli::cli_alert_warning(
        "{.pkg CatastRo} data deleted: {.file {data_dir}} ({siz})."
      )
    }
  }

  Sys.setenv(CATASTROESP_CACHE_DIR = "")

  # Reset cache dir
  invisible()
}

# Internal funs ----

#' Detects cache dir silently
#'
#' @return path to cache dir
#' @noRd
detect_cache_dir_muted <- function() {
  migrate_cache()

  # Try from getenv
  getvar <- Sys.getenv("CATASTROESP_CACHE_DIR")

  if (is.null(getvar) || is.na(getvar) || getvar == "") {
    # Not set - tries to retrieve from cache
    cache_config <- file.path(
      tools::R_user_dir("CatastRo", "config"),
      "CATASTROESP_CACHE_DIR"
    )

    # nocov start
    if (file.exists(cache_config)) {
      cached_path <- readLines(cache_config)

      # Case on empty cached path - default
      if (
        any(
          is.null(cached_path),
          is.na(cached_path),
          cached_path == ""
        )
      ) {
        cache_dir <- catr_set_cache_dir(overwrite = TRUE, verbose = FALSE)
        return(cache_dir)
      }

      # 3. Return from cached path
      Sys.setenv(CATASTROESP_CACHE_DIR = cached_path)
      cached_path
      # nocov end
    } else {
      # 4. Default cache location

      cache_dir <- catr_set_cache_dir(overwrite = TRUE, verbose = FALSE)
      cache_dir
    }
  } else {
    getvar
  }
}

#' Creates `cache_dir` if not exists
#'
#' @param cache_dir path to cache dir
#' @return path to cache dir
#'
#' @noRd
create_cache_dir <- function(cache_dir = NULL) {
  # Check cache dir from options if not set
  if (is.null(cache_dir)) {
    cache_dir <- detect_cache_dir_muted()
  }

  cache_dir <- path.expand(cache_dir)

  # Create cache dir if needed
  if (isFALSE(dir.exists(cache_dir))) {
    dir.create(cache_dir, recursive = TRUE)
  }
  cache_dir
}

#' Migrate cache config from rappdirs to tools
#'
#' One-time function for CatastRo >= 1.0.0
#' @param old Path to old cache config folder
#' @param new Path to new cache config folder
#'
#' @noRd
migrate_cache <- function(
  old = rappdirs::user_config_dir("CatastRo", "R"),
  new = tools::R_user_dir("CatastRo", "config")
) {
  fname <- "CATASTROESP_CACHE_DIR"

  old_fname <- file.path(old, fname)
  new_fname <- file.path(new, fname)

  if (file.exists(new_fname)) {
    unlink(old, force = TRUE, recursive = TRUE)
    return(invisible())
  }

  if (file.exists(old_fname)) {
    cache_dir <- readLines(old_fname)
    catr_set_cache_dir(cache_dir, install = TRUE, verbose = FALSE)
    cli::cli_alert_success(
      c(
        "{.pkg CatastRo} >= 1.0.0: Cache configuration migrated. ",
        "See {.strong Note} in {.fn CatastRo::catr_set_cache_dir} for details."
      )
    )
    cli::cli_alert_info(
      "This is a one-time message, it won't be displayed in the future."
    )
  }
  unlink(old, force = TRUE, recursive = TRUE)

  invisible()
}

test_that("Test cache", {
  skip_on_cran()
  # Get current cache dir
  expect_message(current <- catr_detect_cache_dir())

  # Set a temp cache dir
  expect_message(catr_set_cache_dir(verbose = TRUE))
  testdir <- expect_silent(catr_set_cache_dir(
    file.path(current, "testthat"),
    verbose = FALSE
  ))

  expect_identical(catr_detect_cache_dir(), testdir)

  # Clean
  expect_silent(catr_clear_cache(config = FALSE, verbose = FALSE))
  # Cache dir should be deleted now
  expect_false(dir.exists(testdir))

  # Reset just for testing all cases
  testdir <- file.path(tempdir(), "catastro", "testthat")
  expect_message(catr_set_cache_dir(testdir))

  expect_true(dir.exists(testdir))

  expect_message(catr_clear_cache(config = FALSE, verbose = TRUE))

  # Cache dir should be deleted now
  expect_false(dir.exists(testdir))

  # Restore cache
  expect_message(catr_set_cache_dir(current, verbose = TRUE))
  expect_silent(catr_set_cache_dir(current, verbose = FALSE))
  expect_equal(current, Sys.getenv("CATASTROESP_CACHE_DIR"))
  expect_true(dir.exists(current))
})

test_that("Mock restart", {
  skip_on_cran()
  # Store current value
  getvar <- Sys.getenv("CATASTROESP_CACHE_DIR")

  # New empty value
  Sys.unsetenv("CATASTROESP_CACHE_DIR")
  expect_identical(Sys.getenv("CATASTROESP_CACHE_DIR"), "")

  # Careful!
  cache_config <- file.path(
    tools::R_user_dir("CatastRo", "config"),
    "CATASTROESP_CACHE_DIR"
  )
  tester_has_config_installed <- file.exists(cache_config)

  if (tester_has_config_installed) {
    stored_val <- readLines(cache_config)
    catr_clear_cache(cached_data = FALSE, config = TRUE)
    expect_false(file.exists(cache_config))
    expect_true(Sys.getenv("CATASTROESP_CACHE_DIR") == "")

    # We are clear now, we should detect default cache location
    default_loc <- detect_cache_dir_muted()

    # Should be the tempdir
    expect_identical(file.path(tempdir(), "CatastRo"), default_loc)

    # Now we should restore the cache
    expect_message(
      catr_set_cache_dir(stored_val, overwrite = TRUE, install = TRUE),
      "cache dir is"
    )

    # But for the next test we delete the envar
    Sys.unsetenv("CATASTROESP_CACHE_DIR")
    expect_identical(Sys.getenv("CATASTROESP_CACHE_DIR"), "")
  }

  muted <- detect_cache_dir_muted()
  created <- create_cache_dir()
  muted2 <- detect_cache_dir_muted()

  expect_identical(muted, created)
  expect_identical(muted, muted2)
  expect_false(Sys.getenv("CATASTROESP_CACHE_DIR") == "")

  # Restore cache
  if (tester_has_config_installed) {
    catr_set_cache_dir(
      stored_val,
      install = TRUE,
      overwrite = TRUE,
      verbose = FALSE
    )
  }

  # Session value (may differ from current)
  catr_set_cache_dir(getvar, install = FALSE)

  Sys.setenv("CATASTROESP_CACHE_DIR" = getvar)
})

test_that("Mock migration", {
  skip_on_cran()

  # Store current value
  getvar <- Sys.getenv("CATASTROESP_CACHE_DIR")
  # New empty value
  Sys.unsetenv("CATASTROESP_CACHE_DIR")
  expect_identical(Sys.getenv("CATASTROESP_CACHE_DIR"), "")

  # Delete now cache files
  old <- rappdirs::user_config_dir("CatastRo", "R")
  new <- tools::R_user_dir("CatastRo", "config")
  fname <- "CATASTROESP_CACHE_DIR"

  old_fname <- file.path(old, fname)
  new_fname <- file.path(new, fname)
  tester_has_config_installed <- file.exists(new_fname)

  unlink(old_fname)
  unlink(new_fname)

  expect_false(file.exists(old_fname))
  expect_false(file.exists(new_fname))

  # Create an old cache config
  nnn <- create_cache_dir(old)
  writeLines(tempdir(), old_fname)
  expect_true(file.exists(old_fname))

  # On detect we should see a message
  expect_snapshot(detected <- detect_cache_dir_muted())
  # And never again
  expect_silent(detected2 <- detect_cache_dir_muted())
  expect_identical(detected, detected2)
  expect_identical(detected, tempdir())
  expect_identical(Sys.getenv("CATASTROESP_CACHE_DIR"), detected)

  expect_false(file.exists(old_fname))
  expect_true(file.exists(new_fname))

  # OK, now re-configure the cache
  if (tester_has_config_installed) {
    catr_set_cache_dir(
      getvar,
      install = TRUE,
      overwrite = TRUE,
      verbose = FALSE
    )
  } else {
    catr_set_cache_dir(getvar, install = FALSE, verbose = FALSE)
  }

  after_test <- detect_cache_dir_muted()

  expect_identical(Sys.getenv("CATASTROESP_CACHE_DIR"), getvar)
  expect_identical(after_test, getvar)
})

#' ATOM INSPIRE: Search for municipality codes
#'
#' @description
#' Search for a municipality (as a string, part of a string, or code) and get
#' the corresponding code as per the Cadastre.
#' @encoding UTF-8
#'
#' @family ATOM
#' @family search
#' @family databases
#'
#' @inheritParams catr_atom_get_parcels
#'
#' @return A [tibble][tibble::tbl_df].
#'
#' @export
#'
#' @examplesIf run_example()
#' \donttest{
#' catr_atom_search_munic("Mad")
#' }
catr_atom_search_munic <- function(
  munic,
  to = NULL,
  cache = deprecated(),
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE
) {
  if (lifecycle::is_present(cache)) {
    lifecycle::deprecate_warn(
      when = "1.0.0",
      what = "CatastRo::catr_atom_search_munic(cache)",
      details = "Results are always cached."
    )
  }

  munic <- validate_non_empty_arg(munic)
  to <- ensure_null(to)

  all <- catr_atom_get_address_db_all(
    update_cache = update_cache,
    cache_dir = cache_dir,
    verbose = FALSE
  )
  if (is.null(all)) {
    return(NULL)
  }

  if (!is.null(to)) {
    linesto <- grep(to, all$territorial_office, ignore.case = TRUE)

    # Filter by territorial office if matches found
    if (length(linesto) > 1) {
      all <- all[linesto, ]
    } else {
      if (verbose) {
        cli::cli_alert_warning(
          paste0(
            "Ignoring {.arg to} argument. No results ",
            "found with pattern {.str {munic}} in {.str {to}}."
          )
        )
      }
    }
  }

  to_loc <- ensure_null(grep(munic, all$munic, ignore.case = TRUE))

  if (is.null(to_loc)) {
    cli::cli_alert_warning(
      "No municipality found with pattern {.str {munic}} in {.str {to}}."
    )
    return(NULL)
  }

  # Compute string distances for municipality matching
  with_d <- data.frame(
    munic = all$munic,
    territorial_office = all$territorial_office,
    dist = as.vector(adist(munic, all$munic))
  )
  with_d <- with_d[to_loc, ]
  with_d <- with_d[order(with_d$dist), ]

  # Get lines and cols
  res <- with_d[, c("territorial_office", "munic")]

  # Split to get code

  res$catrcode <- vapply(
    res$munic,
    function(x) {
      unlist(strsplit(x, "-"))[1]
    },
    FUN.VALUE = character(1)
  )

  res
}

test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  cdir <- file.path(tempdir(), "testthat_ex1")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  expect_snapshot(
    fend <- catr_atom_search_munic("LABAJOS", cache_dir = cdir)
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("Test 404 all", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex2")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_atom_search_munic("MELQUE", to = "Segovia", cache_dir = cdir)
  )
  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
  unlink(cdir, recursive = TRUE, force = TRUE)
  # Otherwise work
  expect_silent(
    fend <- catr_atom_search_munic("MELQUE", to = "Segovia", cache_dir = cdir)
  )
  expect_shape(fend, dim = c(1, 3))

  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})

test_that("Test search", {
  skip_on_cran()
  skip_if_offline()
  cdir <- file.path(tempdir(), "testthat_ex2")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  a <- catr_atom_search_munic("Mad", cache_dir = cdir)
  expect_gt(nrow(a), 1)

  # Try with to
  b <- catr_atom_search_munic("Mad", to = 3, cache_dir = cdir)

  expect_gt(nrow(a), nrow(b))

  # Try with no result

  expect_snapshot(
    c <- catr_atom_search_munic("XXX", cache_dir = cdir)
  )
  expect_null(c)

  expect_message(
    d <- catr_atom_search_munic(
      "Melque",
      to = "XXX",
      verbose = TRUE,
      cache_dir = cdir
    ),
    'Ignoring `to` argument. No results found with pattern "Melque" in "XXX".'
  )

  d <- catr_atom_search_munic("Mel", to = "XXX", cache_dir = cdir)

  expect_gt(nrow(d), 5)

  expect_snapshot(
    ff <- catr_atom_search_munic("Melilla", to = "Burgos", cache_dir = cdir)
  )

  expect_null(ff)
  unlink(cdir)
})

test_that("Deprecations", {
  skip_on_cran()
  skip_if_offline()
  cdir <- file.path(tempdir(), "testthat_ex2")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  expect_snapshot(
    a <- catr_atom_search_munic("Mad", cache_dir = cdir, cache = TRUE)
  )
  unlink(cdir, recursive = TRUE, force = TRUE)
})

#' Internal function to read ATOM feed
#'
#' @param file Path to ATOM feed file.
#' @param top Logical. Extract top-level entries?
#' @param encoding Character string. File encoding. Defaults to "UTF-8".
#'
#' @return A [tibble][tibble::tbl_df] with ATOM feed entries.
#'
#' @noRd
catr_read_atom <- function(file, top = TRUE, encoding = "UTF-8") {
  # Encoding error sometimes, thanks @dr_xeo
  feed <- try(
    xml2::as_list(xml2::read_xml(
      file,
      options = "NOCDATA",
      encoding = encoding
    )),
    silent = TRUE
  )

  # On error try without encoding
  if (inherits(feed, "try-error")) {
    feed <- xml2::as_list(xml2::read_xml(file, options = "NOCDATA"))
  }

  # Prepare data
  feed <- feed$feed
  feed <- feed[names(feed) == "entry"]

  # Convert to tibble
  if (top) {
    tbl_all <- lapply(feed, function(x) {
      title <- unlist(x$title)
      url <- unlist(attr(x$link, "href"))
      date <- as.POSIXct(unlist(feed[1]$entry$updated))
      value <- unlist(x$content$div$div)

      # Clean
      value <- trimws(gsub("\\n|\\t", "", value))
      value <- value[grepl("^[0-9]", value)]

      tbl <- tibble::tibble(
        title = trimws(title),
        url = trimws(url),
        value = trimws(value),
        date = date
      )

      tbl
    })
  } else {
    tbl_all <- lapply(feed, function(x) {
      title <- unlist(x$title)
      url <- unlist(attr(x$link, "href"))
      date <- as.POSIXct(unlist(feed[1]$entry$updated))

      tbl <- tibble::tibble(
        title = trimws(title),
        url = trimws(url),
        date = date
      )

      tbl
    })
  }

  tbl_all <- dplyr::bind_rows(tbl_all)

  tbl_all
}

#' OVCCoordenadas: Reverse geocode a cadastral reference
#'
#' @description
#' Implementation of the OVCCoordenadas service
#' [Consulta RCCOOR](`r ovcurl("RCCOOR")`). Returns the cadastral
#' reference found for a set of specific coordinates.
#'
#' @encoding UTF-8
#' @family OVCCoordenadas
#' @family cadastral references
#' @inheritParams catr_ovc_get_rccoor_distancia
#' @export
#' @inherit catr_ovc_get_rccoor_distancia return
#'
#' @references
#' [Consulta RCCOOR](`r ovcurl("RCCOOR")`).
#'
#' @seealso [catr_srs_values], `vignette("ovcservice", package = "CatastRo")`
#' @inheritParams catr_ovc_get_rccoor_distancia
#'
#' @details
#' When the API does not provide any result, the function returns a
#' [tibble][tibble::tbl_df] with the input arguments only.
#'
#' On a successful query, the function returns a [tibble][tibble::tbl_df] with
#' one row by cadastral reference, including the following columns:
#' - `geo.xcen`, `geo.ycen`, `geo.srs`: Input arguments of the query.
#' - `refcat`: Cadastral Reference.
#' - `address`: Address as recorded in the Cadastre.
#' - Rest of fields: Check the API Docs.
#'
#' @examplesIf run_example()
#' \donttest{
#' catr_ovc_get_rccoor(
#'   lat = 38.6196566583596,
#'   lon = -3.45624183836806,
#'   srs = 4326
#' )
#' }
catr_ovc_get_rccoor <- function(lat, lon, srs = 4326, verbose = FALSE) {
  # Sanity checks
  lat <- validate_non_empty_arg(lat)
  lon <- validate_non_empty_arg(lon)

  valid_srs <- CatastRo::catr_srs_values
  valid <- as.character(valid_srs[valid_srs$ovc_service, ]$SRS)

  srs <- match_arg_pretty(srs, valid)
  srs <- paste0("EPSG:", srs)

  # Prepare query
  ##  Build url
  api_entry <- paste0(
    "http://ovc.catastro.meh.es/ovcservweb/",
    "OVCSWLocalizacionRC/OVCCoordenadas.asmx/Consulta_RCCOOR?",
    "SRS=&Coordenada_X=&Coordenada_Y="
  )

  api_entry <- httr2::url_modify_query(
    api_entry,
    SRS = srs,
    Coordenada_X = lon,
    Coordenada_Y = lat
  )

  # Extract results
  resp <- get_request_body(api_entry, verbose = verbose)

  if (is.null(resp)) {
    return(NULL)
  }

  content_list <- xml2::as_list(httr2::resp_body_xml(resp))

  # Check API custom error
  err <- content_list[["consulta_coordenadas"]]

  if (("lerr" %in% names(err))) {
    df <- tibble::as_tibble_row(unlist(err["lerr"]))

    cli::cli_alert_danger(
      paste0("Error code: ", df[1, 1], ". ", df[1, 2])
    )

    empty <- tibble::tibble(
      a = lat,
      b = lon,
      srs = srs
    )

    names(empty) <- c("geo.xcen", "geo.ycen", "geo.srs")
    return(empty)
  }

  res <- content_list[["consulta_coordenadas"]][["coordenadas"]][["coord"]]

  # Get info of the query
  overall <- tibble::as_tibble_row(unlist(res))

  # Extract helper info
  rc_help <- tibble::tibble(
    refcat = paste0(overall$pc.pc1, overall$pc.pc2),
    address = overall$ldt
  )

  # Join all

  out <- dplyr::bind_cols(
    rc_help,
    overall
  )

  # Numeric
  out["geo.xcen"] <- as.numeric(out[["geo.xcen"]])
  out["geo.ycen"] <- as.numeric(out[["geo.ycen"]])

  out
}

test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  expect_snapshot(
    fend <- catr_ovc_get_rccoor(
      lat = 40.963200,
      lon = -5.671420,
      srs = 4326
    )
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
})

test_that("Test 404 all", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_ovc_get_rccoor(
      lat = 40.963200,
      lon = -5.671420,
      srs = 4326
    )
  )
  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
})

test_that("Expect error on bad SRS", {
  skip_on_cran()
  skip_if_offline()

  expect_error(catr_ovc_get_rccoor(
    lat = 40.963200,
    lon = -5.671420,
    "abcd"
  ))
})

test_that("return data.frame given SRS", {
  skip_on_cran()
  skip_if_offline()

  result <- catr_ovc_get_rccoor(
    lat = 38.6196566583596,
    lon = -3.45624183836806,
    srs = "4230"
  )
  expect_s3_class(result, "tbl")
  expect_true(is.numeric(result$geo.xcen))
  expect_true(is.numeric(result$geo.ycen))
})

test_that("return data.frame without SRS", {
  skip_on_cran()
  skip_if_offline()

  result <- catr_ovc_get_rccoor(lat = 38.6196566583596, lon = -3.45624183836806)
  expect_s3_class(result, "tbl")
})

test_that("check fields without SRS", {
  skip_on_cran()
  skip_if_offline()

  result <- catr_ovc_get_rccoor(lat = 38.6196566583596, lon = -3.45624183836806)
  expect_true((is.character(result$address) & is.character(result$refcat)))
})

test_that("check fields given SRS", {
  skip_on_cran()
  skip_if_offline()

  result <- catr_ovc_get_rccoor(
    lat = 38.6196566583596,
    lon = -3.45624183836806,
    srs = "4230"
  )
  expect_true((is.character(result$address) & is.character(result$refcat)))
})

test_that("if data is know return NA", {
  skip_on_cran()
  skip_if_offline()

  result <- catr_ovc_get_rccoor(lat = 99999999, lon = -999999999)
  expect_true(ncol(result) == 3)
})

test_that("unprecised coordinates", {
  skip_on_cran()
  skip_if_offline()

  result <- catr_ovc_get_rccoor(
    lat = 40.963200,
    lon = -5.671420,
    srs = "4326"
  )
  expect_true(ncol(result) == 3)
})

test_that("Verbose", {
  skip_on_cran()
  skip_if_offline()

  expect_message(catr_ovc_get_rccoor(
    lat = 40.963200,
    lon = -5.671420,
    srs = "4326",
    verbose = TRUE
  ))
})

#' Internal function to download and cache a file from a URL
#'
#' @param url Character string. The URL to download.
#' @param name Character string. The name of the file to save.
#' @param cache_dir Character string. The base cache directory.
#' @param subdir Character string. Subdirectory inside the cache directory.
#' @param update_cache Logical. Whether to update the cached file.
#' @param verbose Logical. Whether to print messages.
#'
#' @return Character string. Path of the downloaded file.
#' @encoding UTF-8
#'
#' @noRd
download_url <- function(
  url = NULL,
  name = basename(url),
  cache_dir = NULL,
  subdir = "fixme",
  update_cache = FALSE,
  verbose = TRUE
) {
  cache_dir <- create_cache_dir(cache_dir)
  cache_dir <- create_cache_dir(file.path(cache_dir, subdir))

  # Create destfile and clean
  file_local <- file.path(cache_dir, name)
  file_local <- gsub("//", "/", file_local, fixed = TRUE)

  msg <- paste0("Cache dir is {.path ", cache_dir, "}.")
  make_msg("info", verbose, msg)

  # Check if file already exists
  fileoncache <- file.exists(file_local)

  # If already cached return
  if (isFALSE(update_cache) && fileoncache) {
    msg <- paste0("File already cached: {.file ", file_local, "}.")
    make_msg("success", verbose, msg)

    return(file_local)
  }

  if (fileoncache) {
    make_msg("warning", verbose, "Updating cached file")
  }

  msg <- paste0("Downloading {.url ", url, "}.")
  make_msg("info", verbose, msg)

  req <- httr2::request(url)
  req <- httr2::req_error(req, is_error = function(x) {
    FALSE
  })

  req <- httr2::req_options(
    req,
    ssl_verifypeer = getOption("catastro_ssl_verify", 1L)
  )

  req <- httr2::req_timeout(req, getOption("catastro_timeout", 300))
  req <- httr2::req_retry(req, max_tries = 3)
  if (verbose) {
    req <- httr2::req_progress(req)
  }

  if (!is_online_fun()) {
    cli::cli_alert_danger("Offline")
    cli::cli_alert("Returning {.val NULL}")
    return(NULL)
  }

  # Response

  # Check before the size to see if we need to inform with HEAD
  get_header <- httr2::req_method(req, "HEAD")
  getsize <- httr2::req_perform(get_header)

  size_dwn <- as.numeric(httr2::resp_header(getsize, "content-length", 0))
  class(size_dwn) <- class(object.size("a"))
  thr <- 20 * (1024^2)
  if (size_dwn > thr) {
    sz_dwn <- paste0(format(size_dwn, units = "auto"), ".")
    make_msg("warning", TRUE, "The file to be downloaded has size", sz_dwn)
    req <- httr2::req_progress(req)
  }

  # Testing
  test_offline <- is_404()
  if (test_offline) {
    # Modify to redirect to fake url
    req <- httr2::req_url(
      req,
      "http://ovc.catastro.meh.es/urlnoexist/fake"
    )
    file_local <- tempfile(fileext = ".txt")
  }

  resp <- httr2::req_perform(req, path = file_local)

  if (httr2::resp_is_error(resp)) {
    unlink(file_local, force = TRUE)
    get_status_code <- httr2::resp_status(resp) # nolint
    get_status_desc <- httr2::resp_status_desc(resp) # nolint

    cli::cli_alert_danger(
      c(
        "{.strong Error {get_status_code}} ({get_status_desc}):",
        " {.url {url}}."
      )
    )
    cli::cli_alert_warning(
      c(
        "If you think this is a bug please consider opening an issue on ",
        "{.url https://github.com/ropenspain/CatastRo/issues}"
      )
    )
    cli::cli_alert("Returning {.val NULL}")
    return(NULL)
  }
  msg <- paste0("Download successful on {.file ", file_local, "}.")
  make_msg("success", verbose, msg)

  file_local
}


#' Internal function to get the response body from a URL
#'
#' @param url Character string. The URL to download.
#' @param verbose Logical. Whether to print messages.
#'
#' @return httr2 response object.
#'
#' @noRd
get_request_body <- function(
  url,
  verbose = TRUE
) {
  msg <- paste0("GET {.url ", url, "}.")
  make_msg("info", verbose, msg)

  req <- httr2::request(url)
  req <- httr2::req_error(req, is_error = function(x) {
    FALSE
  })

  req <- httr2::req_options(
    req,
    ssl_verifypeer = getOption("catastro_ssl_verify", 1L)
  )

  req <- httr2::req_timeout(req, getOption("catastro_timeout", 300))
  req <- httr2::req_retry(req, max_tries = 3)
  if (verbose) {
    req <- httr2::req_progress(req)
  }

  if (!is_online_fun()) {
    cli::cli_alert_danger("Offline")
    cli::cli_alert("Returning {.val NULL}")
    return(NULL)
  }

  # Testing
  test_offline <- is_404()
  if (test_offline) {
    # Modify to redirect to fake url
    req <- httr2::req_url(
      req,
      "http://ovc.catastro.meh.es/urlnoexist/fake"
    )
  }

  resp <- httr2::req_perform(req)

  if (httr2::resp_is_error(resp)) {
    get_status_code <- httr2::resp_status(resp) # nolint
    get_status_desc <- httr2::resp_status_desc(resp) # nolint

    cli::cli_alert_danger(
      c(
        "{.strong Error {get_status_code}} ({get_status_desc}):",
        " {.url {url}}."
      )
    )
    cli::cli_alert_warning(
      c(
        "If you think this is a bug please consider opening an issue on ",
        "{.url https://github.com/ropenspain/CatastRo/issues}"
      )
    )
    cli::cli_alert("Returning {.val NULL}")
    return(NULL)
  }

  make_msg("success", verbose, "Success")
  resp
}

#' Wrapper is_online for testing
#' @noRd
is_online_fun <- function(...) {
  httr2::is_online()
}

#' Wrapper is_404 for testing
#' @noRd
is_404 <- function(...) {
  FALSE
}

test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()
  skip_if_not_installed("mapSpain")

  cdir <- file.path(tempdir(), "testthat_ex")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  expect_snapshot(
    fend <- catr_get_code_from_coords(
      c(-16.25462, 28.46824),
      srs = 4326,
      cache_dir = cdir
    )
  )
  expect_null(fend)

  expect_snapshot(
    fend <- catr_get_code_from_coords(
      mapSpain::esp_get_prov_siane("Caceres"),
      cache_dir = cdir
    )
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("Test 404 all", {
  skip_on_cran()
  skip_if_offline()
  skip_if_not_installed("mapSpain")

  cdir <- file.path(tempdir(), "testthat_ex")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_get_code_from_coords(
      c(-16.25462, 28.46824),
      srs = 4326,
      cache_dir = cdir
    )
  )
  expect_null(fend)

  expect_snapshot(
    fend <- catr_get_code_from_coords(
      mapSpain::esp_get_prov_siane("Caceres"),
      cache_dir = cdir
    )
  )
  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("Test 404 mapSpain", {
  skip_on_cran()
  skip_if_offline()
  skip_if_not_installed("mapSpain")

  cdir <- file.path(tempdir(), "testthat_ex")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  local_mocked_bindings(
    esp_get_munic_siane = function(...) {
      cli::cli_alert_info("Mocking mapSpain")
      NULL
    },
    .package = "mapSpain"
  )

  expect_snapshot(
    fend <- catr_get_code_from_coords(
      c(-16.25462, 28.46824),
      srs = 4326,
      cache_dir = cdir
    )
  )
  expect_null(fend)

  unlink(cdir, recursive = TRUE, force = TRUE)
})
test_that("Check", {
  skip_on_cran()
  skip_if_offline()
  cdir <- file.path(tempdir(), "testthat_excoord")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  # Try with coords

  expect_error(catr_get_code_from_coords(c(0, 0)))
  expect_error(catr_get_code_from_coords(c(0, 0, 0)))
  expect_message(
    catr_get_code_from_coords(c(0, 0), srs = 4326)
  )
  expect_s3_class(
    catr_get_code_from_coords(c(-16.25462, 28.46824), srs = 4326),
    "tbl"
  )

  # Try with sf
  m <- mapSpain::esp_get_ccaa_siane(
    ccaa = c("Asturias", "La Rioja"),
    cache_dir = cdir
  )

  expect_message(catr_get_code_from_coords(m, cache_dir = cdir))
  expect_silent(catr_get_code_from_coords(m[1, ]))

  # Try polis
  m2 <- mapSpain::esp_get_ccaa_siane(
    "Murcia",
    cache_dir = cdir
  )
  s3 <- catr_get_code_from_coords(m2, cache_dir = cdir)

  expect_s3_class(s3, "tbl")

  # Try with sfc
  sfc <- sf::st_geometry(m2)

  expect_s3_class(sfc, "sfc")
  s4 <- catr_get_code_from_coords(sfc, cache_dir = cdir)
  expect_s3_class(s4, "tbl")
  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  expect_snapshot(
    fend <- catr_ovc_get_cpmrc("9872023VH5797S")
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
})

test_that("Test 404 all", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_ovc_get_cpmrc("9872023VH5797S")
  )
  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
})

test_that("Expect error on bad SRS", {
  skip_on_cran()
  skip_if_offline()
  expect_error(
    catr_ovc_get_cpmrc(
      rc = "s",
      srs = "abcd"
    )
  )
})

test_that("giving all the arguments", {
  skip_on_cran()
  skip_if_offline()

  result <- catr_ovc_get_cpmrc(
    "13077A01800039",
    "4230",
    "CIUDAD REAL",
    "SANTA CRUZ DE MUDELA"
  )
  expect_s3_class(result, "tbl")
  expect_true(all(c("geo.xcen", "geo.ycen") %in% names(result)))
})

test_that("giving cadastral reference and SRS", {
  skip_on_cran()
  skip_if_offline()

  result <- catr_ovc_get_cpmrc("9872023VH5797S", "4230")
  expect_true((is.numeric(result$xcoord) & is.numeric(result$ycoord)))
})

test_that("giving only the cadastral reference", {
  skip_on_cran()
  skip_if_offline()

  expect_message(catr_ovc_get_cpmrc("9872023VH5797S", verbose = TRUE))

  result <- catr_ovc_get_cpmrc(rc = "13077A01800039")
  expect_s3_class(result, "tbl")
})


test_that("given Municipio, Provincia is needed", {
  skip_on_cran()
  skip_if_offline()

  expect_message(
    nnn <- catr_ovc_get_cpmrc(
      rc = "13077A01800039",
      srs = "4230",
      municipality = "SANTA CRUZ DE MUDELA"
    )
  )

  expect_true(ncol(nnn) == 2)
})

test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  expect_snapshot(
    fend <- catr_ovc_get_cod_provinces()
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
})

test_that("Test 404 all", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_ovc_get_cod_provinces()
  )
  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
})

test_that("Callejero provinces", {
  skip_on_cran()
  skip_if_offline()

  s <- catr_ovc_get_cod_provinces()
  expect_s3_class(s, "tbl")

  expect_message(catr_ovc_get_cod_provinces(verbose = TRUE))
})

test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  cdir <- file.path(tempdir(), "testthat_ex1")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  expect_snapshot(
    fend <- catr_atom_get_buildings("LABAJOS", cache_dir = cdir)
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("Test 404 all", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex2")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_atom_get_buildings("MELQUE", to = "Segovia", cache_dir = cdir)
  )
  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
  unlink(cdir, recursive = TRUE, force = TRUE)
  # Otherwise work
  expect_silent(
    fend <- catr_atom_get_buildings("MELQUE", to = "Segovia", cache_dir = cdir)
  )
  expect_gt(nrow(fend), 20)

  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})
test_that("ATOM Buildings", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "test_bu")
  unlink(cdir, force = TRUE, recursive = TRUE)
  expect_snapshot(catr_atom_get_buildings("xyxghx", cache_dir = cdir))

  expect_message(
    s <- catr_atom_get_buildings(
      "Nava",
      to = "Segovia",
      verbose = TRUE,
      cache_dir = cdir
    )
  )

  # Deprecations
  expect_snapshot(
    s <- catr_atom_get_buildings(
      "Melque",
      to = "Segovia",
      cache = FALSE,
      cache_dir = cdir
    )
  )

  expect_s3_class(s, "sf")
  expect_message(
    catr_atom_get_buildings(
      "Melque",
      to = "XXX",
      verbose = TRUE,
      cache_dir = cdir
    ),
    'Ignoring `to` argument. No results found with pattern "Melque" in "XXX".'
  )
  expect_s3_class(s, "sf")

  # Check other options
  me_bu <- catr_atom_get_buildings(
    "Melque",
    to = "Segovia",
    cache_dir = cdir
  )

  me_bupart <- catr_atom_get_buildings(
    "Melque",
    to = "Segovia",
    what = "buildingpart",
    cache_dir = cdir
  )

  me_other <- catr_atom_get_buildings(
    "Melque",
    to = "Segovia",
    what = "other",
    cache_dir = cdir
  )

  expect_s3_class(me_bu, "sf")
  expect_s3_class(me_bupart, "sf")
  expect_s3_class(me_other, "sf")

  expect_gt(nrow(me_bupart), nrow(me_bu))
  expect_gt(nrow(me_bu), nrow(me_other))

  unlink(cdir, force = TRUE, recursive = TRUE)
})

test_that("ATOM Encoding issue", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "test_bu2")
  unlink(cdir, force = TRUE, recursive = TRUE)

  expect_silent(catr_atom_get_buildings("23078", cache_dir = cdir))
  expect_silent(catr_atom_get_buildings("03050", cache_dir = cdir))
  expect_silent(catr_atom_get_buildings("23051", cache_dir = cdir))
  unlink(cdir, force = TRUE, recursive = TRUE)
})

test_that("Test 404 single", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex2to2bu")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  all <- catr_atom_get_buildings_db_all(cache_dir = cdir)
  all <- catr_atom_get_buildings_db_to("Segovia", cache_dir = cdir)

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_atom_get_buildings(
      "Melque",
      to = "Segovia",
      cache_dir = cdir
    )
  )
  expect_null(fend)
  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("SSL verifier (#40)", {
  skip_on_cran()
  skip_if_offline()
  skip_if_not_installed("withr")

  # This is a check to see if setup.R work. Expected to fail if not
  # called with test_local(), etc.
  expect_equal(getOption("catastro_ssl_verify", 1L), 0L)
})

test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()
  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/",
    "Addresses/ES.SDGC.AD.atom.xml"
  )
  cdir <- file.path(tempdir(), "testthat_ex3")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  expect_snapshot(
    fend <- download_url(
      url,
      cache_dir = cdir,
      subdir = "fixme",
      update_cache = FALSE,
      verbose = FALSE
    )
  )
  expect_null(fend)
  expect_length(list.files(cdir, recursive = TRUE), 0)
  unlink(cdir, recursive = TRUE, force = TRUE)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
})

test_that("Test 404", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/",
    "CadastralParcels/ES.SDGC.CP.atom.xml"
  )

  expect_message(
    s <- download_url(
      url,
      verbose = FALSE,
      cache_dir = cdir,
      update_cache = TRUE
    ),
    "Error "
  )
  expect_null(s)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })

  # Otherwise work
  expect_silent(
    s <- download_url(
      url,
      verbose = FALSE,
      cache_dir = cdir,
      update_cache = TRUE
    )
  )
  expect_length(s, 1)
  expect_true(is.character(s))
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})

test_that("Caching tests", {
  skip_on_cran()
  skip_if_offline()

  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/",
    "Addresses/ES.SDGC.AD.atom.xml"
  )

  cdir <- file.path(tempdir(), "testthat_ex4")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  expect_message(
    fend <- download_url(
      url,
      cache_dir = cdir,
      subdir = "fixme",
      update_cache = FALSE,
      verbose = TRUE
    ),
    "Cache dir is"
  )

  expect_length(list.files(cdir, recursive = TRUE), 1)

  expect_message(
    fend <- download_url(
      url,
      cache_dir = cdir,
      subdir = "fixme",
      update_cache = FALSE,
      verbose = TRUE
    ),
    "File already"
  )

  expect_message(
    fend <- download_url(
      url,
      cache_dir = cdir,
      subdir = "fixme",
      update_cache = TRUE,
      verbose = TRUE
    ),
    "Updating cached"
  )

  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("Caching errors", {
  skip_on_cran()
  skip_if_offline()

  url <- "http://ropenspain.github.io/CatastRo/noexist-this-file.txt"
  cdir <- file.path(tempdir(), "testthat_ex5")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  expect_message(
    fend <- download_url(
      url,
      cache_dir = cdir,
      subdir = "fixme",
      update_cache = FALSE,
      verbose = TRUE
    ),
    "Error"
  )

  expect_null(fend)

  # Warn if size of download is huge

  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/Buildings/46",
    "/46900-VALENCIA/A.ES.SDGC.BU.46900.zip"
  )

  expect_message(
    download_url(
      url,
      cache_dir = cdir,
      subdir = "fixme",
      update_cache = FALSE,
      verbose = FALSE
    ),
    "The file to be downloaded has size"
  )

  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("No connection body", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/",
    "buildings/ES.SDGC.BU.atom.xml"
  )

  expect_snapshot(
    fend <- get_request_body(url, verbose = FALSE)
  )
  expect_null(fend)
  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
})

test_that("Error body", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })
  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/",
    "Addresses/ES.SDGC.AD.atom.xml"
  )

  expect_snapshot(
    fend <- get_request_body(url, verbose = FALSE)
  )
  expect_null(fend)
  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
})


test_that("Tests body", {
  skip_on_cran()
  skip_if_offline()

  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/",
    "Addresses/ES.SDGC.AD.atom.xml"
  )

  expect_message(
    fend <- get_request_body(url, verbose = TRUE),
    "GET"
  )

  expect_s3_class(fend, "httr2_response")

  url <- "http://ropenspain.github.io/CatastRo/noexist-this-file.txt"

  expect_message(
    fend <- get_request_body(url, verbose = TRUE),
    "GET"
  )

  expect_null(fend)
})

test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  expect_snapshot(
    fend <- catr_ovc_get_cod_munic(4, 5)
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
})

test_that("Test 404 all", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_ovc_get_cod_munic(4, 5)
  )
  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
})

test_that("Callejero munic", {
  skip_on_cran()
  skip_if_offline()

  expect_snapshot(error = TRUE, catr_ovc_get_cod_munic(2))
  s <- catr_ovc_get_cod_munic(5, 900)
  expect_gt(ncol(s), 1)

  # With INE code
  s2 <- catr_ovc_get_cod_munic(5, cmun_ine = s$cmun)
  expect_identical(s, s2)

  expect_message(catr_ovc_get_cod_munic(5, 1304))

  nil <- catr_ovc_get_cod_munic(5, 1304)

  expect_true(ncol(nil) == 1)
  expect_s3_class(nil, "tbl")
  expect_true(is.na(nil[1, 1]))
})

#' Create messages based on type
#'
#' @param type Character string. Type of message. Accepted values are
#'   `"generic"`, `"success"`, `"warning"`, `"danger"`, or `"info"`.
#'
#' @param verbose Logical. Whether to print messages to the console.
#' @param ... Character strings to combine into the message.
#'
#' @return
#' Invisibly returns `NULL`. Prints messages to console if `verbose` is
#' `TRUE`.
#' @encoding UTF-8
#'
#' @noRd
make_msg <- function(type = "generic", verbose, ...) {
  if (!verbose) {
    return(invisible())
  }
  dots <- list(...)
  msg <- paste(dots, collapse = " ")

  if (type == "generic") {
    cli::cli_alert(msg)
  }
  if (type == "success") {
    cli::cli_alert_success(msg)
  }
  if (type == "warning") {
    cli::cli_alert_warning(msg)
  }
  if (type == "danger") {
    cli::cli_alert_danger(msg)
  }
  if (type == "info") {
    cli::cli_alert_info(msg)
  }
  invisible()
}

#' Match argument with pretty error message
#'
#' @param arg Argument to match.
#' @param choices Possible choices for the argument.
#'
#' @return
#' The matched argument.
#'
#' @noRd
match_arg_pretty <- function(arg, choices) {
  arg_name <- as.character(substitute(arg)) # nolint

  if (missing(choices)) {
    formal_args <- formals(sys.function(sys_par <- sys.parent()))
    choices <- eval(
      formal_args[[as.character(substitute(arg))]],
      envir = sys.frame(sys_par)
    )
  }
  choices <- as.character(choices)

  if (is.null(arg)) {
    return(choices[1L])
  }

  arg <- as.character(arg)

  if (identical(arg, choices)) {
    return(arg[1])
  }

  lmatch <- match(arg, choices)
  # Hint
  aproxmatch <- pmatch(arg, choices)[1]

  if (length(arg) > 1 || is.na(lmatch)) {
    # Create error message
    if (length(choices) == 1) {
      msg <- paste0("{.str ", choices, "}")
    } else {
      l_choices <- length(choices)
      msg <- paste0("{.str ", choices[-l_choices], "}", collapse = ", ")
      msg <- paste0(msg, " or {.str ", choices[l_choices], "}")
      # Add one of at the begining
      msg <- paste0("one of ", msg)
    }

    msg <- paste0(msg, ", not ")
    bad_arg <- paste0("{.str ", arg, "}", collapse = " or ")
    msg <- paste0(msg, bad_arg, ".")

    # Maybe is a regex?
    reg_msg <- NULL
    if (!is.na(aproxmatch)) {
      aprox <- choices[aproxmatch]
      aprox_val <- paste0("{.str ", aprox, "}", collapse = " or ")
      reg_msg <- paste0("Did you mean ", aprox_val, "?")
    }

    cli::cli_abort(
      c(
        paste0("{.arg {arg_name}} should be ", msg),
        "i" = reg_msg
      ),
      call = NULL
    )
  }

  choices[lmatch]
}

ensure_null <- function(x) {
  x_init <- x
  x <- as.vector(x)
  x[is.null(x)] <- NA
  x[is.na(x)] <- NA
  x[nchar(as.character(x)) == 0] <- NA
  if (all(is.na(x))) {
    return(NULL)
  }

  x_init
}

validate_non_empty_arg <- function(arg, call = parent.frame(1)) {
  arg_name <- as.character(substitute(arg)) # nolint

  if (missing(arg)) {
    cli::cli_abort("{.arg {arg_name}} can't be missing.", call = call)
  }

  arg
}

test_that("Messages", {
  skip_on_cran()
  expect_silent(make_msg(verbose = FALSE))
  expect_snapshot(make_msg(
    "generic",
    TRUE,
    "Hi",
    "I am a generic.",
    "See {.var avar}."
  ))
  expect_snapshot(make_msg("info", TRUE, "Info here.", "See {.pkg igoR}."))

  expect_snapshot(make_msg(
    "warning",
    TRUE,
    "Caution! A warning.",
    "But still OK."
  ))

  expect_snapshot(make_msg("danger", TRUE, "OOPS!", "I did it again :("))

  expect_snapshot(make_msg("success", TRUE, "Hooray!", "5/5 ;)"))
})

test_that("Pretty match", {
  skip_on_cran()
  my_fun <- function(
    arg_one = c(10, 1000, 3000, 5000)
  ) {
    match_arg_pretty(arg_one)
  }

  # OK, returns character
  expect_identical(my_fun(1000), "1000")
  expect_identical(my_fun("1000"), "1000")
  expect_identical(my_fun(NULL), "10")
  expect_identical(my_fun(), "10")
  # Some errors here
  # Single value no match
  expect_snapshot(
    my_fun("error here"),
    error = TRUE
  )

  # Several values no match
  expect_snapshot(
    my_fun(c("an", "error")),
    error = TRUE
  )

  # One value regex
  expect_snapshot(
    my_fun("5"),
    error = TRUE
  )
  # Several value regex
  expect_snapshot(
    my_fun("00"),
    error = TRUE
  )

  my_fun2 <- function(year = 20) {
    match_arg_pretty(year)
  }

  # Pass more options than expected
  expect_snapshot(
    my_fun2(c(1, 2)),
    error = TRUE
  )

  # With custom options
  my_fun3 <- function(an_arg = 20) {
    match_arg_pretty(an_arg, c("30", "20"))
  }
  expect_identical(my_fun3(), "20")
  expect_snapshot(my_fun3("3"), error = TRUE)
  # Pass more options than expected
  expect_snapshot(
    my_fun2(c(1, 2)),
    error = TRUE
  )
})

test_that("Ensure NULL", {
  expect_null(ensure_null(NULL))
  expect_null(ensure_null(c(NULL, NA)))
  expect_null(ensure_null(c(NULL, NA, "")))
  expect_null(ensure_null(c("", character(0))))
  expect_identical(ensure_null(c(1, 2)), c(1, 2))
  expect_identical(letters, letters)
})

test_that("Not empty", {
  a_fun <- function(a, b) {
    a <- validate_non_empty_arg(a)
    b <- validate_non_empty_arg(b)
    c(a, b)
  }

  expect_snapshot(error = TRUE, a_fun())
  expect_snapshot(error = TRUE, a_fun(a = 1))
  expect_identical(a_fun(a = 1, b = 1), c(1, 1))
})

#' OVCCallejero: Extract a list of provinces with their codes
#'
#' @description
#' Implementation of the OVCCallejero service
#' [ConsultaProvincia](`r ovcurl("prov")`). Returns a list of provinces
#' included in the Spanish Cadastre.
#'
#' @encoding UTF-8
#' @family OVCCallejero
#' @family search
#' @inheritParams catr_ovc_get_cpmrc
#' @export
#'
#' @references
#' [ConsultaProvincia](`r ovcurl("prov")`).
#'
#' @return A [tibble][tibble::tbl_df].
#'
#' @examplesIf run_example()
#' \donttest{
#' catr_ovc_get_cod_provinces()
#' }
#'
catr_ovc_get_cod_provinces <- function(verbose = FALSE) {
  # Prepare query
  ##  Build url
  api_entry <- paste0(
    "http://ovc.catastro.meh.es/ovcservweb/",
    "/ovcswlocalizacionrc/ovccallejerocodigos.asmx/ConsultaProvincia?"
  )

  # Extract results
  resp <- get_request_body(api_entry, verbose = verbose)

  if (is.null(resp)) {
    return(NULL)
  }

  content_list <- xml2::as_list(httr2::resp_body_xml(resp))

  res <- content_list[["consulta_provinciero"]][["provinciero"]]

  # Get a list of tibbles
  res_tibble <- lapply(res, function(x) {
    df <- tibble::as_tibble_row(unlist(x))
    df
  })

  # Bind all
  overall <- dplyr::bind_rows(res_tibble)

  overall
}

#' WMS INSPIRE: Download map images
#'
#' @description
#' Get geotagged images from the Spanish Cadastre. This function is a
#' wrapper of [mapSpain::esp_get_tiles()].
#'
#' @encoding UTF-8
#' @family INSPIRE
#' @family WMS
#' @family spatial
#' @export
#'
#' @inheritParams catr_wfs_get_address_bbox
#' @inheritParams catr_atom_get_address_db_all
#' @inheritParams mapSpain::esp_get_tiles
#' @inheritDotParams mapSpain::esp_get_tiles res:mask
#'
#' @param what,styles Layer and style of the WMS layer to be downloaded. See
#'   **Layers and styles**.
#'
#' @return
#' A [`SpatRaster`][terra::rast] is returned, with 3 (RGB) or 4 (RGBA) layers,
#' see [terra::RGB()].
#'
#' @seealso
#' [mapSpain::esp_get_tiles()] and [terra::RGB()]. For plotting see
#' [terra::plotRGB()] and [tidyterra::geom_spatraster_rgb()].
#'
#' @section Bounding box:
#' When `x` is a numeric vector, make sure that the `srs` matches the
#' coordinate values. When `x` is a [`sf`][sf::st_sf] object, the value
#' `srs` is ignored.
#'
#' The query is performed using [EPSG:3857](https://epsg.io/3857) (Web Mercator)
#' and the tile is projected back to the SRS of `x`. In case that the tile
#' looks deformed, try either providing `x` or specify the SRS of the requested
#' tile via the `srs` argument, which should match the SRS of
#' `x`. See **Examples**.
#'
#' @section Layers and styles:
#'
#' ## Layers
#' The argument `what` defines the layer to be extracted. The equivalence with
#' the
#'
#' ```{r, echo=FALSE, results='asis'}
#' cat(paste0(
#'    " [API Docs](https://www.catastro.hacienda.gob.es/",
#'      "webinspire/documentos/inspire-WMS.pdf) ")
#'      )
#'
#' ```
#' reference is:
#' - `"parcel"`: CP.CadastralParcel
#' - `"zoning"`: CP.CadastralZoning
#' - `"building"`: BU.Building
#' - `"buildingpart"`: BU.BuildingPart
#' - `"address"`: AD.Address
#' - `"admboundary"`: AU.AdministrativeBoundary
#' - `"admunit"`: AU.AdministrativeUnit
#'
#' ## Styles
#' The WMS service provides different styles on each layer (`what` argument).
#' Some of the styles available are:
#' - `"parcel"`: styles : `"BoundariesOnly"`, `"ReferencePointOnly"`,
#'   `"ELFCadastre"`.
#' - `"zoning"`: styles : `"BoundariesOnly"`, `"ELFCadastre"`.
#' - `"building"`, `"buildingpart"`: `"ELFCadastre"`
#' - `"address"`: `"Number.ELFCadastre"`
#' - `"admboundary"`, `"admunit"`: `"ELFCadastre"`
#'
#' Check the
#' ```{r, echo=FALSE, results='asis'}
#' cat(paste0(
#'    " [API Docs](https://www.catastro.hacienda.gob.es/",
#'      "webinspire/documentos/inspire-WMS.pdf) ")
#'      )
#'
#' ```
#' for more information.
#'
#' @examplesIf run_example()
#' \donttest{
#'
#' # With a bbox
#'
#' pict <- catr_wms_get_layer(
#'   c(222500, 4019500, 223700, 4020700),
#'   srs = 25830,
#'   what = "parcel"
#' )
#'
#' library(mapSpain)
#' library(ggplot2)
#' library(tidyterra)
#'
#' ggplot() +
#'   geom_spatraster_rgb(data = pict)
#'
#' # With a spatial object
#'
#' parcels <- catr_wfs_get_parcels_neigh_parcel("3662303TF3136B", srs = 25830)
#'
#' # Use styles
#'
#' parcels_img <- catr_wms_get_layer(parcels,
#'   what = "buildingpart",
#'   srs = 25830, # As parcels object
#'   bbox_expand = 0.3,
#'   styles = "ELFCadastre"
#' )
#'
#' ggplot() +
#'   geom_sf(data = parcels, fill = "blue", alpha = 0.5) +
#'   geom_spatraster_rgb(data = parcels_img)
#' }
catr_wms_get_layer <- function(
  x,
  srs = NULL,
  what = c(
    "building",
    "buildingpart",
    "parcel",
    "zoning",
    "address",
    "admboundary",
    "admunit"
  ),
  styles = "default",
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE,
  crop = FALSE,
  options = NULL,
  ...
) {
  x <- ensure_null(x)
  bbox_res <- get_sf_from_bbox(x, srs)
  cache_dir <- create_cache_dir(cache_dir)

  # Manage layer

  what <- match_arg_pretty(what)

  layer <- switch(what,
    "building" = "Catastro.Building",
    "buildingpart" = "Catastro.BuildingPart",
    "parcel" = "Catastro.CadastralParcel",
    "zoning" = "Catastro.CadastralZoning",
    "address" = "Catastro.Address",
    "admboundary" = "Catastro.AdministrativeBoundary",
    "admunit" = "Catastro.AdministrativeUnit"
  )

  # Manage styles and options
  # Custom options
  opts <- list(
    styles = styles,
    version = "1.1.0"
  )

  # Add srs
  if (!is.null(srs)) {
    if (!any(grepl("epsg", srs, ignore.case = TRUE))) {
      opts <- modifyList(
        opts,
        list(srs = paste0("EPSG:", srs))
      )
    }
  }

  # Add to options
  if (is.null(options)) {
    finalopts <- opts
  } else {
    names(options) <- tolower(names(options))
    finalopts <- modifyList(
      opts,
      options
    )
  }

  # Check if need to change crs

  if (finalopts$version >= "1.3.0") {
    newnames <- gsub("srs", "crs", names(finalopts), fixed = TRUE)
    names(finalopts) <- newnames
  }

  # Query

  out <- mapSpain::esp_get_tiles(
    x = bbox_res,
    type = layer,
    update_cache = update_cache,
    cache_dir = cache_dir,
    verbose = verbose,
    options = finalopts,
    ...
  )

  if (crop) {
    out <- terra::crop(out, bbox_res)
  }

  out
}

test_that("Check error", {
  expect_error(catr_wms_get_layer(
    c(760926, 4019259, 761155, 4019366),
    srs = 25829,
    what = "aa"
  ))
})

test_that("Check tiles", {
  skip_on_cran()
  skip_if_offline()
  cdir <- file.path(tempdir(), "testthat_ex")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  obj <- catr_wms_get_layer(
    c(760926, 4019259, 761155, 4019366),
    srs = 25829,
    cache_dir = cdir
  )

  expect_s4_class(obj, "SpatRaster")

  # test crop
  objcrop <- catr_wms_get_layer(
    c(760926, 4019259, 761155, 4019366),
    srs = 25829,
    crop = TRUE,
    cache_dir = cdir
  )

  expect_true(terra::nrow(obj) > terra::nrow(objcrop))

  # Convert to spatial object
  bbox <- get_sf_from_bbox(
    c(760926, 4019259, 761155, 4019366),
    25829
  )
  expect_s3_class(bbox, "sfc")

  obj2 <- catr_wms_get_layer(bbox, cache_dir = cdir)

  expect_s4_class(obj2, "SpatRaster")

  # With styles
  obj3 <- catr_wms_get_layer(
    c(222500, 4019500, 222700, 4019700),
    srs = 25830,
    what = "building",
    styles = "ELFCadastre",
    options = list(version = "1.3.0"),
    cache_dir = cdir
  )

  expect_s4_class(obj3, "SpatRaster")
  unlink(cdir, recursive = TRUE, force = TRUE)
})

#' OVCCallejero: Extract the code of a municipality
#'
#' @description
#' Implementation of the OVCCallejero service
#' [ConsultaMunicipioCodigos](`r ovcurl("mun")`). Returns names and codes
#' of a municipality as per the Cadastre and the INE (National Statistics
#' Institute).
#'
#' @encoding UTF-8
#' @family OVCCallejero
#' @family search
#' @export
#' @inheritParams catr_ovc_get_cpmrc
#' @inherit catr_ovc_get_cpmrc return
#'
#' @references
#' [ConsultaMunicipioCodigos](`r ovcurl("mun")`).
#'
#' @seealso
#' [mapSpain::esp_get_munic_siane()] to get shapes of municipalities, including
#' the INE code.
#'
#' @param cpro The code of a province, as provided by
#'   [catr_ovc_get_cod_provinces()].
#' @param cmun,cmun_ine Code of a municipality, as recorded on the Spanish
#'   Cadastre (`cmun`) or the National Statistics Institute. Either `cmun` or
#'   `cmun_ine` must be provided.
#'
#' @details
#' On a successful query, the function returns a [tibble][tibble::tbl_df]
#' with one row including the following columns:
#'
#' - `munic`: Name of the municipality as per the Cadastre.
#' - `catr_to`: Cadastral territorial office code.
#' - `catr_munic`: Municipality code as recorded on the Cadastre.
#' - `catrcode`: Full Cadastral code for the municipality.
#' - `cpro`: Province code as per the INE.
#' - `cmun`: Municipality code as per the INE.
#' - `inecode`: Full INE code for the municipality.
#' - Rest of fields: Check the API Docs.
#'
#' @examplesIf run_example()
#' \donttest{
#' # Get municipality by cadastral code
#' ab <- catr_ovc_get_cod_munic(cpro = 2, cmun = 900)
#'
#' ab
#'
#' # Same query using the INE code
#'
#' ab2 <- catr_ovc_get_cod_munic(cpro = 2, cmun_ine = 3)
#'
#' ab2
#' }
#'
catr_ovc_get_cod_munic <- function(
  cpro,
  cmun = NULL,
  cmun_ine = NULL,
  verbose = FALSE
) {
  cpro <- validate_non_empty_arg(cpro)
  cmun <- ensure_null(cmun)
  cmun_ine <- ensure_null(cmun_ine)

  munis <- c(cmun, cmun_ine)

  if (is.null(munis)) {
    my_arg <- c("cmun", "cmun_ine") # nolint
    cli::cli_abort(
      "Please provide a non-{.val NULL} value either on {.or {.arg {my_arg}}}."
    )
  }

  # Prepare query
  ##  Build url
  api_entry <- paste0(
    "http://ovc.catastro.meh.es/ovcservweb/",
    "/ovcswlocalizacionrc/ovccallejerocodigos.asmx/ConsultaMunicipioCodigos?",
    "/CodigoProvincia=&CodigoMunicipio=&CodigoMunicipioIne="
  )

  api_entry <- httr2::url_modify_query(
    api_entry,
    CodigoProvincia = cpro,
    CodigoMunicipio = ifelse(is.null(cmun), "", cmun),
    CodigoMunicipioIne = ifelse(is.null(cmun_ine), "", cmun_ine)
  )

  # Extract results
  resp <- get_request_body(api_entry, verbose = verbose)

  if (is.null(resp)) {
    return(NULL)
  }

  content_list <- xml2::as_list(httr2::resp_body_xml(resp))

  # Check API custom error
  err <- content_list[[1]]

  if (("lerr" %in% names(err))) {
    df <- tibble::as_tibble_row(unlist(err["lerr"]))

    cli::cli_alert_danger(
      paste0("Error code: ", df[1, 1], ". ", df[1, 2])
    )

    empty <- tibble::tibble(
      name = NA
    )

    return(empty)
  }

  res <- content_list[[1]][["municipiero"]]

  df <- tibble::as_tibble_row(unlist(res))

  # Fix names
  newnames <- vapply(
    names(df),
    function(x) {
      rev(unlist(strsplit(x, ".", fixed = TRUE)))[1]
    },
    FUN.VALUE = character(1)
  )

  names(df) <- newnames

  # Create friendly codes
  catcodes <- tibble::tibble(
    munic = df$nm,
    catr_to = sprintf("%02d", as.integer(df$cd)),
    catr_munic = sprintf("%03d", as.integer(df$cmc))
  )

  catcodes$catrcode <- paste0(catcodes$catr_to, catcodes$catr_munic)
  inecodes <- tibble::tibble(
    cpro = sprintf("%02d", as.integer(df$cp)),
    cmun = sprintf("%03d", as.integer(df$cm))
  )

  inecodes$inecode <- paste0(inecodes$cpro, inecodes$cmun)

  overall <- dplyr::bind_cols(
    catcodes,
    inecodes,
    df
  )

  overall
}

#' WFS INSPIRE: Download addresses
#'
#' @description
#' Get the spatial data of addresses. The WFS Service allows performing
#' several types of queries:
#' - By bounding box: Implemented on `catr_wfs_get_address_bbox()`.
#'   Extract objects included in the bounding box provided. See
#'   **Bounding box**.
#'
#' @encoding UTF-8
#' @family INSPIRE
#' @family WFS
#' @family addresses
#' @family spatial
#' @export
#'
#' @rdname catr_wfs_get_address
#' @inheritParams catr_set_cache_dir
#'
#' @references
#' ```{r, echo=FALSE, comment="", results="asis"}
#' paste0("[API Documentation](https://www.catastro.hacienda.gob.es/",
#'        "webinspire/documentos/inspire-ad-WFS.pdf).") |>
#'   cat()
#' cat("\n\n")
#' paste0("[INSPIRE Services for Cadastral Cartography](https://www.",
#'        "catastro.hacienda.gob.es/webinspire/index.html).") |>
#'   cat()
#'
#' ```
#'
#' @return A [`sf`][sf::st_sf] object.
#'
#' @param x See **Bounding box**. It may be:
#'   - A numeric vector of length 4 with the coordinates that defines
#'     the bounding box: `c(xmin, ymin, xmax, ymax)`
#'   - A `sf/sfc` object, as provided by the \CRANpkg{sf} package.
#' @param srs SRS/CRS to use on the query. To see allowed values, use
#'   [catr_srs_values], specifically the `wfs_service` column. See
#'   **Bounding box**.
#' @param rc The cadastral reference to be extracted.
#'
#' @section API Limits:
#' The API service is limited to a bounding box of 4km2 and a maximum of 5,000
#' elements.
#'
#' @section Bounding box:
#' When `x` is a numeric vector, make sure that the `srs` matches the
#' coordinate values. Additionally, the function queries the bounding box on
#' [EPSG:25830](https://epsg.io/25830) - ETRS89 / UTM zone 30N, to overcome
#' a potential bug on the API side.
#'
#' When `x` is a [`sf`][sf::st_sf] object, the value `srs` is ignored. In
#' this case, the bounding box of the [`sf`][sf::st_sf] object is
#' used for the query (see [sf::st_bbox()]).
#'
#' The result is always provided in the SRS of the [`sf`][sf::st_sf] object
#' provided as input.
catr_wfs_get_address_bbox <- function(x, srs = NULL, verbose = FALSE) {
  # Sanity checks
  x <- validate_non_empty_arg(x)
  srs <- ensure_null(srs)

  bbox_res <- wfs_get_bbox(x = x, srs = srs, srs_dest = 25830, limit_km2 = 4)

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsAD.aspx",
    verbose = verbose,
    query = list(
      # WFS service
      service = "wfs",
      version = "2.0.0",
      request = "getfeature",
      typenames = "AD.ADDRESS",
      # Stored query
      bbox = paste0(bbox_res, collapse = ","),
      SRSNAME = 25830
    )
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  # Transform back to the desired srs
  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  if (is.null(srs)) {
    srs <- sf::st_crs(x)
  }
  out <- sf::st_transform(out, srs)

  out
}
#' @description
#' - By street code: Implemented on `catr_wfs_get_address_codvia()`. Extract
#'   objects of specific addresses.
#'
#' @param codvia Cadastral street code.
#' @param del Cadastral office code.
#' @param mun Cadastral municipality code.
#'
#' @rdname catr_wfs_get_address
#' @export
catr_wfs_get_address_codvia <- function(
  codvia,
  del,
  mun,
  srs = NULL,
  verbose = FALSE
) {
  codvia <- validate_non_empty_arg(codvia)
  del <- validate_non_empty_arg(del)
  mun <- validate_non_empty_arg(mun)
  srs <- ensure_null(srs)
  # Fake call to validate srs
  if (!is.null(srs)) {
    wfs_get_bbox(c(1, 1, 1, 1), srs = srs)
  }

  q <- list(
    # WFS service
    service = "wfs",
    version = "2.0.0",
    request = "getfeature",
    StoredQuerie_id = "getadbycodvia",
    # Stored query
    codvia = codvia,
    del = del,
    mun = mun
  )

  q$SRSNAME <- srs

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsAD.aspx",
    verbose = verbose,
    query = q
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  out
}

#' @description
#' - By cadastral reference: Implemented on `catr_wfs_get_address_rc()`. Extract
#'   objects of specific cadastral references.
#'
#' @rdname catr_wfs_get_address
#' @export
catr_wfs_get_address_rc <- function(rc, srs = NULL, verbose = FALSE) {
  rc <- validate_non_empty_arg(rc)
  srs <- ensure_null(srs)
  # Fake call to validate srs
  if (!is.null(srs)) {
    wfs_get_bbox(c(1, 1, 1, 1), srs = srs)
  }

  q <- list(
    # WFS service
    service = "wfs",
    version = "2.0.0",
    request = "getfeature",
    StoredQuerie_id = "GetadByRefcat",
    # Stored query
    REFCAT = rc
  )

  q$SRSNAME <- srs

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsAD.aspx",
    verbose = verbose,
    query = q
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  out
}
#' @description
#' - By postal codes: Implemented on `catr_wfs_get_address_postalcode()`.
#'   Extract objects of specific postal codes
#'
#' @param postalcode Postal code.
#'
#' @rdname catr_wfs_get_address
#' @export
#' @examplesIf run_example()
#' \donttest{
#' ad <- catr_wfs_get_address_bbox(
#'   c(
#'     233673, 4015968, 233761, 4016008
#'   ),
#'   srs = 25830
#' )
#'
#' library(ggplot2)
#'
#' ggplot(ad) +
#'   geom_sf()
#' }
catr_wfs_get_address_postalcode <- function(
  postalcode,
  srs = NULL,
  verbose = FALSE
) {
  postalcode <- validate_non_empty_arg(postalcode)
  srs <- ensure_null(srs)
  # Fake call to validate srs
  if (!is.null(srs)) {
    wfs_get_bbox(c(1, 1, 1, 1), srs = srs)
  }

  q <- list(
    # WFS service
    service = "wfs",
    version = "2.0.0",
    request = "getfeature",
    StoredQuerie_id = "getadbypostalcode",
    # Stored query
    postalcode = postalcode
  )

  q$SRSNAME <- srs

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsAD.aspx",
    verbose = verbose,
    query = q
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  out
}

test_that("BBOX Check", {
  skip_on_cran()
  skip_if_offline()

  expect_message(
    fend <- catr_wfs_get_address_bbox(
      c(-20, -20, -19, -20),
      srs = 4326
    ),
    "didn't provide results"
  )
  expect_null(fend)

  obj <- catr_wfs_get_address_bbox(
    c(760926, 4019259, 761155, 4019366),
    srs = 25829
  )

  expect_true(sf::st_crs(obj)$epsg == 25829)

  # Convert to spatial object
  bbox <- c(760926, 4019259, 761155, 4019366)
  class(bbox) <- "bbox"
  bbox <- bbox |>
    sf::st_as_sfc() |>
    sf::st_set_crs(25829)
  expect_s3_class(bbox, "sfc")

  obj2 <- catr_wfs_get_address_bbox(bbox)
  expect_true(sf::st_crs(obj2) == sf::st_crs(25829))
})

test_that("AD CODVIA", {
  skip_on_cran()
  skip_if_offline()

  obj <- catr_wfs_get_address_codvia("1", 11, 39)
  expect_s3_class(obj, "sf")

  # Another SRS
  obj <- catr_wfs_get_address_codvia("1", 11, 39, srs = 4326)
  expect_s3_class(obj, "sf")
  expect_equal(sf::st_crs(obj)$epsg, 4326)

  expect_snapshot(
    obj <- catr_wfs_get_address_codvia("1", 110, 390)
  )
  expect_null(obj)
  expect_snapshot(
    error = TRUE,
    obj <- catr_wfs_get_address_codvia("1", 110, 390, srs = 9999)
  )
})

test_that("AD RC", {
  skip_on_cran()
  skip_if_offline()

  obj <- catr_wfs_get_address_rc("3662303TF3136B")
  expect_s3_class(obj, "sf")

  # Another SRS
  obj <- catr_wfs_get_address_rc("3662303TF3136B", srs = 4326)
  expect_s3_class(obj, "sf")
  expect_equal(sf::st_crs(obj)$epsg, 4326)

  expect_snapshot(
    obj <- catr_wfs_get_address_rc("3662303TF")
  )
  expect_null(obj)
  expect_snapshot(
    error = TRUE,
    obj <- catr_wfs_get_address_rc("3662303TF", srs = 9999)
  )
})

test_that("AD Postal Code", {
  skip_on_cran()
  skip_if_offline()

  obj <- catr_wfs_get_address_postalcode("18518")
  expect_true(nrow(obj) > 1)
  expect_s3_class(obj, "sf")

  # Another SRS
  obj <- catr_wfs_get_address_postalcode("18518", srs = 4326)
  expect_s3_class(obj, "sf")
  expect_equal(sf::st_crs(obj)$epsg, 4326)

  expect_snapshot(
    obj <- catr_wfs_get_address_postalcode("XXXXX")
  )
  expect_null(obj)
  expect_snapshot(
    error = TRUE,
    obj <- catr_wfs_get_address_rc("XXXXX", srs = 9999)
  )
})

#' ATOM INSPIRE: Reference database for ATOM cadastral parcels
#'
#' @description
#' Create a database containing the URLs provided in the INSPIRE ATOM
#' service of the Spanish Cadastre for extracting cadastral parcels.
#'  - `catr_atom_get_parcels_db_all()` provides a top-level table
#'    including information on all the territorial offices (except the
#'    Basque Country and Navarre) listing the municipalities included in
#'    each office.
#'  - `catr_atom_get_parcels_db_to()` provides a table for the specified
#'    territorial office including information for each of the
#'    municipalities of that office.
#'
#' @encoding UTF-8
#' @family INSPIRE
#' @family ATOM
#' @family parcels
#' @family databases
#' @inheritParams catr_atom_get_address_db_all
#' @inherit catr_atom_get_address_db_all return
#' @export
#'
#' @rdname catr_atom_get_parcels_db
#'
#' @source
#' ```{r, echo=FALSE, results='asis'}
#' cat(paste0(
#'    "<https://www.catastro.hacienda.gob.es/INSPIRE/CadastralParcels/",
#'      "ES.SDGC.CP.atom.xml>")
#'      )
#' ```
#'
#' @examplesIf run_example()
#' \donttest{
#' catr_atom_get_parcels_db_all()
#' }
catr_atom_get_parcels_db_all <- function(
  cache = deprecated(),
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE
) {
  if (lifecycle::is_present(cache)) {
    lifecycle::deprecate_warn(
      when = "1.0.0",
      what = "CatastRo::catr_atom_get_parcels_db_all(cache)",
      details = "Results are always cached."
    )
  }

  api_entry <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/",
    "CadastralParcels/ES.SDGC.CP.atom.xml"
  )

  file_local <- download_url(
    url = api_entry,
    cache_dir = cache_dir,
    subdir = "databases",
    update_cache = update_cache,
    verbose = verbose
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  tbl <- catr_read_atom(file_local, top = TRUE)
  names(tbl) <- c("territorial_office", "url", "munic", "date")

  tbl
}
#' @rdname catr_atom_get_parcels_db
#' @export
catr_atom_get_parcels_db_to <- function(
  to,
  cache = deprecated(),
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE
) {
  if (lifecycle::is_present(cache)) {
    lifecycle::deprecate_warn(
      when = "1.0.0",
      what = "CatastRo::catr_atom_get_parcels_db_to(cache)",
      details = "Results are always cached."
    )
  }

  all <- catr_atom_get_parcels_db_all(cache_dir = cache_dir)

  if (is.null(all)) {
    return(NULL)
  }

  alldist <- unique(all[, c("territorial_office", "url")])

  # Escape parentheses in territorial office names for matching
  to <- gsub("\\(|\\)", "", to)
  allto <- gsub("\\(|\\)", "", alldist$territorial_office)

  to_loc <- ensure_null(grep(to, allto, ignore.case = TRUE))
  if (is.null(to_loc)) {
    cli::cli_alert_warning(
      "No territorial office found with pattern {.str {to}}."
    )
    return(NULL)
  }

  # Compute string distances for territorial office matching
  with_d <- data.frame(
    to = alldist$territorial_office,
    dist = as.vector(adist(to, alldist$territorial_office))
  )
  with_d <- with_d[to_loc, ]
  with_d <- with_d[order(with_d$dist), ]

  tb <- with_d$to

  if (length(tb) > 1) {
    cli::cli_alert_info(
      "Found {length(tb)} territorial offices with pattern {.str {to}}."
    )

    cli::cli_alert_success("Selecting {.str {tb[1]}}.")
    cli::cli_alert_danger("Discarding:")
    bullets <- tb[-1]
    bullets <- paste0("{.str ", bullets, "}")
    names(bullets) <- rep(" ", length(bullets))
    cli::cli_bullets(bullets)

    tb <- tb[1]
  }

  make_msg(
    "info",
    verbose,
    paste0("Extracting information for {.str ", tb, "}.")
  )

  api_entry <- as.character(alldist[alldist$territorial_office == tb, "url"])

  file_local <- download_url(
    url = api_entry,
    cache_dir = cache_dir,
    subdir = "databases",
    update_cache = update_cache,
    verbose = verbose
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  tbl <- catr_read_atom(file_local, top = FALSE)

  names(tbl) <- c("munic", "url", "date")

  tbl
}

test_that("NULL result", {
  skip_on_cran()
  skip_if_offline()
  local_mocked_bindings(catr_atom_get_parcels_db_all = function(...) {
    NULL
  })

  expect_null(catr_atom_get_parcels_db_to("Madrid"))
})

test_that("Test offline db_all", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  cdir <- file.path(tempdir(), "testthat_ex1")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  expect_snapshot(
    fend <- catr_atom_get_parcels_db_all(cache_dir = cdir)
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("Test offline db_to", {
  skip_on_cran()
  skip_if_offline()

  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  cdir <- file.path(tempdir(), "testthat_ex1to")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  expect_snapshot(
    fend <- catr_atom_get_parcels_db_to("Madrid", cache_dir = cdir)
  )
  expect_null(fend)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
  unlink(cdir, recursive = TRUE, force = TRUE)
})

test_that("Test 404 all", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex2")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_atom_get_parcels_db_all(cache_dir = cdir)
  )
  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
  unlink(cdir, recursive = TRUE, force = TRUE)
  # Otherwise work
  expect_silent(
    fend <- catr_atom_get_parcels_db_all(cache_dir = cdir)
  )
  expect_gt(nrow(fend), 20)

  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})

test_that("Test 404 to", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex2to")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  fend <- catr_atom_get_parcels_db_to("Madrid", cache_dir = cdir)

  expect_null(fend)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })

  # Now it should throw another

  unlink(cdir, recursive = TRUE, force = TRUE)
  # Otherwise work
  expect_silent(
    fend <- catr_atom_get_parcels_db_to("Madrid", cache_dir = cdir)
  )
  expect_gt(nrow(fend), 100)

  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})

test_that("Test atom cp", {
  skip_on_cran()
  skip_if_offline()

  expect_message(catr_atom_get_parcels_db_all(
    verbose = TRUE,
    cache_dir = tempdir()
  ))
  expect_snapshot(
    no_res <- catr_atom_get_parcels_db_to(
      to = "aaaana",
      cache_dir = tempdir()
    )
  )
  expect_null(no_res)

  expect_silent(
    nmel <- catr_atom_get_parcels_db_to(
      to = "Melilla",
      cache_dir = tempdir()
    )
  )
  expect_s3_class(nmel, "tbl")
  expect_shape(nmel, dim = c(1, 3))

  # Several patterns
  expect_snapshot(
    several <- catr_atom_get_parcels_db_to(
      to = "lencia",
      cache_dir = tempdir()
    )
  )

  expect_silent(
    pal <- catr_atom_get_parcels_db_to(
      to = "Palencia",
      cache_dir = tempdir()
    )
  )

  expect_identical(several, pal)

  # full name
  expect_silent(
    val <- catr_atom_get_parcels_db_to(
      to = "valencia",
      cache_dir = tempdir()
    )
  )
  expect_false(pal$munic[1] == val$munic[1])
})

test_that("Deprecations", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex2to")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  expect_snapshot(
    fend <- catr_atom_get_parcels_db_to(
      to = "Madrid",
      cache = FALSE,
      cache_dir = cdir
    )
  )

  expect_snapshot(
    fend <- catr_atom_get_parcels_db_all(cache_dir = cdir, cache = FALSE)
  )

  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})

test_that("Test 404 to bis", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex2to2")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  all <- catr_atom_get_parcels_db_all(cache_dir = cdir)

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_snapshot(
    fend <- catr_atom_get_parcels_db_to("Madrid", cache_dir = cdir)
  )
  expect_null(fend)
  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })
  unlink(cdir, recursive = TRUE, force = TRUE)
})

list(
  rd_family_title = list(
    ATOM = "Other INSPIRE ATOM services:",
    WFS = "Other INSPIRE WFS services:",
    WMS = "Other INSPIRE WMS services:",
    INSPIRE = "INSPIRE API functions:",
    OVCCoordenadas = "OVCCoordenadas API:",
    OVCCallejero = "OVCCoordenadas API:"
  )
)

#' OVCCoordenadas: Reverse geocode cadastral references on a region
#'
#' @description
#' Implementation of the OVCCoordenadas service
#' [Consulta RCCOOR Distancia](`r ovcurl("RCCOORD")`). Return cadastral
#' references for coordinates. If none found, the API returns references
#' in a 50 square meter area around the requested coordinates.
#'
#' @encoding UTF-8
#' @family OVCCoordenadas
#' @family cadastral references
#' @inheritParams catr_ovc_get_cpmrc
#' @export
#' @inherit catr_ovc_get_cpmrc return
#'
#' @references
#' [Consulta RCCOOR Distancia](`r ovcurl("RCCOORD")`).
#'
#' @param lat Latitude for the query, expressed in the CRS/SRS defined by
#'   `srs`.
#' @param lon Longitude for the query, expressed in the CRS/SRS defined by
#'   `srs`.
#'
#' @seealso [catr_srs_values], `vignette("ovcservice", package = "CatastRo")`
#'
#' @details
#' When the API does not provide any result, the function returns a
#' [tibble][tibble::tbl_df] with the input arguments only.
#'
#' On a successful query, the function returns a [tibble][tibble::tbl_df] with
#' one row by cadastral reference, including the following columns:
#' - `geo.xcen`, `geo.ycen`, `geo.srs`: Input arguments of the query.
#' - `refcat`: Cadastral reference.
#' - `address`: Address as recorded in the Cadastre.
#' - `cmun_ine`: Municipality code as registered on the INE (National
#'    Statistics Institute).
#' - Rest of fields: Check the API Docs.
#'
#' @examplesIf run_example()
#' \donttest{
#' catr_ovc_get_rccoor_distancia(
#'   lat = 40.963200,
#'   lon = -5.671420,
#'   srs = 4326
#' )
#' }
catr_ovc_get_rccoor_distancia <- function(
  lat,
  lon,
  srs = 4326,
  verbose = FALSE
) {
  # Sanity checks
  lat <- validate_non_empty_arg(lat)
  lon <- validate_non_empty_arg(lon)

  valid_srs <- CatastRo::catr_srs_values
  valid <- as.character(valid_srs[valid_srs$ovc_service, ]$SRS)

  srs <- match_arg_pretty(srs, valid)
  srs <- paste0("EPSG:", srs)

  # Prepare query
  ##  Build url
  api_entry <- paste0(
    "http://ovc.catastro.meh.es/ovcservweb/",
    "OVCSWLocalizacionRC/OVCCoordenadas.asmx/Consulta_RCCOOR_Distancia?",
    "SRS=&Coordenada_X=&Coordenada_Y="
  )

  api_entry <- httr2::url_modify_query(
    api_entry,
    SRS = srs,
    Coordenada_X = lon,
    Coordenada_Y = lat
  )

  # Extract results
  resp <- get_request_body(api_entry, verbose = verbose)

  if (is.null(resp)) {
    return(NULL)
  }

  content_list <- xml2::as_list(httr2::resp_body_xml(resp))

  # nolint start
  res <- content_list[["consulta_coordenadas_distancias"]][[
    "coordenadas_distancias"
  ]][["coordd"]]
  # nolint end

  # Get overall info of the query
  overall <- unlist(res["geo"])
  overall <- tibble::as_tibble_row(overall)

  # Extract Ref Cast info
  rc <- res[["lpcd"]]

  if (is.null(rc)) {
    cli::cli_alert_warning("Query does not produce results.")
    return(overall)
  }

  rc_all <- lapply(rc, function(x) {
    tibble::as_tibble_row(unlist(x))
  })
  rc_all <- dplyr::bind_rows(rc_all)

  # Build additional fields, as the RC, address and munic (INE)
  rc_help <- tibble::tibble(
    refcat = paste0(rc_all$pc.pc1, rc_all$pc.pc2),
    address = rc_all$ldt,
    cmun_ine = paste0(rc_all$dt.loine.cp, rc_all$dt.loine.cm)
  )

  # Join all

  out <- dplyr::bind_cols(
    overall,
    rc_help,
    rc_all
  )

  # Numeric
  out["geo.xcen"] <- as.numeric(out[["geo.xcen"]])
  out["geo.ycen"] <- as.numeric(out[["geo.ycen"]])

  out
}

#' OVCCoordenadas: Geocode a cadastral reference
#'
#' @description
#' Implementation of the OVCCoordenadas service
#' [Consulta CPMRC](`r ovcurl("CPMRC")`). Returns coordinates for a
#' specific cadastral reference.
#'
#' @encoding UTF-8
#' @family OVCCoordenadas
#' @family cadastral references
#' @inheritParams catr_set_cache_dir
#' @export
#'
#' @references
#' [Consulta CPMRC](`r ovcurl("CPMRC")`).
#'
#' @seealso [catr_srs_values], `vignette("ovcservice", package = "CatastRo")`
#'
#' @param rc The cadastral reference to be geocoded.
#' @param province,municipality Optional, used for narrowing the search.
#' @param srs SRS/CRS to use on the query. To see allowed values, use
#'   [catr_srs_values], specifically the `ovc_service` column.
#'
#' @return A [tibble][tibble::tbl_df]. See **Details**
#'
#' @details
#' When the API does not provide any result, the function returns a
#' [tibble][tibble::tbl_df] with the input arguments only.
#'
#' On a successful query, the function returns a [tibble][tibble::tbl_df]
#' with one row per cadastral reference, including the following columns:
#' - `xcoord`, `ycoord`: X and Y coordinates in the specified SRS.
#' - `refcat`: Cadastral Reference.
#' - `address`: Address as recorded in the Cadastre.
#' - Rest of fields: Check the API Docs.
#'
#' @examplesIf run_example()
#' \donttest{
#'
#' # Using all arguments
#' catr_ovc_get_cpmrc("13077A01800039",
#'   4230,
#'   province = "CIUDAD REAL",
#'   municipality = "SANTA CRUZ DE MUDELA"
#' )
#'
#' # Only the cadastral reference
#' catr_ovc_get_cpmrc("9872023VH5797S")
#' }
#'
catr_ovc_get_cpmrc <- function(
  rc,
  srs = 4326,
  province = NULL,
  municipality = NULL,
  verbose = FALSE
) {
  # Sanity checks
  rc <- validate_non_empty_arg(rc)

  valid_srs <- CatastRo::catr_srs_values
  valid <- as.character(valid_srs[valid_srs$ovc_service, ]$SRS)

  srs <- match_arg_pretty(srs, valid)
  srs <- paste0("EPSG:", srs)

  # Prepare query
  ##  Build url
  api_entry <- paste0(
    "http://ovc.catastro.meh.es/ovcservweb/",
    "OVCSWLocalizacionRC/OVCCoordenadas.asmx/Consulta_CPMRC?",
    "Provincia=&Municipio=&SRS=&RC="
  )

  # Replace NAs and NULL on optional params
  province <- ensure_null(province)
  municipality <- ensure_null(municipality)

  api_entry <- httr2::url_modify_query(
    api_entry,
    RC = rc,
    SRS = srs,
    Provincia = ifelse(is.null(province), "", province),
    Municipio = ifelse(is.null(municipality), "", municipality)
  )

  # Extract results
  resp <- get_request_body(api_entry, verbose = verbose)

  if (is.null(resp)) {
    return(NULL)
  }

  content_list <- xml2::as_list(httr2::resp_body_xml(resp))

  # Check API custom error
  err <- content_list[["consulta_coordenadas"]]

  if (("lerr" %in% names(err))) {
    df <- tibble::as_tibble_row(unlist(err["lerr"]))

    cli::cli_alert_danger(
      paste0("Error code: ", df[1, 1], ". ", df[1, 2])
    )

    empty <- tibble::tibble(
      r = rc,
      srs = srs
    )

    names(empty) <- c("refcat", "geo.srs")
    return(empty)
  }

  res <- content_list[["consulta_coordenadas"]][["coordenadas"]][["coord"]]

  # Get info of the query
  overall <- tibble::as_tibble_row(unlist(res))

  # Extract helper info
  rc_help <- tibble::tibble(
    xcoord = as.double(overall$geo.xcen),
    ycoord = as.double(overall$geo.ycen),
    refcat = paste0(overall$pc.pc1, overall$pc.pc2),
    address = overall$ldt
  )

  # Join all

  out <- dplyr::bind_cols(
    rc_help,
    overall
  )

  out
}

#' Client tool for WFS INSPIRE services
#'
#' @description
#' Access WFS INSPIRE services. This function is used internally in WFS calls
#' and is exposed for users and developers accessing other cadastral or
#' INSPIRE resources.
#'
#' @encoding UTF-8
#' @family INSPIRE
#' @family WFS
#' @export
#' @inheritParams catr_set_cache_dir
#'
#' @rdname inspire_wfs_get
#'
#' @param scheme Character string. Protocol to access the resource on the
#'   Internet.
#' @param hostname Character string. Host that holds the resource.
#' @param path Character string. Specific resource in the host to access.
#' @param query Named list. Names and values of arguments to query.
#'
#' @return
#' Character string. Path of the resulting file in the [tempfile()] folder.
#'
#' @details
#' This function is used internally in all the WFS calls. We expose it to make
#' it available to other users and/or developers for accessing other
#' cadastral or INSPIRE resources. See **Examples**.
#'
#' @examplesIf run_example()
#' # Accessing the Cadastre of Navarra
#' # Try also https://ropenspain.github.io/CatastRoNav/
#'
#' file_local <- inspire_wfs_get(
#'   hostname = "inspire.navarra.es",
#'   path = "services/BU/wfs",
#'   query = list(
#'     service = "WFS",
#'     request = "getfeature",
#'     typenames = "BU:Building",
#'     bbox = "609800,4740100,611000,4741300",
#'     SRSNAME = "EPSG:25830"
#'   )
#' )
#'
#' if (!is.null(file_local)) {
#'   pamp <- sf::read_sf(file_local)
#'
#'   library(ggplot2)
#'   ggplot(pamp) +
#'     geom_sf()
#' }
inspire_wfs_get <- function(
  scheme = "https",
  hostname = "ovc.catastro.meh.es",
  path = "INSPIRE/wfsCP.aspx",
  query = list(),
  verbose = FALSE
) {
  # Validate query
  if (!is.list(query)) {
    cli::cli_abort(
      "{.arg query} should be a list, not {.obj_type_friendly {query}}."
    )
  }

  l_init <- length(query)
  query <- lapply(query, ensure_null)
  query <- query[lengths(query) > 0]
  nm <- unlist(lapply(names(query), ensure_null))
  query <- query[nm]
  names(query) <- tolower(nm)
  l_end <- length(query)

  dif_nm <- l_init - l_end

  if (dif_nm > 0) {
    cli::cli_alert_warning(
      "Removing {dif_nm} empty and/or unnamed element{?/s} in {.arg query}."
    )
  }

  if (l_end == 0) {
    cli::cli_abort(
      "{.arg query} can't be {.obj_type_friendly {query}}."
    )
  }

  # SRS should be checked
  if ("srsname" %in% names(query)) {
    srs <- query$srsname
    query$srsname <- ifelse(grepl("^EPS", srs), srs, paste0("EPSG:", srs))
  }

  # We don't use httr2 since some needed values (::, ,) are masked
  q <- paste0(names(query), "=", query, collapse = "&")

  # Build url
  url <- paste0(trimws(hostname), "/", trimws(path), "?", q)

  # Clean double slashes and ?? just in case
  url <- gsub("//", "/", url, fixed = TRUE)
  url <- gsub("??", "?", url, fixed = TRUE)

  url <- paste0(trimws(scheme), "://", url)
  # Create id from md5sum
  tmpfile <- tempfile(fileext = "txt")
  writeLines(url, tmpfile)
  id <- unname(tools::md5sum(tmpfile))
  file_gml <- paste0(id, ".gml")
  unlink(tmpfile)

  file_local <- download_url(
    url,
    file_gml,
    cache_dir = tempdir(),
    subdir = "wfs_inspire_cache",
    verbose = verbose
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  # Check results
  top20lines <- readLines(file_local, n = 20, warn = FALSE)

  if (!any(grepl("<Exception", top20lines))) {
    return(file_local)
  }

  # If not gml
  xml_file <- gsub("gml$", "xml", file_local)
  file.copy(file_local, xml_file)

  err <- xml2::read_xml(xml_file, encoding = "UTF-8")
  msg <- unlist(xml2::as_list(err)["ExceptionReport"], use.names = FALSE)

  cli::cli_alert_danger(
    c("The query {.url {url}} didn't provide results:\n", msg)
  )

  # Clean temp
  unlink(list.files(
    tempdir(),
    recursive = TRUE,
    pattern = id,
    full.names = TRUE
  ))
  NULL
}

#' Prepare the bbox of an object for WFS
#'
#' Results in 3857 since the Catastro API fails in some other projections.
#' Also warn if beyond the API limits.
#'
#' @param x sf or double vector of length 4.
#' @param srs SRS of the bbox, not needed if x is sf.
#' @param srs_dest Destination srs.
#' @param limit_km2 API limit
#'
#' @noRd
wfs_get_bbox <- function(x, srs = NULL, srs_dest = 3857, limit_km2 = Inf) {
  if (!(inherits(x, "sf") || inherits(x, "sfc"))) {
    if (length(x) != 4) {
      cli::cli_abort(
        "Length of {.arg x} should be {.val {4L}}, not {.val {length(x)}}."
      )
    }
    if (is.null(srs)) {
      cli::cli_abort(
        paste0(
          "You should also provide the {.arg srs} argument when x is ",
          "{.obj_type_friendly {x}}."
        )
      )
    }

    srs_db <- CatastRo::catr_srs_values
    valid <- srs_db[srs_db$wfs_service, ]$SRS
    srs <- as.numeric(match_arg_pretty(srs, as.character(valid)))

    sfobj <- x
    class(sfobj) <- "bbox"
    sfobj <- sf::st_as_sfc(sfobj)
    sfobj <- sf::st_set_crs(sfobj, srs)
  } else {
    sfobj <- sf::st_as_sfc(sf::st_bbox(x))
  }

  sfobj <- sf::st_transform(sfobj, srs_dest)

  # API limits (using 3857)
  obj_for_area <- sf::st_transform(sfobj, 3857)
  area <- sf::st_area(obj_for_area)
  # Dirty convert to km2
  area <- round(as.double(area) / 1000000, 1)

  if (area > limit_km2) {
    cli::cli_alert_warning(
      "API Endpoint Restriction: {limit_km2} km2. Your query is {area} km2."
    )
    cli::cli_alert_info(
      paste0(
        "Operation may fail, check the results or use a ",
        "smaller area on {.arg x}."
      )
    )
  }
  sf::st_bbox(sfobj)
}

test_that("wfs_get_bbox", {
  expect_snapshot(error = TRUE, wfs_get_bbox(c(1, 2)))
  expect_snapshot(error = TRUE, wfs_get_bbox(c(1, 2, 3, 4)))
  expect_silent(ok <- wfs_get_bbox(c(1, 1, 1, 1), srs = 4326))
  expect_s3_class(ok, "bbox")

  expect_equal(sf::st_crs(ok)$epsg, 3857)

  expect_true(is.numeric(as.vector(ok)))
  expect_length(ok, 4)
  expect_identical(ok, wfs_get_bbox(sf::st_as_sfc(ok)))

  # Trigger limit
  buf <- c(0, 0, 10000, 10000)
  class(buf) <- "bbox"
  buf <- sf::st_as_sfc(buf)
  buf <- sf::st_set_crs(buf, 3857)
  expect_snapshot(wfs_get_bbox(buf, limit_km2 = 1))

  # Check transformation to another srs
  geobox <- c(1, 1, 2, 1)
  another <- wfs_get_bbox(geobox, srs = 4326, srs_dest = 25830)
  merc <- wfs_get_bbox(geobox, srs = 4326, srs_dest = 3857)

  expect_false(any(another == merc))
})
test_that("Test offline", {
  skip_on_cran()
  skip_if_offline()
  local_mocked_bindings(is_online_fun = function(...) {
    FALSE
  })

  cdir <- file.path(tempdir(), "wfs_inspire_cache")
  unlink(cdir, recursive = TRUE)
  expect_snapshot(
    fend <- inspire_wfs_get(
      path = "INSPIRE/wfsBU.aspx",
      query = list(
        request = "getfeature",
        Typenames = "BU.BUILDING",
        SRSname = 25829,
        bbox = "742438,4046840,742613,4046970"
      )
    )
  )
  expect_null(fend)
  expect_length(
    list.files(cdir, recursive = TRUE),
    0
  )
  unlink(cdir, recursive = TRUE, force = TRUE)

  local_mocked_bindings(is_online_fun = function(...) {
    httr2::is_online()
  })
  expect_identical(is_online_fun(), httr2::is_online())
})

test_that("Test 404", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "wfs_inspire_cache")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  local_mocked_bindings(is_404 = function(...) {
    TRUE
  })

  expect_message(
    s <- inspire_wfs_get(
      path = "INSPIRE/wfsBU.aspx",
      query = list(
        request = "getfeature",
        Typenames = "BU.BUILDING",
        SRSname = 25829,
        bbox = "742438,4046840,742613,4046970"
      )
    ),
    "Error "
  )
  expect_null(s)

  local_mocked_bindings(is_404 = function(...) {
    FALSE
  })

  # Otherwise work
  expect_silent(
    s <- inspire_wfs_get(
      path = "INSPIRE/wfsBU.aspx",
      query = list(
        request = "getfeature",
        Typenames = "BU.BUILDING",
        SRSname = 25829,
        bbox = "742438,4046840,742613,4046970"
      )
    )
  )
  expect_length(s, 1)
  expect_true(is.character(s))
  expect_true(file.exists(s))
  expect_silent(tosf <- sf::read_sf(s))
  expect_s3_class(tosf, "sf")

  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})

test_that("Error on call", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "wfs_inspire_cache")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  expect_message(
    s <- inspire_wfs_get(
      path = "INSPIRE/wfsBU.aspx",
      query = list(
        request = "getfeatureaa",
        Typenames = "BU.BUILDING",
        SRSname = 25829,
        bbox = "742438,4046840,742613,4046970"
      )
    ),
    "didn't provide results:"
  )
  expect_null(s)

  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})

test_that("Bad query", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "wfs_inspire_cache")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }

  expect_snapshot(
    error = TRUE,
    s <- inspire_wfs_get(
      path = "INSPIRE/wfsBU.aspx",
      query = 20
    )
  )

  expect_snapshot(
    error = TRUE,
    s <- inspire_wfs_get(
      path = "INSPIRE/wfsBU.aspx",
      query = list(20, NA, NULL)
    )
  )

  expect_message(
    s <- inspire_wfs_get(
      path = "INSPIRE/wfsBU.aspx",
      query = list(
        request = "getfeature",
        STOREDQUERIE_ID = "GETOTHERBUILDINGBYPARCEL",
        NULL,
        refcat = "9398516VK3799G",
        NA,
        "another one bite the dust",
        srsname = "EPSG::25829"
      ),
      verbose = TRUE
    ),
    "Removing 3 empty and/or unnamed elements in"
  )

  sfobj1 <- read_geo_file_sf(s)

  unlink(s)

  # What about my EPSG?
  sfobj2 <- inspire_wfs_get(
    path = "INSPIRE/wfsBU.aspx",
    query = list(
      request = "getfeature",
      STOREDQUERIE_ID = "GETOTHERBUILDINGBYPARCEL",
      refcat = "9398516VK3799G",
      srsname = 25829
    )
  ) |>
    read_geo_file_sf()

  expect_identical(sfobj1, sfobj2)
  expect_s3_class(sfobj1, "sf")
  expect_equal(sf::st_crs(sfobj1)$epsg, 25829)

  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
})

#' @title Reference SRS codes for \CRANpkg{CatastRo} APIs
#'
#' @description
#' A [tibble][tibble::tbl_df] including the valid SRS (also known as CRS)
#' values that may be used on each API service. Values are provided as
#' [EPSG codes](https://en.wikipedia.org/wiki/EPSG_Geodetic_Parameter_Dataset).
#'
#' @docType data
#' @encoding UTF-8
#' @family databases
#' @family WFS
#' @family OVCCoordenadas
#' @name catr_srs_values
#'
#' @references
#' ```{r, echo=FALSE, results='asis'}
#' cat(paste0("- [OVCCoordenadas](https://ovc.catastro.meh.es/",
#' "ovcservweb/ovcswlocalizacionrc/ovccoordenadas.asmx)."))
#'
#' ```
#'
#' - [INSPIRE WFS
#' Service](https://www.catastro.hacienda.gob.es/webinspire/index.html).
#'
#' @seealso [sf::st_crs()].
#'
#' @format
#' A [tibble][tibble::tbl_df] with `r nrow(CatastRo::catr_srs_values)` rows
#' and columns:
#' \describe{
#'   \item{SRS}{Spatial Reference System (CRS) value, identified by the
#'     corresponding
#'     [EPSG](https://en.wikipedia.org/wiki/EPSG_Geodetic_Parameter_Dataset)
#'     code.}
#'   \item{Description}{Description of the SRS/EPSG code.}
#'   \item{ovc_service}{Logical. Is this code valid on OVC services?}
#'   \item{wfs_service}{Logical. Is this code valid on INSPIRE WFS services?}
#' }
#' @details
#' ```{r, echo=FALSE}
#' tb <- CatastRo::catr_srs_values
#' for(i in seq_len(ncol(tb))){
#'   tb[,i] <- as.vector(paste0("`", tb[[i]], "`"))
#' }
#' nm <- paste0("**", names(tb), "**")
#' knitr::kable(tb, col.names = nm, caption = "Content of [catr_srs_values]")
#'
#' ```
#'
#' @examples
#' data("catr_srs_values")
#'
#' # OVC valid codes
#' library(dplyr)
#'
#' catr_srs_values |> filter(ovc_service)
#'
#' # WFS valid codes
#'
#' catr_srs_values |> filter(wfs_service)
#'
#' # Use with sf::st_crs()
#'
#' catr_srs_values |>
#'   filter(wfs_service & ovc_service) |>
#'   print() |>
#'   # First value
#'   slice_head(n = 1) |>
#'   pull(SRS) |>
#'   # As crs
#'   sf::st_crs(.)
NULL

#' WFS INSPIRE: Download buildings
#'
#' @description
#' Get the spatial data of buildings. The WFS Service allows performing
#' two types of queries:
#' - By bounding box: Implemented on `catr_wfs_get_buildings_bbox()`.
#'   Extract objects included in the bounding box provided. See
#'   **Bounding box**.
#'
#' @encoding UTF-8
#' @family INSPIRE
#' @family WFS
#' @family buildings
#' @family spatial
#' @export
#'
#' @rdname catr_wfs_get_buildings
#'
#' @inheritParams catr_wfs_get_address_bbox
#' @inheritParams catr_atom_get_buildings
#' @inherit catr_wfs_get_address_bbox return references
#' @inheritSection catr_wfs_get_address_bbox API Limits
#' @inheritSection catr_wfs_get_address_bbox Bounding box
catr_wfs_get_buildings_bbox <- function(
  x,
  what = c("building", "buildingpart", "other"),
  srs = NULL,
  verbose = FALSE
) {
  # Sanity checks
  x <- validate_non_empty_arg(x)
  srs <- ensure_null(srs)
  what <- match_arg_pretty(what)

  # Switch to stored queries
  stored_query <- switch(what,
    "building" = "BU.BUILDING",
    "buildingpart" = "BU.BUILDINGPART",
    "other" = "BU.OTHERCONSTRUCTION"
  )

  bbox_res <- wfs_get_bbox(x = x, srs = srs, srs_dest = 25830, limit_km2 = 4)

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsBU.aspx",
    verbose = verbose,
    query = list(
      # WFS service
      service = "wfs",
      version = "2.0.0",
      request = "getfeature",
      typenames = stored_query,
      # Stored query
      bbox = paste0(bbox_res, collapse = ","),
      SRSNAME = 25830
    )
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  # Transform back to the desired srs
  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  if (is.null(srs)) {
    srs <- sf::st_crs(x)
  }
  out <- sf::st_transform(out, srs)
}

#' @description
#' - By cadastral reference: Implemented on `catr_wfs_get_buildings_rc()`.
#'   Extract objects of specific cadastral references.
#'
#' @rdname catr_wfs_get_buildings
#'
#' @export
#' @examplesIf run_example()
#' \donttest{
#' # Using bbox
#' building <- catr_wfs_get_buildings_bbox(
#'   c(
#'     376550,
#'     4545424,
#'     376600,
#'     4545474
#'   ),
#'   srs = 25830
#' )
#' library(ggplot2)
#' ggplot(building) +
#'   geom_sf() +
#'   labs(title = "Search using bbox")
#'
#' # Using rc
#' rc <- catr_wfs_get_buildings_rc("6656601UL7465N")
#' library(ggplot2)
#' ggplot(rc) +
#'   geom_sf() +
#'   labs(title = "Search using rc")
#' }
catr_wfs_get_buildings_rc <- function(
  rc,
  what = c("building", "buildingpart", "other"),
  srs = NULL,
  verbose = FALSE
) {
  # Sanity checks
  rc <- validate_non_empty_arg(rc)
  srs <- ensure_null(srs)
  what <- match_arg_pretty(what)
  # Fake call to validate srs
  if (!is.null(srs)) {
    wfs_get_bbox(c(1, 1, 1, 1), srs = srs)
  }

  # Switch to stored queries
  stored_query <- switch(what,
    "building" = "GetBuildingByParcel",
    "buildingpart" = "GetBuildingPartByParcel",
    "other" = "GetOtherBuildingByParcel"
  )

  q <- list(
    # WFS service
    service = "wfs",
    version = "2.0.0",
    request = "getfeature",
    StoredQuerie_id = stored_query,
    # Stored query
    REFCAT = rc
  )
  q$SRSNAME <- srs

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsBU.aspx",
    verbose = verbose,
    query = q
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  out
}

test_that("BBOX Check", {
  skip_on_cran()
  skip_if_offline()

  expect_message(
    fend <- catr_wfs_get_buildings_bbox(
      c(-20, -20, -19, -20),
      srs = 4326
    ),
    "didn't provide results"
  )
  expect_null(fend)

  obj <- catr_wfs_get_buildings_bbox(
    c(760926, 4019259, 761155, 4019366),
    srs = 25829
  )

  expect_true(sf::st_crs(obj)$epsg == 25829)

  # Convert to spatial object
  bbox <- c(760926, 4019259, 761155, 4019366)
  class(bbox) <- "bbox"
  bbox <- bbox |>
    sf::st_as_sfc() |>
    sf::st_set_crs(25829)
  expect_s3_class(bbox, "sfc")

  obj2 <- catr_wfs_get_buildings_bbox(bbox)
  expect_true(sf::st_crs(obj2) == sf::st_crs(25829))

  # Another types
  parts <- catr_wfs_get_buildings_bbox(
    bbox,
    what = "buildingpart"
  )

  expect_s3_class(parts, "sf")

  expect_gt(nrow(parts), nrow(obj2))

  ot <- # Another types
    parts <- catr_wfs_get_buildings_bbox(
      bbox,
      what = "other"
    )

  expect_s3_class(ot, "sf")

  expect_gt(nrow(obj2), nrow(ot))
})

test_that("Check error on bad rc", {
  skip_on_cran()
  skip_if_offline()

  expect_message(f <- catr_wfs_get_buildings_rc(rc = "1234"))
  expect_null(f)
})

test_that("BU Check srs", {
  skip_on_cran()
  skip_if_offline()

  obj <- catr_wfs_get_buildings_rc("3662303TF3136B", srs = 3857)
  expect_true(sf::st_crs(obj) == sf::st_crs(3857))
})


test_that("BU Part Check", {
  skip_on_cran()
  skip_if_offline()

  obj <- catr_wfs_get_buildings_rc("9398516VK3799G", what = "buildingpart")
  expect_true(nrow(obj) > 1)
  expect_s3_class(obj, "sf")
})

test_that("BU Other Check", {
  skip_on_cran()
  skip_if_offline()

  obj <- catr_wfs_get_buildings_rc("9398516VK3799G", what = "other")
  expect_s3_class(obj, "sf")
})

#' Decide whether an example should run
#'
#' @description
#' Determine whether an example should run based on the current platform and
#' network availability.
#'
#' @return Logical. `TRUE` if examples should run, `FALSE` otherwise.
#' @encoding UTF-8
#'
#' @details
#' Returns `FALSE` on CRAN, macOS, or when offline.
#'
#' @keywords internal
#' @export
#' @examples
#' run_example()
#'
run_example <- function() {
  if (on_mac()) {
    return(FALSE)
  }
  if (!httr2::is_online()) {
    return(FALSE) # nocov
  }
  if (on_cran()) {
    return(FALSE)
  }

  TRUE
}

#' Check if running on CRAN
#'
#' @return Logical. `TRUE` if running on CRAN, `FALSE` otherwise.
#'
#' @noRd
on_cran <- function() {
  env <- Sys.getenv("NOT_CRAN")
  if (identical(env, "")) {
    !interactive() # nocov
  } else {
    !isTRUE(as.logical(env))
  }
}

#' Check if running on macOS
#'
#' @return Logical. `TRUE` if running on macOS, `FALSE` otherwise.
#'
#' @noRd
on_mac <- function() {
  tolower(Sys.info()[["sysname"]]) %in% c("mac", "darwin")
}

test_that("On mac", {
  skip_on_cran()
  skip_if_offline()
  skip_on_os("mac")

  expect_false(on_mac())
  expect_true(run_example())
  local_mocked_bindings(on_mac = function(...) {
    TRUE
  })
  expect_false(run_example())
})

test_that("On CRAN", {
  skip_on_cran()
  skip_if_offline()
  skip_on_os("mac")

  # Imagine we are in CRAN
  env_orig <- Sys.getenv("NOT_CRAN")
  Sys.setenv("NOT_CRAN" = "false")
  expect_true(on_cran())
  expect_false(run_example())

  # Restore
  Sys.setenv("NOT_CRAN" = env_orig)
  expect_identical(Sys.getenv("NOT_CRAN"), env_orig)
  expect_true(run_example())
})

#' Get the cadastral municipality code from coordinates
#'
#' @description
#' Get the municipality code for coordinates using a [`sf`][sf::st_sf]
#' object or a pair of coordinates via [catr_ovc_get_cod_munic()].
#'
#' @encoding UTF-8
#' @family search
#' @inheritParams catr_ovc_get_cod_munic
#' @inheritParams catr_ovc_get_cpmrc
#' @inheritParams catr_set_cache_dir
#' @inheritDotParams mapSpain::esp_get_munic_siane year resolution region munic
#' @export
#' @inherit catr_ovc_get_cod_munic return details
#' @inherit catr_ovc_get_cpmrc seealso
#'
#' @seealso
#' [mapSpain::esp_get_munic_siane()], [catr_ovc_get_cod_munic()],
#' [sf::st_centroid()].
#'
#' @param x It may be:
#'   - A pair of coordinates `c(x,y)`. In this case the `srs` of the coordinates
#'     must be provided.
#'   - A [`sf`][sf::st_sf] object. If the object has several geometries, only
#'     the first geometry is used. The function extracts coordinates using
#'     `sf::st_centroid(x, of_largest_polygon = TRUE)`.
#'
#' @examplesIf run_example()
#' \donttest{
#' # Use with coords
#' catr_get_code_from_coords(c(-16.25462, 28.46824), srs = 4326)
#'
#' # Use with sf
#' prov <- mapSpain::esp_get_prov("Caceres")
#' catr_get_code_from_coords(prov)
#' }
catr_get_code_from_coords <- function(
  x,
  srs = NULL,
  verbose = FALSE,
  cache_dir = NULL,
  ...
) {
  x <- validate_non_empty_arg(x)

  if (!(inherits(x, "sf") || inherits(x, "sfc"))) {
    if (length(x) != 2) {
      cli::cli_abort(
        "Length of {.arg x} should be {.val {2L}}, not {.val {length(x)}}."
      )
    }
    if (is.null(srs)) {
      cli::cli_abort(
        paste0(
          "You should also provide the {.arg srs} argument when x is ",
          "{.obj_type_friendly {x}}."
        )
      )
    }

    x <- sf::st_point(x)
    x <- sf::st_sfc(x)

    # Set crs
    sf::st_crs(x) <- sf::st_crs(srs)
  }

  x <- sf::st_geometry(x)

  if (length(x) > 1) {
    cli::cli_alert_info(
      "Selecting the first geometry (you provided {.val {length(x)}})."
    )
  }

  x <- sf::st_transform(x[1], 3857)
  x <- sf::st_centroid(x, of_largest_polygon = TRUE)

  # Get munic
  cache_dir <- create_cache_dir(cache_dir)

  mun <- mapSpain::esp_get_munic_siane(
    cache_dir = cache_dir,
    verbose = verbose,
    moveCAN = FALSE,
    rawcols = FALSE,
    ...
  )
  if (is.null(mun)) {
    return(NULL)
  }
  mun <- sf::st_transform(mun, sf::st_crs(x))

  aa <- sf::st_intersects(mun, x, sparse = FALSE)

  if (!any(as.vector(aa))) {
    cli::cli_alert_warning("Coordinates not found")
    return(NULL)
  }

  getcode <- mun[as.vector(aa), ]

  catr_ovc_get_cod_munic(getcode$cpro, cmun_ine = getcode$cmun)
}

#' WFS INSPIRE: Download cadastral parcels
#'
#' @description
#' Get the spatial data of cadastral parcels and zones. The WFS Service
#' allows several types of queries:
#' - By bounding box: Implemented on `catr_wfs_get_parcels_bbox()`. Extract
#'   objects included in the bounding box provided. See **Bounding box**.
#'
#' @encoding UTF-8
#' @family INSPIRE
#' @family WFS
#' @family parcels
#' @family spatial
#' @export
#'
#' @rdname catr_wfs_get_parcels
#'
#' @inheritParams catr_wfs_get_address_bbox
#' @inheritParams catr_atom_get_parcels
#' @inherit catr_wfs_get_address_bbox return references
#' @inheritSection catr_wfs_get_address_bbox Bounding box
#'
#' @section API Limits:
#' The API service is limited to the following constraints:
#' - `"parcel`: Bounding box of 1km2 and a maximum of 5,000 elements.
#' - `"zoning"`: Bounding box of 25km2 and a maximum of 5,000 elements.
catr_wfs_get_parcels_bbox <- function(
  x,
  what = c("parcel", "zoning"),
  srs = NULL,
  verbose = FALSE
) {
  # Sanity checks
  x <- validate_non_empty_arg(x)
  srs <- ensure_null(srs)
  what <- match_arg_pretty(what)

  # Switch to stored queries
  stored_query <- switch(what,
    "parcel" = "CP.CADASTRALPARCEL",
    "zoning" = "CP.CADASTRALZONING"
  )

  limit <- switch(what,
    "parcel" = 1,
    "zoning" = 25
  )

  bbox_res <- wfs_get_bbox(
    x = x,
    srs = srs,
    srs_dest = 25830,
    limit_km2 = limit
  )

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsCP.aspx",
    verbose = verbose,
    query = list(
      # WFS service
      service = "wfs",
      version = "2.0.0",
      request = "getfeature",
      typenames = stored_query,
      # Stored query
      bbox = paste0(bbox_res, collapse = ","),
      SRSNAME = 25830
    )
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  # Transform back to the desired srs
  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  if (is.null(srs)) {
    srs <- sf::st_crs(x)
  }
  out <- sf::st_transform(out, srs)
}
#' @description
#' - By zoning: Implemented on `catr_wfs_get_parcels_zoning()`. Extract
#'   objects of a specific cadastral zone.
#'
#' @param cod_zona Cadastral zone code.
#'
#' @rdname catr_wfs_get_parcels
#' @export
catr_wfs_get_parcels_zoning <- function(cod_zona, srs = NULL, verbose = FALSE) {
  # Sanity checks
  cod_zona <- validate_non_empty_arg(cod_zona)
  srs <- ensure_null(srs)
  # Fake call to validate srs
  if (!is.null(srs)) {
    wfs_get_bbox(c(1, 1, 1, 1), srs = srs)
  }

  q <- list(
    # WFS service
    service = "wfs",
    version = "2.0.0",
    request = "getfeature",
    StoredQuerie_id = "GetZoning",
    # Stored query
    cod_zona = cod_zona
  )

  q$SRSNAME <- srs

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsCP.aspx",
    verbose = verbose,
    query = q
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  out
}
#' @description
#' - By cadastral parcel: Implemented on `catr_wfs_get_parcels_parcel()`.
#'   Extract cadastral parcels of a specific cadastral reference.
#'
#' @rdname catr_wfs_get_parcels
#' @export
catr_wfs_get_parcels_parcel <- function(rc, srs = NULL, verbose = FALSE) {
  # Sanity checks
  rc <- validate_non_empty_arg(rc)
  srs <- ensure_null(srs)
  # Fake call to validate srs
  if (!is.null(srs)) {
    wfs_get_bbox(c(1, 1, 1, 1), srs = srs)
  }

  q <- list(
    # WFS service
    service = "wfs",
    version = "2.0.0",
    request = "getfeature",
    StoredQuerie_id = "GetParcel",
    # Stored query
    refcat = rc
  )

  q$SRSNAME <- srs

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsCP.aspx",
    verbose = verbose,
    query = q
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  out
}
#' @description
#' - Neighbor cadastral parcels: Implemented on
#'   `catr_wfs_get_parcels_neigh_parcel()`. Extract neighbor cadastral parcels
#'   of a specific cadastral reference.
#'
#' @rdname catr_wfs_get_parcels
#' @export
catr_wfs_get_parcels_neigh_parcel <- function(rc, srs = NULL, verbose = FALSE) {
  # Sanity checks
  rc <- validate_non_empty_arg(rc)
  srs <- ensure_null(srs)
  # Fake call to validate srs
  if (!is.null(srs)) {
    wfs_get_bbox(c(1, 1, 1, 1), srs = srs)
  }

  q <- list(
    # WFS service
    service = "wfs",
    version = "2.0.0",
    request = "getfeature",
    StoredQuerie_id = "GetNeighbourParcel",
    # Stored query
    refcat = rc
  )

  q$SRSNAME <- srs

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsCP.aspx",
    verbose = verbose,
    query = q
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  out
}
#' @description
#' - Cadastral parcels by zoning: Implemented on
#'  `catr_wfs_get_parcels_parcel_zoning()`. Extract cadastral parcels of a
#'  specific cadastral zone.
#'
#' @rdname catr_wfs_get_parcels
#' @export
#' @examplesIf run_example()
#' \donttest{
#' cp <- catr_wfs_get_parcels_bbox(
#'   c(
#'     233673, 4015968, 233761, 4016008
#'   ),
#'   srs = 25830
#' )
#'
#' library(ggplot2)
#'
#' ggplot(cp) +
#'   geom_sf()
#' }
catr_wfs_get_parcels_parcel_zoning <- function(
  cod_zona,
  srs = NULL,
  verbose = FALSE
) {
  # Sanity checks
  cod_zona <- validate_non_empty_arg(cod_zona)
  srs <- ensure_null(srs)
  # Fake call to validate srs
  if (!is.null(srs)) {
    wfs_get_bbox(c(1, 1, 1, 1), srs = srs)
  }

  q <- list(
    # WFS service
    service = "wfs",
    version = "2.0.0",
    request = "getfeature",
    StoredQuerie_id = "GetParcelsByZoning",
    # Stored query
    cod_zona = cod_zona
  )

  q$SRSNAME <- srs

  file_local <- inspire_wfs_get(
    path = "INSPIRE/wfsCP.aspx",
    verbose = verbose,
    query = q
  )

  if (is.null(file_local)) {
    return(NULL)
  }

  out <- read_geo_file_sf(file_local)
  unlink(file_local)
  out
}

test_that("BBOX Check errors", {
  skip_on_cran()
  skip_if_offline()

  expect_error(catr_wfs_get_parcels_bbox(x = "1234", what = "xxx"))
  expect_error(catr_wfs_get_parcels_bbox(x = "1234"))
  expect_error(catr_wfs_get_parcels_bbox(x = c("1234", "a", "3", "4")))
  expect_error(catr_wfs_get_parcels_bbox(x = c(1, 2, 3)))
  expect_error(catr_wfs_get_parcels_bbox(x = c(1, 2, 3, 4)))
  expect_message(s <- catr_wfs_get_parcels_bbox(x = c(1, 2, 3, 4), srs = 3857))
  expect_null(s)
})

test_that("BBOX Check projections", {
  skip_on_cran()
  skip_if_offline()

  # Check messages
  obj <- catr_wfs_get_parcels_bbox(
    c(760926, 4019259, 761155, 4019366),
    srs = 25829
  )

  expect_true(sf::st_crs(obj)$epsg == 25829)

  # Convert to spatial object
  obj2 <- sf::st_transform(obj, 4326)
  obj2 <- catr_wfs_get_parcels_bbox(obj2)
  expect_true(sf::st_crs(obj2)$epsg == 4326)

  # Another type
  obj3 <- catr_wfs_get_parcels_bbox(obj2, what = "zoning")
  expect_gt(nrow(obj2), nrow(obj3))
})

test_that("CP Zone", {
  skip_on_cran()
  skip_if_offline()

  obj <- catr_wfs_get_parcels_zoning("41624TF3146S")
  expect_s3_class(obj, "sf")
  obj2 <- catr_wfs_get_parcels_parcel_zoning("41624TF3146S", srs = 3857)
  expect_s3_class(obj2, "sf")
  expect_gt(nrow(obj2), nrow(obj))

  expect_message(
    obj <- catr_wfs_get_parcels_zoning("41624TF3146SZZ", srs = 3857)
  )
  expect_null(obj)
  expect_message(
    obj <- catr_wfs_get_parcels_parcel_zoning("41624TF3146SZZ", srs = 3857)
  )
  expect_null(obj)
})

test_that("CP Parcels", {
  skip_on_cran()
  skip_if_offline()

  neigh <- catr_wfs_get_parcels_neigh_parcel("3662303TF3136B")
  sing <- catr_wfs_get_parcels_parcel("3662303TF3136B")

  expect_gt(nrow(neigh), nrow(sing))

  # NULL
  expect_null(catr_wfs_get_parcels_neigh_parcel("3662303TFXXXXX", srs = 3857))
  expect_null(catr_wfs_get_parcels_parcel("3662303TFXXXXX", srs = 3857))
})

# Helper
ovcurl <- function(x) {
  # nocov start
  base <- "https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc"

  app <- switch(x,
    "CPMRC" = "ovccoordenadas.asmx?op=Consulta_CPMRC",
    "mun" = "ovccallejerocodigos.asmx?op=ConsultaMunicipioCodigos",
    "prov" = "ovccallejerocodigos.asmx?op=ConsultaProvincia",
    "RCCOORD" = "ovccoordenadas.asmx?op=Consulta_RCCOOR_Distancia",
    "RCCOOR" = "ovccoordenadas.asmx?op=Consulta_RCCOOR",
    NULL
  )

  paste0(c(base, app), collapse = "/")
  # nocov end
}

#' Read geospatial file into sf object
#'
#' @param file_local Character string. Local file path or URL to the
#'   geospatial file.
#' @param hint Character string. Hint for identifying files in zipped archives.
#' @param layer_hint Character string. Optional hint for layer names.
#' @param ... Additional arguments passed to `sf::read_sf()`.
#'
#' @return An `sf` object containing the geospatial data.
#' @encoding UTF-8
#'
#' @noRd
read_geo_file_sf <- function(
  file_local,
  hint = basename(file_local),
  layer_hint = NULL,
  ...
) {
  # Warn if file size is huge and no query

  if (all(!grepl("^http", file_local), file.exists(file_local))) {
    fsize <- file.size(file_local)
    fsize_unit <- fsize
    class(fsize_unit) <- class(object.size("a"))
    thr <- 20 * (1024^2)
    if (fsize > thr) {
      fsize_unit <- paste0("(", format(fsize_unit, units = "auto"), ").")
      make_msg("warning", TRUE, "Reading large file", fsize_unit)
      make_msg("generic", TRUE, "This may take a while.")
    }
  }

  # Create and read 'vsizip' construct for shp.zip
  if (grepl(".zip$", file_local, ignore.case = TRUE)) {
    shp_zip <- unzip(file_local, list = TRUE)
    shp_zip <- shp_zip$Name
    shp_zip <- shp_zip[grepl(hint, shp_zip)]
    shp_end <- shp_zip[1]

    # Read with vszip
    file_local <- file.path("/vsizip/", file_local, shp_end)
    file_local <- gsub("//", "/", file_local, fixed = TRUE)
  }

  layers <- sf::st_layers(file_local)
  if (!is.null(layer_hint)) {
    layers <- layers[grepl(layer_hint, layers$name, ignore.case = TRUE), ]
  }

  data_sf <- sf::read_sf(file_local, layer = layers$name[1], quiet = TRUE)

  data_sf <- sanitize_sf(data_sf)

  data_sf
}

#' Convert sf object to UTF-8
#'
#' @param data_sf An `sf` object to convert to UTF-8 encoding.
#'
#' @return An `sf` object with UTF-8 encoding applied.
#'
#' @source Extracted from [`sf`][sf::st_sf] package.
#'
#' @noRd
sanitize_sf <- function(data_sf) {
  # From sf/read.R - https://github.com/r-spatial/sf/blob/master/R/read.R
  set_utf8 <- function(x) {
    n <- names(x)
    Encoding(n) <- "UTF-8"
    to_utf8 <- function(x) {
      if (is.character(x)) {
        Encoding(x) <- "UTF-8"
      }
      x
    }
    structure(lapply(x, to_utf8), names = n)
  }
  # end

  # To UTF-8
  names <- names(data_sf)

  if (inherits(data_sf, "sf")) {
    g <- sf::st_geometry(data_sf)

    nm <- "geometry"
    data_utf8 <-
      as.data.frame(
        set_utf8(sf::st_drop_geometry(data_sf)),
        stringsAsFactors = FALSE
      )
  } else {
    data_utf8 <- set_utf8(data_sf)
  }

  data_utf8 <- tibble::as_tibble(data_utf8)

  if (!inherits(data_sf, "sf")) {
    return(data_utf8)
  }

  # Regenerate with right encoding
  data_sf <- sf::st_as_sf(data_utf8, g)

  # Rename geometry to geometry
  newnames <- names(data_sf)
  newnames[newnames == "g"] <- nm
  colnames(data_sf) <- newnames
  data_sf <- sf::st_set_geometry(data_sf, nm)

  # Some CRS are not properly defined (i.e may have additionalm properties)
  # Normalize with the EPSG number

  epsg_num <- sf::st_crs(data_sf)$epsg
  if (!identical(sf::st_crs(data_sf), sf::st_crs(epsg_num))) {
    sf::st_crs(data_sf) <- sf::st_crs(epsg_num)
  }

  data_sf <- sf::st_make_valid(data_sf)

  data_sf
}

get_sf_from_bbox <- function(bbox, srs = NULL) {
  if (inherits(bbox, "sf") || inherits(bbox, "sfc")) {
    return(bbox)
  }

  # Sanity check
  if (!(is.numeric(bbox) && length(bbox) == 4)) {
    cli::cli_abort(
      "{.arg bbox} has length {.val {4L}}, not {.val {length(bbox)}}."
    )
  }

  srs <- ensure_null(srs)
  if (is.null(srs)) {
    cli::cli_abort(
      "Please provide a valid non-empty value for {.arg srs}."
    )
  }

  # Create template for a spatial bbox
  template_sf <- sf::st_sfc(sf::st_point(c(0, 0)))
  template_bbox <- sf::st_bbox(template_sf)

  # Create the spatial object
  bbox_new <- bbox
  class(bbox_new) <- class(template_bbox)

  bbox_new <- sf::st_as_sfc(bbox_new)
  bbox_new <- sf::st_set_crs(bbox_new, srs)

  bbox_new
}

test_that("Read shp", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/Buildings/46/",
    "46900-VALENCIA/A.ES.SDGC.BU.46900.zip"
  )

  fake_local <- download_url(
    url,
    basename(url),
    cdir,
    "fake",
    update_cache = FALSE,
    verbose = FALSE
  )

  s <- read_geo_file_sf(fake_local, hint = "building.gml")

  expect_s3_class(s, "sf")
  expect_s3_class(s, "tbl_df")
  expect_true(file.exists(fake_local))

  unlink(cdir, recursive = TRUE, force = TRUE)
  expect_false(dir.exists(cdir))
})

test_that("Read shp address", {
  skip_on_cran()
  skip_if_offline()

  cdir <- file.path(tempdir(), "testthat_ex")
  if (dir.exists(cdir)) {
    unlink(cdir, recursive = TRUE, force = TRUE)
  }
  url <- paste0(
    "https://www.catastro.hacienda.gob.es/INSPIRE/Addresses/40/",
    "40146-MELQUE%20DE%20CERCOS/A.ES.SDGC.AD.40146.zip"
  )

  fake_local <- download_url(
    url,
    basename(url),
    cdir,
    "fake",
    update_cache = FALSE,
    verbose = FALSE
  )

  s <- read_geo_file_sf(fake_local, hint = "gml", "address")

  expect_s3_class(s, "sf")
  expect_s3_class(s, "tbl_df")
  expect_true(file.exists(fake_local))

  # But
  tb <- read_geo_file_sf(fake_local, hint = "gml", "fare")
  expect_s3_class(tb, c("tbl_df", "tbl", "data.frame"), exact = TRUE)

  unlink(cdir, recursive = TRUE, force = TRUE)
  expect_false(dir.exists(cdir))
})

test_that("get_sf_from_bbox", {
  a <- get_sf_from_bbox(c(1, 2, 3, 4), srs = 3857)
  b <- get_sf_from_bbox(a)

  expect_identical(a, b)

  c <- sf::st_sf(x = 1, geometry = b)
  cc <- get_sf_from_bbox(c)
  expect_identical(c, cc)

  expect_snapshot(error = TRUE, get_sf_from_bbox(c(1, 2)))
  expect_snapshot(error = TRUE, get_sf_from_bbox(c(1, 2, 1, 2)))
})

---
title: "Get started"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!-- CatastRo.Rmd is generated from CatastRo.Rmd.orig. Please edit that file -->



**CatastRo** provide access to different API services of the [Spanish
Cadastre](https://www.sedecatastro.gob.es/). With **CatastRo** it is possible to
download official information of addresses, properties, parcels and buildings.

## OVCCoordenadas Service

The
[OVCCoordenadas](https://ovc.catastro.meh.es/ovcservweb/OVCSWLocalizacionRC/OVCCoordenadas.asmx)
service allows to retrieve the coordinates of a known cadastral reference
(geocoding). It is also possible to retrieve the cadastral references around a
specific pair of coordinates (reverse geocoding). **CatastRo** returns the
results on a `tibble` format. This functionality is described in detail in the
corresponding vignette (see `vignette("ovcservice", package = "CatastRo")`).

## INSPIRE Services

> The INSPIRE Directive aims to create a European Union spatial data
> infrastructure for the purposes of EU environmental policies and policies or
> activities which may have an impact on the environment. This European Spatial
> Data Infrastructure will enable the sharing of environmental spatial
> information among public sector organisations, facilitate public access to
> spatial information across Europe and assist in policy-making across
> boundaries.
>
> *From <https://knowledge-base.inspire.ec.europa.eu/overview_en>*

The implementation of the INSPIRE directive on the Spanish Cadastre (see
[Catastro INSPIRE](https://www.catastro.minhap.es/webinspire/index.html)) allows
to retrieve spatial objects from the database of the cadastre:

-   **Vector objects:** Parcels, addresses, buildings, cadastral zones and more.
    These objects are provided by **CatastRo** as `sf` objects as provided by
    the **sf** package.
-   **Imagery:** Image layers representing the same information than the vector
    objects. These objects are provided by **CatastRo** as `SpatRaster` objects
    as provided by the **terra** package.

Note that **the coverage of this service is 95% of the Spanish territory**,
excluding Basque Country and Navarre[^1] that have their own independent
cadastral offices.

[^1]: The package [**CatastRoNav**](https://ropenspain.github.io/CatastRoNav/)
    provides access to the Cadastre of Navarre, with similar
    functionalities than **CatastRo**.

There are three types of functions, each one querying a different service:

1.  **ATOM service**: The ATOM service allows to batch-download vector objects
    of different cadastral elements for a specific municipality.

2.  **WFS service**: The WFS service allows to download vector objects of
    specific cadastral elements. Note that there are some *restrictions on the
    extension and number of elements* to query. For batch-downloading the ATOM
    service is preferred.

3.  **WMS service**: This service allows to download georeferenced images of
    different cadastral elements.

## Examples

### Working with layers

On this example we would demonstrate some of the main capabilities of the
package by recreating a cadastral map of the surroundings of the [Santiago
Bernabéu Stadium](https://en.wikipedia.org/wiki/Santiago_Bernab%C3%A9u_Stadium).
We would make use of the **WMS and WFS services** to get different layers in
order to show some of the capabilities of the package:


```r
# Extract building by bounding box
# Check https://boundingbox.klokantech.com/

library(CatastRo)

stadium <- catr_wfs_get_buildings_bbox(
  c(-3.6891446916, 40.4523311971, -3.687462138, 40.4538643165),
  srs = 4326
)

# Now extract cadastral parcels. We can use spatial objects on the query

stadium_parcel <- catr_wfs_get_parcels_bbox(stadium)

# Project for tiles

stadium_parcel_pr <- sf::st_transform(stadium_parcel, 25830)

# Extract imagery: Labels of the parcel

labs <- catr_wms_get_layer(stadium_parcel_pr,
  what = "parcel",
  styles = "BoundariesOnly",
  srs = 25830
)

# Plot
library(ggplot2)
library(tidyterra) # For terra tiles

ggplot() +
  geom_spatraster_rgb(data = labs) +
  geom_sf(data = stadium_parcel_pr, fill = NA, col = "red", linewidth = 2) +
  geom_sf(data = stadium, fill = "red", alpha = .5) +
  coord_sf(crs = 25830)
```

<div class="figure">
<img src="./santbernabeu-1.png" alt="Example: Santiago Bernabeu" width="100%" />
<p class="caption">Example: Santiago Bernabeu</p>
</div>

### Thematic maps

We can create also thematic maps using the information available on the spatial
objects. We would produce a visualization of the urban growth of Granada using
**CatastRo**, replicating the map produced by Dominic Royé on his post
[Visualize urban
growth](https://dominicroye.github.io/en/2019/visualize-urban-growth/), using
the **ATOM services**.

In first place, we extract the coordinates of the city center of Granada using
**mapSpain**:


```r
library(dplyr)
library(sf)
library(mapSpain)

# Use mapSpain for getting the coords

city <- esp_get_capimun(munic = "^Granada$")
```

Next step consists on extracting the buildings using the ATOM service. We would
use also the function `catr_get_code_from_coords()` to identify the code of
Granada in the Cadastre, and we would download the buildings with
`catr_atom_get_buildings()`.


```r
city_catr_code <- catr_get_code_from_coords(city)

city_catr_code
#> # A tibble: 1 × 12
#>   munic   catr_to catr_munic catrcode cpro  cmun  inecode nm      cd    cmc   cp    cm   
#>   <chr>   <chr>   <chr>      <chr>    <chr> <chr> <chr>   <chr>   <chr> <chr> <chr> <chr>
#> 1 GRANADA 18      900        18900    18    087   18087   GRANADA 18    900   18    87

city_bu <- catr_atom_get_buildings(city_catr_code$catrcode)
```

Next step for creating the visualization is to limit the analysis to a circle of
radius 1.5 km around the city center:


```r
buff <- city %>%
  # Adjust CRS to 25830: (Buildings)
  st_transform(st_crs(city_bu)) %>%
  # Buffer
  st_buffer(1500)


# Cut buildings

dataviz <- st_intersection(city_bu, buff)

ggplot(dataviz) +
  geom_sf()
```

<div class="figure">
<img src="./minimal-1.png" alt="Minimal cadastral map of Granada" width="100%" />
<p class="caption">Minimal cadastral map of Granada</p>
</div>

Let's extract now the construction year, available in the column `beginning`:


```r
# Extract 4 initial positions
year <- substr(dataviz$beginning, 1, 4)

# Replace all that doesn't look as a number with 0000
year[!(year %in% 0:2500)] <- "0000"


# To numeric
year <- as.integer(year)

# New column
dataviz <- dataviz %>%
  mutate(year = year)
```

Last step is to create groups based on the year and create the data
visualization. We use here the function `cut()` to create classes for every
decade starting in year 1900:


```r
dataviz <- dataviz %>%
  mutate(year_cat = cut(year,
    breaks = c(0, seq(1900, 2030, by = 10)),
    dig.lab = 4
  ))


ggplot(dataviz) +
  geom_sf(aes(fill = year_cat), color = NA, na.rm = TRUE) +
  scale_fill_manual(
    values = hcl.colors(15, "Spectral"),
    na.translate = FALSE
  ) +
  theme_void() +
  labs(title = "GRANADA", fill = "") +
  theme(
    panel.background = element_rect(fill = "black"),
    plot.background = element_rect(fill = "black"),
    legend.justification = .5,
    legend.text = element_text(
      colour = "white",
      size = 12
    ),
    plot.title = element_text(
      colour = "white", hjust = .5,
      margin = margin(t = 30),
      size = 30
    ),
    plot.caption = element_text(
      colour = "white",
      margin = margin(b = 20), hjust = .5
    ),
    plot.margin = margin(r = 40, l = 40)
  )
```

<div class="figure">
<img src="./dviz-1.png" alt="Granada: Urban growth" width="100%" />
<p class="caption">Granada: Urban growth</p>
</div>

## References

-   Royé D (2019). "Visualize urban growth."
    <https://dominicroye.github.io/en/2019/visualize-urban-growth/>.

---
title: "Get started"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!-- CatastRo.Rmd is generated from CatastRo.Rmd.orig. Please edit that file -->



**CatastRo** provide access to different API services of the [Spanish
Cadastre](https://www.sedecatastro.gob.es/). With **CatastRo** it is possible to
download official information of addresses, properties, parcels and buildings.

## OVCCoordenadas Service

The
[OVCCoordenadas](https://ovc.catastro.meh.es/ovcservweb/OVCSWLocalizacionRC/OVCCoordenadas.asmx)
service allows to retrieve the coordinates of a known cadastral reference
(geocoding). It is also possible to retrieve the cadastral references around a
specific pair of coordinates (reverse geocoding). **CatastRo** returns the
results on a `tibble` format. This functionality is described in detail in the
corresponding vignette (see `vignette("ovcservice", package = "CatastRo")`).

## INSPIRE Services

> The INSPIRE Directive aims to create a European Union spatial data
> infrastructure for the purposes of EU environmental policies and policies or
> activities which may have an impact on the environment. This European Spatial
> Data Infrastructure will enable the sharing of environmental spatial
> information among public sector organisations, facilitate public access to
> spatial information across Europe and assist in policy-making across
> boundaries.
>
> *From <https://knowledge-base.inspire.ec.europa.eu/overview_en>*

The implementation of the INSPIRE directive on the Spanish Cadastre (see
[Catastro INSPIRE](https://www.catastro.hacienda.gob.es/webinspire/index.html))
allows to retrieve spatial objects from the database of the cadastre:

-   **Vector objects:** Parcels, addresses, buildings, cadastral zones and more.
    These objects are provided by **CatastRo** as `sf` objects as provided by
    the **sf** package.
-   **Imagery:** Image layers representing the same information than the vector
    objects. These objects are provided by **CatastRo** as `SpatRaster` objects
    as provided by the **terra** package.

Note that **the coverage of this service is 95% of the Spanish territory**,
excluding Basque Country and Navarre[^1] that have their own independent
cadastral offices.

[^1]: The package [**CatastRoNav**](https://ropenspain.github.io/CatastRoNav/)
    provides access to the Cadastre of Navarre, with similar functionalities
    than **CatastRo**.

There are three types of functions, each one querying a different service:

1.  **ATOM service**: The ATOM service allows to batch-download vector objects
    of different cadastral elements for a specific municipality.

2.  **WFS service**: The WFS service allows to download vector objects of
    specific cadastral elements. Note that there are some *restrictions on the
    extension and number of elements* to query. For batch-downloading the ATOM
    service is preferred.

3.  **WMS service**: This service allows to download georeferenced images of
    different cadastral elements.

## Examples

### Working with layers

On this example we would demonstrate some of the main capabilities of the
package by recreating a cadastral map of the surroundings of the [Santiago
Bernabéu Stadium](https://en.wikipedia.org/wiki/Santiago_Bernab%C3%A9u_Stadium).
We would make use of the **WMS and WFS services** to get different layers in
order to show some of the capabilities of the package:


``` r
# Extract building by bounding box
# Check https://boundingbox.klokantech.com/

library(CatastRo)

stadium <- catr_wfs_get_buildings_bbox(
  c(-3.6891446916, 40.4523311971, -3.687462138, 40.4538643165),
  srs = 4326
)

# Now extract cadastral parcels. We can use spatial objects on the query

stadium_parcel <- catr_wfs_get_parcels_bbox(stadium)

# Project for tiles

stadium_parcel_pr <- sf::st_transform(stadium_parcel, 25830)

# Extract imagery: Labels of the parcel

labs <- catr_wms_get_layer(stadium_parcel_pr,
  what = "parcel",
  styles = "BoundariesOnly",
  srs = 25830
)

# Plot
library(ggplot2)
library(tidyterra) # For terra tiles

ggplot() +
  geom_spatraster_rgb(data = labs) +
  geom_sf(data = stadium_parcel_pr, fill = NA, col = "red", linewidth = 2) +
  geom_sf(data = stadium, fill = "red", alpha = .5) +
  coord_sf(crs = 25830)
```

<div class="figure">
<img src="./santbernabeu-1.png" alt="Example: Santiago Bernabeu" width="100%" />
<p class="caption">Example: Santiago Bernabeu</p>
</div>

### Thematic maps

We can create also thematic maps using the information available on the spatial
objects. We would produce a visualization of the urban growth of Granada using
**CatastRo**, replicating the map produced by Dominic Royé on his post
[Visualize urban
growth](https://dominicroye.github.io/en/2019/visualize-urban-growth/), using
the **ATOM services**.

In first place, we extract the coordinates of the city center of Granada using
**mapSpain**:


``` r
library(dplyr)
library(sf)
library(mapSpain)

# Use mapSpain for getting the coords

city <- esp_get_capimun(munic = "^Granada$")
```

Next step consists on extracting the buildings using the ATOM service. We would
use also the function `catr_get_code_from_coords()` to identify the code of
Granada in the Cadastre, and we would download the buildings with
`catr_atom_get_buildings()`.


``` r
city_catr_code <- catr_get_code_from_coords(city)

city_catr_code
#> # A tibble: 1 × 12
#>   munic catr_to catr_munic catrcode cpro  cmun  inecode nm    cd    cmc   cp    cm   
#>   <chr> <chr>   <chr>      <chr>    <chr> <chr> <chr>   <chr> <chr> <chr> <chr> <chr>
#> 1 GRAN… 18      900        18900    18    087   18087   GRAN… 18    900   18    87

city_bu <- catr_atom_get_buildings(city_catr_code$catrcode)
```

Next step for creating the visualization is to limit the analysis to a circle of
radius 1.5 km around the city center:


``` r
buff <- city %>%
  # Adjust CRS to 25830: (Buildings)
  st_transform(st_crs(city_bu)) %>%
  # Buffer
  st_buffer(1500)


# Cut buildings

dataviz <- st_intersection(city_bu, buff)

ggplot(dataviz) +
  geom_sf()
```

<div class="figure">
<img src="./minimal-1.png" alt="Minimal cadastral map of Granada" width="100%" />
<p class="caption">Minimal cadastral map of Granada</p>
</div>

Let's extract now the construction year, available in the column `beginning`:


``` r
# Extract 4 initial positions
year <- substr(dataviz$beginning, 1, 4)

# Replace all that doesn't look as a number with 0000
year[!(year %in% 0:2500)] <- "0000"


# To numeric
year <- as.integer(year)

# New column
dataviz <- dataviz %>%
  mutate(year = year)
```

Last step is to create groups based on the year and create the data
visualization. We use here the function `cut()` to create classes for every
decade starting in year 1900:


``` r
dataviz <- dataviz %>%
  mutate(year_cat = cut(year,
    breaks = c(0, seq(1900, 2030, by = 10)),
    dig.lab = 4
  ))


ggplot(dataviz) +
  geom_sf(aes(fill = year_cat), color = NA, na.rm = TRUE) +
  scale_fill_manual(
    values = hcl.colors(15, "Spectral"),
    na.translate = FALSE
  ) +
  theme_void() +
  labs(title = "GRANADA", fill = "") +
  theme(
    panel.background = element_rect(fill = "black"),
    plot.background = element_rect(fill = "black"),
    legend.justification = .5,
    legend.text = element_text(
      colour = "white",
      size = 12
    ),
    plot.title = element_text(
      colour = "white", hjust = .5,
      margin = margin(t = 30),
      size = 30
    ),
    plot.caption = element_text(
      colour = "white",
      margin = margin(b = 20), hjust = .5
    ),
    plot.margin = margin(r = 40, l = 40)
  )
```

<div class="figure">
<img src="./dviz-1.png" alt="Granada: Urban growth" width="100%" />
<p class="caption">Granada: Urban growth</p>
</div>

## References

-   Royé D (2019). "Visualize urban growth."
    <https://dominicroye.github.io/en/2019/visualize-urban-growth/>.

---
title: "OVCCoordenadas web service"
description: Retrieve spatial coordinates of urban properties.
author:
  - Ángel Delgado Panadero
  - Diego Hernangómez
tbl-cap-location: bottom
vignette: >
  %\VignetteIndexEntry{OVCCoordenadas web service}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

<!-- ovcservice.qmd is generated from ovcservice.qmd.orig. Please edit that file -->



**CatastRo** provides an **R** interface to the OVCCoordenadas web service from
the [Sede electrónica del Catastro](https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccoordenadas.asmx).

This service retrieves the spatial coordinates of an urban property. You do not
need to be the owner to get the information. You only need to know the
cadastral reference of the property. Although the cadastral reference is the
only required argument, providing the address can improve results and help
avoid errors.

The service can also obtain the cadastral reference of an urban property from
longitude and latitude. It allows you to choose the spatial reference system
(SRS, also known as CRS) used to express the coordinates.

The service also handles cases where the exact location of the registered urban
property is unknown. In such cases, it returns all properties located within a
50-meter square around the given point.

The documentation for this service is available
[here](https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccoordenadas.asmx).

These functions use the `catr_ovc_get_*()` prefix and return tibbles from the
**tibble** package.

## CatastRo API

The OVCCoordenadas web service can be accessed using the following functions:

- `catr_ovc_get_rccoor()`
- `catr_ovc_get_rccoor_distancia()`
- `catr_ovc_get_cpmrc()`

## Reverse geocoding cadastral references

`catr_ovc_get_rccoor()` takes the coordinates (`lat` and `lon`) and the spatial
reference system (`srs`) used to express them. It returns a tibble with the
cadastral reference of the property at that spatial point, including other
information such as the address (town, street and number).


``` r
library(CatastRo)

result <- catr_ovc_get_rccoor(
  lat = 38.6196566583596,
  lon = -3.45624183836806,
  srs = "4230"
)

knitr::kable(result)
```



|refcat         |address                                                                                              |pc.pc1  |pc.pc2  |  geo.xcen| geo.ycen|geo.srs   |ldt                                                                                                  |
|:--------------|:----------------------------------------------------------------------------------------------------|:-------|:-------|---------:|--------:|:---------|:----------------------------------------------------------------------------------------------------|
|13077A01800039 |DS DISEMINADO  Polígono 18 Parcela 39 000100200VH67C EL TIRADERO. SANTA CRUZ DE MUDELA (CIUDAD REAL) |13077A0 |1800039 | -3.456242| 38.61966|EPSG:4230 |DS DISEMINADO  Polígono 18 Parcela 39 000100200VH67C EL TIRADERO. SANTA CRUZ DE MUDELA (CIUDAD REAL) |



This function accepts the following values for the `srs` argument:


``` r
data(catr_srs_values)

# OVC valid codes.
library(dplyr)

catr_srs_values |>
  filter(ovc_service == TRUE) |>
  select(SRS, Description) |>
  knitr::kable()
```



|   SRS|Description            |
|-----:|:----------------------|
|  4230|Geográficas en ED 50   |
|  4258|Geográficas en ETRS89  |
|  4326|Geográficas en WGS 80  |
| 23029|UTM huso 29N en ED50   |
| 23030|UTM huso 30N en ED50   |
| 23031|UTM huso 31N en ED50   |
| 25829|UTM huso 29N en ETRS89 |
| 25830|UTM huso 30N en ETRS89 |
| 25831|UTM huso 31N en ETRS89 |
| 32627|UTM huso 27N en WGS 84 |
| 32628|UTM huso 28N en WGS 84 |
| 32629|UTM huso 29N en WGS 84 |
| 32630|UTM huso 30N en WGS 84 |
| 32631|UTM huso 31N en WGS 84 |



It is also possible to get all cadastral references within a 50-meter square
centered on the coordinates `lat` and `lon` using the function
`catr_ovc_get_rccoor_distancia()`.


``` r
catr_ovc_get_rccoor_distancia(
  lat = 40.96002,
  lon = -5.663408,
  srs = "4230"
) |>
  knitr::kable()
```



|  geo.xcen| geo.ycen|geo.srs   |refcat         |address                                         |cmun_ine |pc.pc1  |pc.pc2  |dt.loine.cp |dt.loine.cm |dt.lourb.dir.cv |dt.lourb.dir.pnp |ldt                                             |dis   |
|---------:|--------:|:---------|:--------------|:-----------------------------------------------|:--------|:-------|:-------|:-----------|:-----------|:---------------|:----------------|:-----------------------------------------------|:-----|
| -5.663408| 40.96002|EPSG:4230 |5877501TL7357F |AV REYES DE ESPAÑA 1 SALAMANCA (SALAMANCA)      |37274    |5877501 |TL7357F |37          |274         |643             |1                |AV REYES DE ESPAÑA 1 SALAMANCA (SALAMANCA)      |21.81 |
| -5.663408| 40.96002|EPSG:4230 |5778706TL7357H |AV REYES DE ESPAÑA 2 N2-4 SALAMANCA (SALAMANCA) |37274    |5778706 |TL7357H |37          |274         |643             |2                |AV REYES DE ESPAÑA 2 N2-4 SALAMANCA (SALAMANCA) |23.18 |



## Geocoding a cadastral reference

The opposite query is also possible. When provided with a cadastral reference
(`rc`), province (`province`) and municipality (`municipality`), the function
`catr_ovc_get_cpmrc()` returns the coordinates (`lat` and `lon`) in a specified
`srs`, along with the address (town, street and number).


``` r
catr_ovc_get_cpmrc(
  rc = "13077A01800039",
  srs = "4230",
  province = "CIUDAD REAL",
  municipality = "SANTA CRUZ DE MUDELA"
) |>
  knitr::kable()
```



|    xcoord|   ycoord|refcat         |address                                                                                              |pc.pc1  |pc.pc2  |geo.xcen          |geo.ycen         |geo.srs   |ldt                                                                                                  |
|---------:|--------:|:--------------|:----------------------------------------------------------------------------------------------------|:-------|:-------|:-----------------|:----------------|:---------|:----------------------------------------------------------------------------------------------------|
| -3.456242| 38.61966|13077A01800039 |DS DISEMINADO  Polígono 18 Parcela 39 000100200VH67C EL TIRADERO. SANTA CRUZ DE MUDELA (CIUDAD REAL) |13077A0 |1800039 |-3.45624183836806 |38.6196566583596 |EPSG:4230 |DS DISEMINADO  Polígono 18 Parcela 39 000100200VH67C EL TIRADERO. SANTA CRUZ DE MUDELA (CIUDAD REAL) |



The `province` and `municipality` arguments are optional, but if `municipality`
is provided, `province` must also be provided. If a value is passed to the
`province` argument while `municipality` is `NULL`, the function
`catr_ovc_get_cpmrc()` displays a message and returns an empty tibble.


``` r
catr_ovc_get_cpmrc(
  rc = "13077A01800039",
  municipality = "SANTA CRUZ DE MUDELA"
) |>
  knitr::kable()
#> ✖ OVC service error 11: LA PROVINCIA ES OBLIGATORIA
```



|refcat         |geo.srs   |
|:--------------|:---------|
|13077A01800039 |EPSG:4326 |



When using only `rc`, the result is provided as expected:


``` r
# Get the result without a warning.
catr_ovc_get_cpmrc(rc = "13077A01800039") |>
  knitr::kable()
```



|    xcoord|   ycoord|refcat         |address                                                                                              |pc.pc1  |pc.pc2  |geo.xcen          |geo.ycen         |geo.srs   |ldt                                                                                                  |
|---------:|--------:|:--------------|:----------------------------------------------------------------------------------------------------|:-------|:-------|:-----------------|:----------------|:---------|:----------------------------------------------------------------------------------------------------|
| -3.457532| 38.61843|13077A01800039 |DS DISEMINADO  Polígono 18 Parcela 39 000100200VH67C EL TIRADERO. SANTA CRUZ DE MUDELA (CIUDAD REAL) |13077A0 |1800039 |-3.45753233627867 |38.6184314024661 |EPSG:4326 |DS DISEMINADO  Polígono 18 Parcela 39 000100200VH67C EL TIRADERO. SANTA CRUZ DE MUDELA (CIUDAD REAL) |

---
title: "Get started"
description: Brief introduction to the CatastRo package.
vignette: >
  %\VignetteIndexEntry{Get started}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
bibliography: REFERENCES.bib
link-citations: true
---

<!-- CatastRo.qmd is generated from CatastRo.qmd.orig. Please edit that file -->



**CatastRo** provides access to services from the [Spanish
Cadastre](https://www.sedecatastro.gob.es/). With **CatastRo**, you can
download official address, cadastral parcel, building and map data.

## OVCCoordenadas service

The
[OVCCoordenadas](https://ovc.catastro.meh.es/ovcservweb/OVCSWLocalizacionRC/OVCCoordenadas.asmx)
service retrieves the coordinates of a known cadastral reference
(geocoding). It is also possible to retrieve the cadastral references around a
specific pair of coordinates (reverse geocoding). **CatastRo** returns the
results as a tibble. This functionality is described in detail in the
corresponding vignette (see `vignette("ovcservice", package = "CatastRo")`).

## INSPIRE services

<figure>

<blockquote class="blockquote">

The INSPIRE Directive aims to create a European Union Spatial Data
Infrastructure (SDI) for the purposes of EU environmental policies and policies
or activities which may have an impact on the environment. This European Spatial
Data Infrastructure enables the sharing of environmental spatial information
among public sector organizations, facilitates public access to spatial
information across Europe and assist in policy-making across boundaries.

</blockquote>

```{=html}
<figcaption class="blockquote-footer">Source: <cite title="INSPIRE Knowledge
Base"><a href="https://knowledge-base.inspire.ec.europa.eu/overview_en"
class="external-link uri">INSPIRE Knowledge Base</a>
</cite>
</figcaption>
```

</figure>

The implementation of the INSPIRE directive in the Spanish Cadastre (see
[Spanish Cadastre INSPIRE](https://www.catastro.hacienda.gob.es/webinspire/index.html))
allows retrieval of spatial objects from the cadastral database:

- **Vector objects:** Parcels, addresses, buildings, cadastral zones and more.
  **CatastRo** returns these objects as `sf` objects, using the **sf** package.
- **Imagery:** Image layers representing the same information as the vector
  objects. **CatastRo** returns these objects as `SpatRaster` objects, using
  the **terra** package.

Note that these services cover 95% of the Spanish territory,
excluding the Basque Country and Navarre[^1], which have their own independent
cadastral offices.

[^1]: The package [**CatastRoNav**](https://ropenspain.github.io/CatastRoNav/)
    provides access to the Cadastre of Navarre, with similar functionality to
    **CatastRo**.

There are three types of functions, each querying a different service:

1.  **ATOM service**: The ATOM service downloads complete municipal datasets
    for different cadastral elements.

2.  **WFS service**: The WFS service downloads vector objects for specific
    cadastral elements. Note that there are restrictions on the extent and
    number of elements that can be queried. For full municipal downloads,
    prefer the ATOM service.

3.  **WMS service**: The WMS service downloads georeferenced map images for
    different cadastral elements.

## Examples

### Working with layers

This example demonstrates some of the main capabilities of the package by
recreating a cadastral map of the surroundings of the [Santiago Bernabéu
Stadium](https://en.wikipedia.org/wiki/Santiago_Bernab%C3%A9u_Stadium). We use
the WMS and WFS services to retrieve different layers.


``` r
# Extract buildings by bounding box.
# Check https://boundingbox.klokantech.com/

library(CatastRo)

stadium <- catr_wfs_get_buildings_bbox(
  c(-3.6891446916, 40.4523311971, -3.687462138, 40.4538643165),
  srs = 4326
)

# Extract cadastral parcels using spatial objects in the query.

stadium_parcel <- catr_wfs_get_parcels_bbox(stadium)

# Project for tiles.

stadium_parcel_pr <- sf::st_transform(stadium_parcel, 25830)

# Extract parcel label imagery.

labs <- catr_wms_get_layer(
  stadium_parcel_pr,
  what = "parcel",
  styles = "BoundariesOnly",
  srs = 25830
)

# Plot.
library(ggplot2)
library(tidyterra) # For terra tiles.

ggplot() +
  geom_spatraster_rgb(data = labs) +
  geom_sf(
    data = stadium_parcel_pr,
    fill = NA, col = "red", linewidth = 2
  ) +
  geom_sf(data = stadium, fill = "red", alpha = .5) +
  coord_sf(crs = 25830)
```

<div class="figure">
<img src="./santbernabeu-1.png" alt="Figure 1: Santiago Bernabéu example" width="100%" />
<p class="caption">Figure 1: Santiago Bernabéu example</p>
</div>

### Thematic maps

We can also create thematic maps using the information available on the spatial
objects. We produce a visualization of the urban growth of Granada using
**CatastRo**, replicating the map produced by [Dominic
Royé](https://dominicroye.github.io) [@roye19], using the **ATOM service**.

First, we extract the coordinates of Granada's city center with
**mapSpain**:


``` r
library(dplyr)
library(sf)
library(mapSpain)

# Use mapSpain to get the coordinates.

city <- esp_get_capimun(munic = "^Granada$")
```

The next step is to extract the buildings using the ATOM service. We also use
`catr_get_code_from_coords()` to identify Granada's code in the Cadastre and
download the buildings with `catr_atom_get_buildings()`.


``` r
city_catr_code <- catr_get_code_from_coords(city)

city_catr_code
#> # A tibble: 1 × 12
#>   munic   catr_to catr_munic catrcode cpro  cmun  inecode nm     cd    cmc   cp   
#>   <chr>   <chr>   <chr>      <chr>    <chr> <chr> <chr>   <chr>  <chr> <chr> <chr>
#> 1 GRANADA 18      900        18900    18    087   18087   GRANA… 18    900   18   
#> # ℹ 1 more variable: cm <chr>

city_bu <- catr_atom_get_buildings(city_catr_code$catrcode)
```

Next, we limit the analysis to a circle with a radius of 1.5 km around the city
center:


``` r
buff <- city |>
  # Adjust CRS to 25830 for buildings.
  st_transform(st_crs(city_bu)) |>
  # Buffer.
  st_buffer(1500)

# Cut buildings.
dataviz <- st_intersection(city_bu, buff)

ggplot(dataviz) +
  geom_sf()
```

<div class="figure">
<img src="./minimal-1.png" alt="Figure 2: Minimal cadastral map of Granada" width="100%" />
<p class="caption">Figure 2: Minimal cadastral map of Granada</p>
</div>

Now let's extract the construction year, available in the column `beginning`:


``` r
# Extract the first four positions.
year <- substr(dataviz$beginning, 1, 4)

# Replace entries that do not look like numbers with 0000.
year[!(year %in% 0:2500)] <- "0000"

# Convert to numeric.
year <- as.integer(year)

# Add a new column.
dataviz <- dataviz |>
  mutate(year = year)
```

The last step is to group the data by construction year and create the
visualization. Here, `cut()` creates classes for each decade from 1900 onward:


``` r
dataviz <- dataviz |>
  mutate(
    year_cat = cut(year, breaks = c(0, seq(1900, 2030, by = 10)), dig.lab = 4)
  )

ggplot(dataviz) +
  geom_sf(aes(fill = year_cat), color = NA, na.rm = TRUE) +
  scale_fill_manual(
    values = hcl.colors(15, "Spectral"),
    na.translate = FALSE
  ) +
  theme_void() +
  labs(title = "GRANADA", fill = "") +
  theme(
    panel.background = element_rect(fill = "black"),
    plot.background = element_rect(fill = "black"),
    legend.justification = .5,
    legend.text = element_text(
      colour = "white",
      size = 12
    ),
    plot.title = element_text(
      colour = "white",
      hjust = .5,
      margin = margin(t = 30),
      size = 30
    ),
    plot.caption = element_text(
      colour = "white",
      margin = margin(b = 20),
      hjust = .5
    ),
    plot.margin = margin(r = 40, l = 40)
  )
```

<div class="figure">
<img src="./dviz-1.png" alt="Figure 2: Granada - Urban growth" width="100%" />
<p class="caption">Figure 2: Granada - Urban growth</p>
</div>

## References

# get_sf_from_bbox

    Code
      get_sf_from_bbox(c(1, 2))
    Condition
      Error in `get_sf_from_bbox()`:
      ! `bbox` has length 4, not 2.

---

    Code
      get_sf_from_bbox(c(1, 2, 1, 2))
    Condition
      Error in `get_sf_from_bbox()`:
      ! Please provide a valid non-empty value for `srs`.

# wfs_get_bbox

    Code
      wfs_get_bbox(c(1, 2))
    Condition
      Error in `wfs_get_bbox()`:
      ! Length of `x` should be 4, not 2.

---

    Code
      wfs_get_bbox(c(1, 2, 3, 4))
    Condition
      Error in `wfs_get_bbox()`:
      ! You should also provide the `srs` argument when x is a double vector.

---

    Code
      wfs_get_bbox(buf, limit_km2 = 1)
    Message
      ! API Endpoint Restriction: 1 km2. Your query is 100 km2.
      i Operation may fail, check the results or use a smaller area on `x`.
    Output
       xmin  ymin  xmax  ymax 
          0     0 10000 10000 

# Test offline

    Code
      fend <- inspire_wfs_get(path = "INSPIRE/wfsBU.aspx", query = list(request = "getfeature",
        Typenames = "BU.BUILDING", SRSname = 25829, bbox = "742438,4046840,742613,4046970"))
    Message
      x Offline
      > Returning "NULL"

# Bad query

    Code
      s <- inspire_wfs_get(path = "INSPIRE/wfsBU.aspx", query = 20)
    Condition
      Error in `inspire_wfs_get()`:
      ! `query` should be a list, not a number.

---

    Code
      s <- inspire_wfs_get(path = "INSPIRE/wfsBU.aspx", query = list(20, NA, NULL))
    Message
      ! Removing 3 empty and/or unnamed elements in `query`.
    Condition
      Error in `inspire_wfs_get()`:
      ! `query` can't be an empty list.

# Test offline

    Code
      fend <- catr_atom_search_munic("LABAJOS", cache_dir = cdir)
    Message
      x Offline
      > Returning "NULL"

# Test 404 all

    Code
      fend <- catr_atom_search_munic("MELQUE", to = "Segovia", cache_dir = cdir)
    Message
      x Error 404 (Not Found): <https://www.catastro.hacienda.gob.es/INSPIRE/Addresses/ES.SDGC.AD.atom.xml>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# Test search

    Code
      c <- catr_atom_search_munic("XXX", cache_dir = cdir)
    Message
      ! No municipality found with pattern "XXX" in .

---

    Code
      ff <- catr_atom_search_munic("Melilla", to = "Burgos", cache_dir = cdir)
    Message
      ! No municipality found with pattern "Melilla" in "Burgos".

# Deprecations

    Code
      a <- catr_atom_search_munic("Mad", cache_dir = cdir, cache = TRUE)
    Condition
      Warning:
      The `cache` argument of `catr_atom_search_munic()` is deprecated as of CatastRo 1.0.0.
      i Results are always cached.

# Test offline

    Code
      fend <- catr_get_code_from_coords(c(-16.25462, 28.46824), srs = 4326,
      cache_dir = cdir)
    Message
      x Offline
      > Returning "NULL"

---

    Code
      fend <- catr_get_code_from_coords(mapSpain::esp_get_prov_siane("Caceres"),
      cache_dir = cdir)
    Message
      x Offline
      > Returning "NULL"

# Test 404 all

    Code
      fend <- catr_get_code_from_coords(c(-16.25462, 28.46824), srs = 4326,
      cache_dir = cdir)
    Message
      x Error 404 (Not Found): <http://ovc.catastro.meh.es/ovcservweb//ovcswlocalizacionrc/ovccallejerocodigos.asmx/ConsultaMunicipioCodigos?%2FCodigoProvincia=&CodigoProvincia=38&CodigoMunicipio=&CodigoMunicipioIne=038>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

---

    Code
      fend <- catr_get_code_from_coords(mapSpain::esp_get_prov_siane("Caceres"),
      cache_dir = cdir)
    Message
      x Error 404 (Not Found): <http://ovc.catastro.meh.es/ovcservweb//ovcswlocalizacionrc/ovccallejerocodigos.asmx/ConsultaMunicipioCodigos?%2FCodigoProvincia=&CodigoProvincia=10&CodigoMunicipio=&CodigoMunicipioIne=125>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# Test 404 mapSpain

    Code
      fend <- catr_get_code_from_coords(c(-16.25462, 28.46824), srs = 4326,
      cache_dir = cdir)
    Message
      i Mocking mapSpain

# AD CODVIA

    Code
      obj <- catr_wfs_get_address_codvia("1", 110, 390)
    Message
      x Error 404 (Not Found): <https://ovc.catastro.meh.es/INSPIRE/wfsAD.aspx?service=wfs&version=2.0.0&request=getfeature&storedquerie_id=getadbycodvia&codvia=1&del=110&mun=390>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

---

    Code
      obj <- catr_wfs_get_address_codvia("1", 110, 390, srs = 9999)
    Condition
      Error:
      ! `srs` should be one of "3785", "3857", "4258", "4326", "25829", "25830" or "25831", not "9999".

# AD RC

    Code
      obj <- catr_wfs_get_address_rc("3662303TF")
    Message
      x The query <https://ovc.catastro.meh.es/INSPIRE/wfsAD.aspx?service=wfs&version=2.0.0&request=getfeature&storedquerie_id=GetadByRefcat&refcat=3662303TF> didn't provide results:
      Invalid length of REFCAT parameter

---

    Code
      obj <- catr_wfs_get_address_rc("3662303TF", srs = 9999)
    Condition
      Error:
      ! `srs` should be one of "3785", "3857", "4258", "4326", "25829", "25830" or "25831", not "9999".

# AD Postal Code

    Code
      obj <- catr_wfs_get_address_postalcode("XXXXX")
    Message
      x Error 404 (Not Found): <https://ovc.catastro.meh.es/INSPIRE/wfsAD.aspx?service=wfs&version=2.0.0&request=getfeature&storedquerie_id=getadbypostalcode&postalcode=XXXXX>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

---

    Code
      obj <- catr_wfs_get_address_rc("XXXXX", srs = 9999)
    Condition
      Error:
      ! `srs` should be one of "3785", "3857", "4258", "4326", "25829", "25830" or "25831", not "9999".

# Messages

    Code
      make_msg("generic", TRUE, "Hi", "I am a generic.", "See {.var avar}.")
    Message
      > Hi I am a generic. See `avar`.

---

    Code
      make_msg("info", TRUE, "Info here.", "See {.pkg igoR}.")
    Message
      i Info here. See igoR.

---

    Code
      make_msg("warning", TRUE, "Caution! A warning.", "But still OK.")
    Message
      ! Caution! A warning. But still OK.

---

    Code
      make_msg("danger", TRUE, "OOPS!", "I did it again :(")
    Message
      x OOPS! I did it again :(

---

    Code
      make_msg("success", TRUE, "Hooray!", "5/5 ;)")
    Message
      v Hooray! 5/5 ;)

# Pretty match

    Code
      my_fun("error here")
    Condition
      Error:
      ! `arg_one` should be one of "10", "1000", "3000" or "5000", not "error here".

---

    Code
      my_fun(c("an", "error"))
    Condition
      Error:
      ! `arg_one` should be one of "10", "1000", "3000" or "5000", not "an" or "error".

---

    Code
      my_fun("5")
    Condition
      Error:
      ! `arg_one` should be one of "10", "1000", "3000" or "5000", not "5".
      i Did you mean "5000"?

---

    Code
      my_fun("00")
    Condition
      Error:
      ! `arg_one` should be one of "10", "1000", "3000" or "5000", not "00".

---

    Code
      my_fun2(c(1, 2))
    Condition
      Error:
      ! `year` should be "20", not "1" or "2".

---

    Code
      my_fun3("3")
    Condition
      Error:
      ! `an_arg` should be one of "30" or "20", not "3".
      i Did you mean "30"?

---

    Code
      my_fun2(c(1, 2))
    Condition
      Error:
      ! `year` should be "20", not "1" or "2".

# Not empty

    Code
      a_fun()
    Condition
      Error in `a_fun()`:
      ! `a` can't be missing.

---

    Code
      a_fun(a = 1)
    Condition
      Error in `a_fun()`:
      ! `b` can't be missing.

# Test offline db_all

    Code
      fend <- catr_atom_get_parcels_db_all(cache_dir = cdir)
    Message
      x Offline
      > Returning "NULL"

# Test offline db_to

    Code
      fend <- catr_atom_get_parcels_db_to("Madrid", cache_dir = cdir)
    Message
      x Offline
      > Returning "NULL"

# Test 404 all

    Code
      fend <- catr_atom_get_parcels_db_all(cache_dir = cdir)
    Message
      x Error 404 (Not Found): <https://www.catastro.hacienda.gob.es/INSPIRE/CadastralParcels/ES.SDGC.CP.atom.xml>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# Test atom cp

    Code
      no_res <- catr_atom_get_parcels_db_to(to = "aaaana", cache_dir = tempdir())
    Message
      ! No territorial office found with pattern "aaaana".

---

    Code
      several <- catr_atom_get_parcels_db_to(to = "lencia", cache_dir = tempdir())
    Message
      i Found 2 territorial offices with pattern "lencia".
      v Selecting "Territorial office 34 Palencia".
      x Discarding:
        "Territorial office 46 Valencia"

# Deprecations

    Code
      fend <- catr_atom_get_parcels_db_to(to = "Madrid", cache = FALSE, cache_dir = cdir)
    Condition
      Warning:
      The `cache` argument of `catr_atom_get_parcels_db_to()` is deprecated as of CatastRo 1.0.0.
      i Results are always cached.

---

    Code
      fend <- catr_atom_get_parcels_db_all(cache_dir = cdir, cache = FALSE)
    Condition
      Warning:
      The `cache` argument of `catr_atom_get_parcels_db_all()` is deprecated as of CatastRo 1.0.0.
      i Results are always cached.

# Test 404 to bis

    Code
      fend <- catr_atom_get_parcels_db_to("Madrid", cache_dir = cdir)
    Message
      x Error 404 (Not Found): <http://www.catastro.hacienda.gob.es/INSPIRE/CadastralParcels/28/ES.SDGC.CP.atom_28.xml>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# Test offline

    Code
      fend <- catr_ovc_get_cod_munic(4, 5)
    Message
      x Offline
      > Returning "NULL"

# Test 404 all

    Code
      fend <- catr_ovc_get_cod_munic(4, 5)
    Message
      x Error 404 (Not Found): <http://ovc.catastro.meh.es/ovcservweb//ovcswlocalizacionrc/ovccallejerocodigos.asmx/ConsultaMunicipioCodigos?%2FCodigoProvincia=&CodigoProvincia=4&CodigoMunicipio=5&CodigoMunicipioIne=>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# Callejero munic

    Code
      catr_ovc_get_cod_munic(2)
    Condition
      Error in `catr_ovc_get_cod_munic()`:
      ! Please provide a non-"NULL" value either on `cmun` or `cmun_ine`.

# Test offline

    Code
      fend <- catr_ovc_get_cod_provinces()
    Message
      x Offline
      > Returning "NULL"

# Test 404 all

    Code
      fend <- catr_ovc_get_cod_provinces()
    Message
      x Error 404 (Not Found): <http://ovc.catastro.meh.es/ovcservweb//ovcswlocalizacionrc/ovccallejerocodigos.asmx/ConsultaProvincia?>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# Test offline

    Code
      fend <- catr_atom_get_parcels("LABAJOS", cache_dir = cdir)
    Message
      x Offline
      > Returning "NULL"

# Test 404 all

    Code
      fend <- catr_atom_get_parcels("MELQUE", to = "Segovia", cache_dir = cdir)
    Message
      x Error 404 (Not Found): <https://www.catastro.hacienda.gob.es/INSPIRE/CadastralParcels/ES.SDGC.CP.atom.xml>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# ATOM parcels

    Code
      catr_atom_get_parcels("xyxghx", cache_dir = cdir)
    Message
      ! No municipality found with pattern "xyxghx".
      i Check available municipalities with `CatastRo::catr_atom_get_address_db_all()`.
    Output
      NULL

---

    Code
      s <- catr_atom_get_parcels("Melque", to = "Segovia", cache = FALSE, cache_dir = cdir)
    Condition
      Warning:
      The `cache` argument of `catr_atom_get_parcels()` is deprecated as of CatastRo 1.0.0.
      i Results are always cached.
      Warning:
      The `cache` argument of `catr_atom_get_parcels_db_to()` is deprecated as of CatastRo 1.0.0.
      i Results are always cached.

# Test 404 single

    Code
      fend <- catr_atom_get_parcels("Melque", to = "Segovia", cache_dir = cdir)
    Message
      x Error 404 (Not Found): <https://www.catastro.hacienda.gob.es/INSPIRE/CadastralParcels/40/40146-MELQUE%20DE%20CERCOS/A.ES.SDGC.CP.40146.zip>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# Mock migration

    Code
      detected <- detect_cache_dir_muted()
    Message
      v CatastRo >= 1.0.0: Cache configuration migrated. See Note in `CatastRo::catr_set_cache_dir()` for details.
      i This is a one-time message, it won't be displayed in the future.

# Test offline

    Code
      fend <- download_url(url, cache_dir = cdir, subdir = "fixme", update_cache = FALSE,
        verbose = FALSE)
    Message
      x Offline
      > Returning "NULL"

# No connection body

    Code
      fend <- get_request_body(url, verbose = FALSE)
    Message
      x Offline
      > Returning "NULL"

# Error body

    Code
      fend <- get_request_body(url, verbose = FALSE)
    Message
      x Error 404 (Not Found): <https://www.catastro.hacienda.gob.es/INSPIRE/Addresses/ES.SDGC.AD.atom.xml>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

# Test offline

    Code
      fend <- catr_ovc_get_cpmrc("9872023VH5797S")
    Message
      x Offline
      > Returning "NULL"

# Test 404 all

    Code
      fend <- catr_ovc_get_cpmrc("9872023VH5797S")
    Message
      x Error 404 (Not Found): <http://ovc.catastro.meh.es/ovcservweb/OVCSWLocalizacionRC/OVCCoordenadas.asmx/Consulta_CPMRC?RC=9872023VH5797S&SRS=EPSG%3A4326&Provincia=&Municipio=>.
      ! If you think this is a bug please consider opening an issue on <https://github.com/ropenspain/CatastRo/issues>
      > Returning "NULL"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatastRo-package.R
\docType{package}
\name{CatastRo-package}
\alias{CatastRo}
\alias{CatastRo-package}
\title{CatastRo: Interface to the API 'Sede Electronica Del Catastro'}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Access public spatial data available under the 'INSPIRE' directive. Tools for downloading references and addresses of properties, as well as map images.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ropenspain.github.io/CatastRo/}
  \item \url{https://github.com/rOpenSpain/CatastRo}
  \item Report bugs at \url{https://github.com/rOpenSpain/CatastRo/issues}
}

}
\author{
\strong{Maintainer}: Diego Hernangómez \email{diego.hernangomezherrero@gmail.com} (\href{https://orcid.org/0000-0001-8457-4658}{ORCID})

Authors:
\itemize{
  \item Diego Hernangómez \email{diego.hernangomezherrero@gmail.com} (\href{https://orcid.org/0000-0001-8457-4658}{ORCID})
  \item Ángel Delgado Panadero \email{delgadopanadero@gmail.com} (\href{https://orcid.org/0000-0002-8189-9251}{ORCID}) [copyright holder]
}

Other contributors:
\itemize{
  \item Iñaki Ucar (\href{https://orcid.org/0000-0001-6403-5550}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ovc-munic.R
\encoding{UTF-8}
\name{catr_ovc_get_cod_munic}
\alias{catr_ovc_get_cod_munic}
\title{OVCCallejero: Extract the code of a municipality}
\usage{
catr_ovc_get_cod_munic(cpro, cmun = NULL, cmun_ine = NULL, verbose = FALSE)
}
\arguments{
\item{cpro}{The code of a province, as provided by
\code{\link[=catr_ovc_get_cod_provinces]{catr_ovc_get_cod_provinces()}}.}

\item{cmun, cmun_ine}{Code of a municipality, as recorded on the Spanish
Cadastre (\code{cmun}) or the National Statistics Institute. Either \code{cmun} or
\code{cmun_ine} must be provided.}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
A \link[tibble:tbl_df]{tibble}. See \strong{Details}
}
\description{
Implementation of the OVCCallejero service
\href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccallejerocodigos.asmx?op=ConsultaMunicipioCodigos}{ConsultaMunicipioCodigos}. Returns names and codes
of a municipality as per the Cadastre and the INE (National Statistics
Institute).
}
\details{
On a successful query, the function returns a \link[tibble:tbl_df]{tibble}
with one row including the following columns:
\itemize{
\item \code{munic}: Name of the municipality as per the Cadastre.
\item \code{catr_to}: Cadastral territorial office code.
\item \code{catr_munic}: Municipality code as recorded on the Cadastre.
\item \code{catrcode}: Full Cadastral code for the municipality.
\item \code{cpro}: Province code as per the INE.
\item \code{cmun}: Municipality code as per the INE.
\item \code{inecode}: Full INE code for the municipality.
\item Rest of fields: Check the API Docs.
}
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{
# Get municipality by cadastral code
ab <- catr_ovc_get_cod_munic(cpro = 2, cmun = 900)

ab

# Same query using the INE code

ab2 <- catr_ovc_get_cod_munic(cpro = 2, cmun_ine = 3)

ab2
}
\dontshow{\}) # examplesIf}
}
\references{
\href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccallejerocodigos.asmx?op=ConsultaMunicipioCodigos}{ConsultaMunicipioCodigos}.
}
\seealso{
\code{\link[mapSpain:esp_get_munic_siane]{mapSpain::esp_get_munic_siane()}} to get shapes of municipalities, including
the INE code.

OVCCoordenadas API:
\code{\link[=catr_ovc_get_cod_provinces]{catr_ovc_get_cod_provinces()}}

Other search:
\code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}},
\code{\link[=catr_get_code_from_coords]{catr_get_code_from_coords()}},
\code{\link[=catr_ovc_get_cod_provinces]{catr_ovc_get_cod_provinces()}}
}
\concept{OVCCallejero}
\concept{search}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atom-search.R
\encoding{UTF-8}
\name{catr_atom_search_munic}
\alias{catr_atom_search_munic}
\title{ATOM INSPIRE: Search for municipality codes}
\usage{
catr_atom_search_munic(
  munic,
  to = NULL,
  cache = deprecated(),
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE
)
}
\arguments{
\item{munic}{Municipality to extract, can be part of a string or a
cadastral code. See \code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}} for getting the cadastral
codes.}

\item{to}{Optional argument for defining the territorial office to which
\code{munic} belongs. This argument is a helper for narrowing the search.}

\item{cache}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} \code{cache} is no longer
supported, this function always caches results.}

\item{update_cache}{logical, should the cached file be refreshed? Defaults to
\code{FALSE}. When set to \code{TRUE}, it forces a new download.}

\item{cache_dir}{Path to a cache directory. On \code{NULL}, the function
stores cached files in a temporary directory (see \code{\link[base:tempdir]{base::tempdir()}}).}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
A \link[tibble:tbl_df]{tibble}.
}
\description{
Search for a municipality (as a string, part of a string, or code) and get
the corresponding code as per the Cadastre.
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{
catr_atom_search_munic("Mad")
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other INSPIRE ATOM services:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}}

Other search:
\code{\link[=catr_get_code_from_coords]{catr_get_code_from_coords()}},
\code{\link[=catr_ovc_get_cod_munic]{catr_ovc_get_cod_munic()}},
\code{\link[=catr_ovc_get_cod_provinces]{catr_ovc_get_cod_provinces()}}

Other databases:
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link{catr_srs_values}}
}
\concept{ATOM}
\concept{databases}
\concept{search}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wms-all.R
\encoding{UTF-8}
\name{catr_wms_get_layer}
\alias{catr_wms_get_layer}
\title{WMS INSPIRE: Download map images}
\usage{
catr_wms_get_layer(
  x,
  srs = NULL,
  what = c("building", "buildingpart", "parcel", "zoning", "address", "admboundary",
    "admunit"),
  styles = "default",
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE,
  crop = FALSE,
  options = NULL,
  ...
)
}
\arguments{
\item{x}{See \strong{Bounding box}. It may be:
\itemize{
\item A numeric vector of length 4 with the coordinates that defines
the bounding box: \code{c(xmin, ymin, xmax, ymax)}
\item A \code{sf/sfc} object, as provided by the \CRANpkg{sf} package.
}}

\item{srs}{SRS/CRS to use on the query. To see allowed values, use
\link{catr_srs_values}, specifically the \code{wfs_service} column. See
\strong{Bounding box}.}

\item{what, styles}{Layer and style of the WMS layer to be downloaded. See
\strong{Layers and styles}.}

\item{update_cache}{logical, should the cached file be refreshed? Defaults to
\code{FALSE}. When set to \code{TRUE}, it forces a new download.}

\item{cache_dir}{Path to a cache directory. On \code{NULL}, the function
stores cached files in a temporary directory (see \code{\link[base:tempdir]{base::tempdir()}}).}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}

\item{crop}{logical. If \code{TRUE}, the results will be cropped to the specified
\code{x} extent. If \code{x} is an \code{\link[sf:sf]{sf}} object with one \code{POINT},
\code{crop} is set to \code{FALSE}. See \code{\link[terra:crop]{terra::crop()}}.}

\item{options}{A named list containing additional options to pass to the
query.}

\item{...}{
  Arguments passed on to \code{\link[mapSpain:esp_get_tiles]{mapSpain::esp_get_tiles}}
  \describe{
    \item{\code{res}}{character string or number. Only valid for WMS providers.
Resolution (in pixels) of the final tile.}
    \item{\code{bbox_expand}}{number. Expansion percentage of the bounding box of \code{x}.}
    \item{\code{transparent}}{logical. Provides transparent background, if supported.}
    \item{\code{mask}}{logical. \code{TRUE} if the result should be masked to \code{x}. See
\code{\link[terra:mask]{terra::mask()}}.}
  }}
}
\value{
A \code{\link[terra:rast]{SpatRaster}} is returned, with 3 (RGB) or 4 (RGBA) layers,
see \code{\link[terra:RGB]{terra::RGB()}}.
}
\description{
Get geotagged images from the Spanish Cadastre. This function is a
wrapper of \code{\link[mapSpain:esp_get_tiles]{mapSpain::esp_get_tiles()}}.
}
\section{Bounding box}{

When \code{x} is a numeric vector, make sure that the \code{srs} matches the
coordinate values. When \code{x} is a \code{\link[sf:st_sf]{sf}} object, the value
\code{srs} is ignored.

The query is performed using \href{https://epsg.io/3857}{EPSG:3857} (Web Mercator)
and the tile is projected back to the SRS of \code{x}. In case that the tile
looks deformed, try either providing \code{x} or specify the SRS of the requested
tile via the \code{srs} argument, which should match the SRS of
\code{x}. See \strong{Examples}.
}

\section{Layers and styles}{

\subsection{Layers}{

The argument \code{what} defines the layer to be extracted. The equivalence with
the
\href{https://www.catastro.hacienda.gob.es/webinspire/documentos/inspire-WMS.pdf}{API Docs}
reference is:
\itemize{
\item \code{"parcel"}: CP.CadastralParcel
\item \code{"zoning"}: CP.CadastralZoning
\item \code{"building"}: BU.Building
\item \code{"buildingpart"}: BU.BuildingPart
\item \code{"address"}: AD.Address
\item \code{"admboundary"}: AU.AdministrativeBoundary
\item \code{"admunit"}: AU.AdministrativeUnit
}
}

\subsection{Styles}{

The WMS service provides different styles on each layer (\code{what} argument).
Some of the styles available are:
\itemize{
\item \code{"parcel"}: styles : \code{"BoundariesOnly"}, \code{"ReferencePointOnly"},
\code{"ELFCadastre"}.
\item \code{"zoning"}: styles : \code{"BoundariesOnly"}, \code{"ELFCadastre"}.
\item \code{"building"}, \code{"buildingpart"}: \code{"ELFCadastre"}
\item \code{"address"}: \code{"Number.ELFCadastre"}
\item \code{"admboundary"}, \code{"admunit"}: \code{"ELFCadastre"}
}

Check the \href{https://www.catastro.hacienda.gob.es/webinspire/documentos/inspire-WMS.pdf}{API Docs}
for more information.
}
}

\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{

# With a bbox

pict <- catr_wms_get_layer(
  c(222500, 4019500, 223700, 4020700),
  srs = 25830,
  what = "parcel"
)

library(mapSpain)
library(ggplot2)
library(tidyterra)

ggplot() +
  geom_spatraster_rgb(data = pict)

# With a spatial object

parcels <- catr_wfs_get_parcels_neigh_parcel("3662303TF3136B", srs = 25830)

# Use styles

parcels_img <- catr_wms_get_layer(parcels,
  what = "buildingpart",
  srs = 25830, # As parcels object
  bbox_expand = 0.3,
  styles = "ELFCadastre"
)

ggplot() +
  geom_sf(data = parcels, fill = "blue", alpha = 0.5) +
  geom_spatraster_rgb(data = parcels_img)
}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[mapSpain:esp_get_tiles]{mapSpain::esp_get_tiles()}} and \code{\link[terra:RGB]{terra::RGB()}}. For plotting see
\code{\link[terra:plotRGB]{terra::plotRGB()}} and \code{\link[tidyterra:geom_spatraster_rgb]{tidyterra::geom_spatraster_rgb()}}.

INSPIRE API functions:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_wfs_get_address_bbox]{catr_wfs_get_address_bbox()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=inspire_wfs_get]{inspire_wfs_get()}}

Other spatial:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_wfs_get_address_bbox]{catr_wfs_get_address_bbox()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}}
}
\concept{INSPIRE}
\concept{WMS}
\concept{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache-utils.R
\encoding{UTF-8}
\name{catr_set_cache_dir}
\alias{catr_set_cache_dir}
\alias{catr_detect_cache_dir}
\title{Set your \CRANpkg{CatastRo} cache dir}
\usage{
catr_set_cache_dir(
  cache_dir = NULL,
  overwrite = FALSE,
  install = FALSE,
  verbose = TRUE
)

catr_detect_cache_dir()
}
\arguments{
\item{cache_dir}{Path to a cache directory. On \code{NULL}, the function
stores cached files in a temporary directory (see \code{\link[base:tempdir]{base::tempdir()}}).}

\item{overwrite}{Logical. If \code{TRUE}, overwrites an existing
\code{CATASTROESP_CACHE_DIR} already present on your machine.}

\item{install}{Logical. If \code{TRUE}, installs the key on your local
machine for use in future sessions. Defaults to \code{FALSE}. If \code{cache_dir}
is \code{FALSE}, this argument is set to \code{FALSE} automatically.}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
\code{catr_set_cache_dir()} returns an (invisible) character with the path to
your \code{cache_dir}, but it is mainly called for its side effect.

\code{catr_detect_cache_dir()} returns the path to the \code{cache_dir} used in this
session.
}
\description{
Store your \code{cache_dir} path on your local machine for future sessions.
Type \code{Sys.getenv("CATASTROESP_CACHE_DIR")} to find your cached path or
use \code{\link[=catr_detect_cache_dir]{catr_detect_cache_dir()}}.
}
\details{
By default, when no \code{cache_dir} is set, the package uses a folder inside
\code{\link[base:tempdir]{base::tempdir()}} (so files are temporary and are removed when the \strong{R}
session ends). To persist a cache across \strong{R} sessions, use
\code{catr_set_cache_dir(cache_dir, install = TRUE)} which writes the chosen
path to a small configuration file under
\code{tools::R_user_dir("CatastRo", "config")}.
}
\note{
In \CRANpkg{CatastRo} >= 1.0.0 the location of the configuration file has
moved from \code{rappdirs::user_config_dir("CatastRo", "R")} to
\code{tools::R_user_dir("CatastRo", "config")}. We have implemented a
functionality that migrates previous configuration files from one
location to another with a message. This message appears only once
informing of the migration.
}
\section{Caching strategies}{


Some files can be read from their online source without caching using the
option \code{cache = FALSE}. Otherwise the source file is downloaded to
your computer. \CRANpkg{CatastRo} implements the following caching options:
\itemize{
\item For occasional use, rely on the default \code{\link[=tempdir]{tempdir()}}-based cache (no
install).
\item Modify the cache for a single session setting
\code{catr_set_cache_dir(cache_dir = "a/path/here")}.
\item For reproducible workflows, install a persistent cache with
\code{catr_set_cache_dir(cache_dir = "a/path/here", install = TRUE)}.
This cache is kept across \strong{R} sessions.
\item For caching specific files, use the \code{cache_dir} argument in the
corresponding function.
}

Sometimes cached files may be corrupt. In that case, try re-downloading
the data setting \code{update_cache = TRUE} in the corresponding function.

If you experience any problem downloading, try downloading the
corresponding file by another method and save it in your
\code{cache_dir}. Use the option \code{verbose = TRUE} for debugging the API query
and \code{\link[=catr_detect_cache_dir]{catr_detect_cache_dir()}} to identify your cached path.
}

\examples{

# Caution! It modifies your current state
\dontrun{
my_cache <- catr_detect_cache_dir()

# Set an example cache
ex <- file.path(tempdir(), "example", "cachenew")
catr_set_cache_dir(ex)

catr_detect_cache_dir()

# Restore initial cache
catr_set_cache_dir(my_cache)
identical(my_cache, catr_detect_cache_dir())
}


catr_detect_cache_dir()

}
\seealso{
\code{\link[tools:R_user_dir]{tools::R_user_dir()}}

Other cache utilities:
\code{\link[=catr_clear_cache]{catr_clear_cache()}}
}
\concept{cache utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atom-bu.R
\encoding{UTF-8}
\name{catr_atom_get_buildings}
\alias{catr_atom_get_buildings}
\title{ATOM INSPIRE: Download all buildings of a municipality}
\usage{
catr_atom_get_buildings(
  munic,
  to = NULL,
  what = c("building", "buildingpart", "other"),
  cache = deprecated(),
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE
)
}
\arguments{
\item{munic}{Municipality to extract, can be part of a string or a
cadastral code. See \code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}} for getting the cadastral
codes.}

\item{to}{Optional argument for defining the territorial office to which
\code{munic} belongs. This argument is a helper for narrowing the search.}

\item{what}{Information to load. Options are:
\itemize{
\item \code{"building"} for buildings.
\item \code{"buildingpart"} for parts of a building.
\item \code{"other"} for other elements such as swimming pools.
}}

\item{cache}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} \code{cache} is no longer
supported, this function always caches results.}

\item{update_cache}{logical, should the cached file be refreshed? Defaults to
\code{FALSE}. When set to \code{TRUE}, it forces a new download.}

\item{cache_dir}{Path to a cache directory. On \code{NULL}, the function
stores cached files in a temporary directory (see \code{\link[base:tempdir]{base::tempdir()}}).}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
A \code{\link[sf:st_sf]{sf}} object.
}
\description{
Retrieve the spatial data of all buildings belonging to a single municipality
using the INSPIRE ATOM service.
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{
s <- catr_atom_get_buildings("Nava de la Asuncion", to = "Segovia")

library(ggplot2)
ggplot(s) +
  geom_sf() +
  coord_sf(
    xlim = c(374500, 375500),
    ylim = c(4556500, 4557500)
  ) +
  labs(
    title = "Buildings",
    subtitle = "Nava de la Asuncion, Segovia"
  )
}
\dontshow{\}) # examplesIf}
}
\references{
\href{https://www.catastro.hacienda.gob.es/webinspire/documentos/inspire-ATOM.pdf}{API Documentation}.

\href{https://www.catastro.hacienda.gob.es/webinspire/index.html}{INSPIRE Services for Cadastral Cartography}.
}
\seealso{
INSPIRE API functions:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_wfs_get_address_bbox]{catr_wfs_get_address_bbox()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=catr_wms_get_layer]{catr_wms_get_layer()}},
\code{\link[=inspire_wfs_get]{inspire_wfs_get()}}

Other INSPIRE ATOM services:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}}

Other buildings:
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}}

Other spatial:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_wfs_get_address_bbox]{catr_wfs_get_address_bbox()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=catr_wms_get_layer]{catr_wms_get_layer()}}
}
\concept{ATOM}
\concept{INSPIRE}
\concept{buildings}
\concept{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wfs-ad.R
\encoding{UTF-8}
\name{catr_wfs_get_address_bbox}
\alias{catr_wfs_get_address_bbox}
\alias{catr_wfs_get_address_codvia}
\alias{catr_wfs_get_address_rc}
\alias{catr_wfs_get_address_postalcode}
\title{WFS INSPIRE: Download addresses}
\usage{
catr_wfs_get_address_bbox(x, srs = NULL, verbose = FALSE)

catr_wfs_get_address_codvia(codvia, del, mun, srs = NULL, verbose = FALSE)

catr_wfs_get_address_rc(rc, srs = NULL, verbose = FALSE)

catr_wfs_get_address_postalcode(postalcode, srs = NULL, verbose = FALSE)
}
\arguments{
\item{x}{See \strong{Bounding box}. It may be:
\itemize{
\item A numeric vector of length 4 with the coordinates that defines
the bounding box: \code{c(xmin, ymin, xmax, ymax)}
\item A \code{sf/sfc} object, as provided by the \CRANpkg{sf} package.
}}

\item{srs}{SRS/CRS to use on the query. To see allowed values, use
\link{catr_srs_values}, specifically the \code{wfs_service} column. See
\strong{Bounding box}.}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}

\item{codvia}{Cadastral street code.}

\item{del}{Cadastral office code.}

\item{mun}{Cadastral municipality code.}

\item{rc}{The cadastral reference to be extracted.}

\item{postalcode}{Postal code.}
}
\value{
A \code{\link[sf:st_sf]{sf}} object.
}
\description{
Get the spatial data of addresses. The WFS Service allows performing
several types of queries:
\itemize{
\item By bounding box: Implemented on \code{catr_wfs_get_address_bbox()}.
Extract objects included in the bounding box provided. See
\strong{Bounding box}.
}

\itemize{
\item By street code: Implemented on \code{catr_wfs_get_address_codvia()}. Extract
objects of specific addresses.
}

\itemize{
\item By cadastral reference: Implemented on \code{catr_wfs_get_address_rc()}. Extract
objects of specific cadastral references.
}

\itemize{
\item By postal codes: Implemented on \code{catr_wfs_get_address_postalcode()}.
Extract objects of specific postal codes
}
}
\section{API Limits}{

The API service is limited to a bounding box of 4km2 and a maximum of 5,000
elements.
}

\section{Bounding box}{

When \code{x} is a numeric vector, make sure that the \code{srs} matches the
coordinate values. Additionally, the function queries the bounding box on
\href{https://epsg.io/25830}{EPSG:25830} - ETRS89 / UTM zone 30N, to overcome
a potential bug on the API side.

When \code{x} is a \code{\link[sf:st_sf]{sf}} object, the value \code{srs} is ignored. In
this case, the bounding box of the \code{\link[sf:st_sf]{sf}} object is
used for the query (see \code{\link[sf:st_bbox]{sf::st_bbox()}}).

The result is always provided in the SRS of the \code{\link[sf:st_sf]{sf}} object
provided as input.
}

\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{
ad <- catr_wfs_get_address_bbox(
  c(
    233673, 4015968, 233761, 4016008
  ),
  srs = 25830
)

library(ggplot2)

ggplot(ad) +
  geom_sf()
}
\dontshow{\}) # examplesIf}
}
\references{
\href{https://www.catastro.hacienda.gob.es/webinspire/documentos/inspire-ad-WFS.pdf}{API Documentation}.

\href{https://www.catastro.hacienda.gob.es/webinspire/index.html}{INSPIRE Services for Cadastral Cartography}.
}
\seealso{
INSPIRE API functions:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=catr_wms_get_layer]{catr_wms_get_layer()}},
\code{\link[=inspire_wfs_get]{inspire_wfs_get()}}

Other INSPIRE WFS services:
\code{\link{catr_srs_values}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=inspire_wfs_get]{inspire_wfs_get()}}

Other addresses:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}}

Other spatial:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=catr_wms_get_layer]{catr_wms_get_layer()}}
}
\concept{INSPIRE}
\concept{WFS}
\concept{addresses}
\concept{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ovc-cpmrc.R
\encoding{UTF-8}
\name{catr_ovc_get_cpmrc}
\alias{catr_ovc_get_cpmrc}
\title{OVCCoordenadas: Geocode a cadastral reference}
\usage{
catr_ovc_get_cpmrc(
  rc,
  srs = 4326,
  province = NULL,
  municipality = NULL,
  verbose = FALSE
)
}
\arguments{
\item{rc}{The cadastral reference to be geocoded.}

\item{srs}{SRS/CRS to use on the query. To see allowed values, use
\link{catr_srs_values}, specifically the \code{ovc_service} column.}

\item{province, municipality}{Optional, used for narrowing the search.}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
A \link[tibble:tbl_df]{tibble}. See \strong{Details}
}
\description{
Implementation of the OVCCoordenadas service
\href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccoordenadas.asmx?op=Consulta_CPMRC}{Consulta CPMRC}. Returns coordinates for a
specific cadastral reference.
}
\details{
When the API does not provide any result, the function returns a
\link[tibble:tbl_df]{tibble} with the input arguments only.

On a successful query, the function returns a \link[tibble:tbl_df]{tibble}
with one row per cadastral reference, including the following columns:
\itemize{
\item \code{xcoord}, \code{ycoord}: X and Y coordinates in the specified SRS.
\item \code{refcat}: Cadastral Reference.
\item \code{address}: Address as recorded in the Cadastre.
\item Rest of fields: Check the API Docs.
}
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{

# Using all arguments
catr_ovc_get_cpmrc("13077A01800039",
  4230,
  province = "CIUDAD REAL",
  municipality = "SANTA CRUZ DE MUDELA"
)

# Only the cadastral reference
catr_ovc_get_cpmrc("9872023VH5797S")
}
\dontshow{\}) # examplesIf}
}
\references{
\href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccoordenadas.asmx?op=Consulta_CPMRC}{Consulta CPMRC}.
}
\seealso{
\link{catr_srs_values}, \code{vignette("ovcservice", package = "CatastRo")}

OVCCoordenadas API:
\code{\link[=catr_ovc_get_rccoor]{catr_ovc_get_rccoor()}},
\code{\link[=catr_ovc_get_rccoor_distancia]{catr_ovc_get_rccoor_distancia()}},
\code{\link{catr_srs_values}}

Other cadastral references:
\code{\link[=catr_ovc_get_rccoor]{catr_ovc_get_rccoor()}},
\code{\link[=catr_ovc_get_rccoor_distancia]{catr_ovc_get_rccoor_distancia()}}
}
\concept{OVCCoordenadas}
\concept{cadastral references}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\encoding{UTF-8}
\name{catr_srs_values}
\alias{catr_srs_values}
\title{Reference SRS codes for \CRANpkg{CatastRo} APIs}
\format{
A \link[tibble:tbl_df]{tibble} with 16 rows
and columns:
\describe{
\item{SRS}{Spatial Reference System (CRS) value, identified by the
corresponding
\href{https://en.wikipedia.org/wiki/EPSG_Geodetic_Parameter_Dataset}{EPSG}
code.}
\item{Description}{Description of the SRS/EPSG code.}
\item{ovc_service}{Logical. Is this code valid on OVC services?}
\item{wfs_service}{Logical. Is this code valid on INSPIRE WFS services?}
}
}
\description{
A \link[tibble:tbl_df]{tibble} including the valid SRS (also known as CRS)
values that may be used on each API service. Values are provided as
\href{https://en.wikipedia.org/wiki/EPSG_Geodetic_Parameter_Dataset}{EPSG codes}.
}
\details{
Table: Content of \link{catr_srs_values}\tabular{llll}{
   \strong{SRS} \tab \strong{Description} \tab \strong{ovc_service} \tab \strong{wfs_service} \cr
   \code{3785} \tab \verb{Web Mercator} \tab \code{FALSE} \tab \code{TRUE} \cr
   \code{3857} \tab \verb{Web Mercator} \tab \code{FALSE} \tab \code{TRUE} \cr
   \code{4230} \tab \verb{Geográficas en ED 50} \tab \code{TRUE} \tab \code{FALSE} \cr
   \code{4258} \tab \verb{Geográficas en ETRS89} \tab \code{TRUE} \tab \code{TRUE} \cr
   \code{4326} \tab \verb{Geográficas en WGS 80} \tab \code{TRUE} \tab \code{TRUE} \cr
   \code{23029} \tab \verb{UTM huso 29N en ED50} \tab \code{TRUE} \tab \code{FALSE} \cr
   \code{23030} \tab \verb{UTM huso 30N en ED50} \tab \code{TRUE} \tab \code{FALSE} \cr
   \code{23031} \tab \verb{UTM huso 31N en ED50} \tab \code{TRUE} \tab \code{FALSE} \cr
   \code{25829} \tab \verb{UTM huso 29N en ETRS89} \tab \code{TRUE} \tab \code{TRUE} \cr
   \code{25830} \tab \verb{UTM huso 30N en ETRS89} \tab \code{TRUE} \tab \code{TRUE} \cr
   \code{25831} \tab \verb{UTM huso 31N en ETRS89} \tab \code{TRUE} \tab \code{TRUE} \cr
   \code{32627} \tab \verb{UTM huso 27N en WGS 84} \tab \code{TRUE} \tab \code{FALSE} \cr
   \code{32628} \tab \verb{UTM huso 28N en WGS 84} \tab \code{TRUE} \tab \code{FALSE} \cr
   \code{32629} \tab \verb{UTM huso 29N en WGS 84} \tab \code{TRUE} \tab \code{FALSE} \cr
   \code{32630} \tab \verb{UTM huso 30N en WGS 84} \tab \code{TRUE} \tab \code{FALSE} \cr
   \code{32631} \tab \verb{UTM huso 31N en WGS 84} \tab \code{TRUE} \tab \code{FALSE} \cr
}
}
\examples{
data("catr_srs_values")

# OVC valid codes
library(dplyr)

catr_srs_values |> filter(ovc_service)

# WFS valid codes

catr_srs_values |> filter(wfs_service)

# Use with sf::st_crs()

catr_srs_values |>
  filter(wfs_service & ovc_service) |>
  print() |>
  # First value
  slice_head(n = 1) |>
  pull(SRS) |>
  # As crs
  sf::st_crs(.)
}
\references{
\itemize{
\item \href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccoordenadas.asmx}{OVCCoordenadas}.
\item \href{https://www.catastro.hacienda.gob.es/webinspire/index.html}{INSPIRE WFS Service}.
}
}
\seealso{
\code{\link[sf:st_crs]{sf::st_crs()}}.

Other databases:
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}}

Other INSPIRE WFS services:
\code{\link[=catr_wfs_get_address_bbox]{catr_wfs_get_address_bbox()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=inspire_wfs_get]{inspire_wfs_get()}}

OVCCoordenadas API:
\code{\link[=catr_ovc_get_cpmrc]{catr_ovc_get_cpmrc()}},
\code{\link[=catr_ovc_get_rccoor]{catr_ovc_get_rccoor()}},
\code{\link[=catr_ovc_get_rccoor_distancia]{catr_ovc_get_rccoor_distancia()}}
}
\concept{OVCCoordenadas}
\concept{WFS}
\concept{databases}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atom-bu-db.R
\encoding{UTF-8}
\name{catr_atom_get_buildings_db_all}
\alias{catr_atom_get_buildings_db_all}
\alias{catr_atom_get_buildings_db_to}
\title{ATOM INSPIRE: Reference database for ATOM buildings}
\source{
\url{https://www.catastro.hacienda.gob.es/INSPIRE/buildings/ES.SDGC.BU.atom.xml}
}
\usage{
catr_atom_get_buildings_db_all(
  cache = deprecated(),
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE
)

catr_atom_get_buildings_db_to(
  to,
  cache = deprecated(),
  update_cache = FALSE,
  cache_dir = NULL,
  verbose = FALSE
)
}
\arguments{
\item{cache}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} \code{cache} is no longer
supported, this function always caches results.}

\item{update_cache}{logical, should the cached file be refreshed? Defaults to
\code{FALSE}. When set to \code{TRUE}, it forces a new download.}

\item{cache_dir}{Path to a cache directory. On \code{NULL}, the function
stores cached files in a temporary directory (see \code{\link[base:tempdir]{base::tempdir()}}).}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}

\item{to}{character. Territorial office. Internally uses \code{\link[base:grep]{base::grep()}} for
matching.}
}
\value{
A \link[tibble:tbl_df]{tibble} with the information requested with the following
fields:
\itemize{
\item \code{territorial_office}: territorial office, corresponding to each province
of Spain except the Basque Country and Navarre.
\item \code{url}: ATOM URL for the corresponding territorial office.
\item \code{munic}: Name of the municipality.
\item \code{date}: Reference date of the data. Note that \strong{the information from this
service is updated twice a year}.
}
}
\description{
Create a database containing the URLs provided in the INSPIRE ATOM
service of the Spanish Cadastre for extracting buildings.
\itemize{
\item \code{catr_atom_get_buildings_db_all()} provides a top-level table
including information on all the territorial offices (except the
Basque Country and Navarre) listing the municipalities included in
each office.
\item \code{catr_atom_get_buildings_db_to()} provides a table for the specified
territorial office including information for each of the
municipalities of that office.
}
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{
catr_atom_get_buildings_db_all()
}
\dontshow{\}) # examplesIf}
}
\seealso{
INSPIRE API functions:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_wfs_get_address_bbox]{catr_wfs_get_address_bbox()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=catr_wms_get_layer]{catr_wms_get_layer()}},
\code{\link[=inspire_wfs_get]{inspire_wfs_get()}}

Other INSPIRE ATOM services:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}}

Other buildings:
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}}

Other databases:
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}},
\code{\link{catr_srs_values}}
}
\concept{ATOM}
\concept{INSPIRE}
\concept{buildings}
\concept{databases}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ovc-provinces.R
\encoding{UTF-8}
\name{catr_ovc_get_cod_provinces}
\alias{catr_ovc_get_cod_provinces}
\title{OVCCallejero: Extract a list of provinces with their codes}
\usage{
catr_ovc_get_cod_provinces(verbose = FALSE)
}
\arguments{
\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
A \link[tibble:tbl_df]{tibble}.
}
\description{
Implementation of the OVCCallejero service
\href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccallejerocodigos.asmx?op=ConsultaProvincia}{ConsultaProvincia}. Returns a list of provinces
included in the Spanish Cadastre.
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{
catr_ovc_get_cod_provinces()
}
\dontshow{\}) # examplesIf}
}
\references{
\href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccallejerocodigos.asmx?op=ConsultaProvincia}{ConsultaProvincia}.
}
\seealso{
OVCCoordenadas API:
\code{\link[=catr_ovc_get_cod_munic]{catr_ovc_get_cod_munic()}}

Other search:
\code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}},
\code{\link[=catr_get_code_from_coords]{catr_get_code_from_coords()}},
\code{\link[=catr_ovc_get_cod_munic]{catr_ovc_get_cod_munic()}}
}
\concept{OVCCallejero}
\concept{search}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-wfs.R
\encoding{UTF-8}
\name{inspire_wfs_get}
\alias{inspire_wfs_get}
\title{Client tool for WFS INSPIRE services}
\usage{
inspire_wfs_get(
  scheme = "https",
  hostname = "ovc.catastro.meh.es",
  path = "INSPIRE/wfsCP.aspx",
  query = list(),
  verbose = FALSE
)
}
\arguments{
\item{scheme}{Character string. Protocol to access the resource on the
Internet.}

\item{hostname}{Character string. Host that holds the resource.}

\item{path}{Character string. Specific resource in the host to access.}

\item{query}{Named list. Names and values of arguments to query.}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
Character string. Path of the resulting file in the \code{\link[=tempfile]{tempfile()}} folder.
}
\description{
Access WFS INSPIRE services. This function is used internally in WFS calls
and is exposed for users and developers accessing other cadastral or
INSPIRE resources.
}
\details{
This function is used internally in all the WFS calls. We expose it to make
it available to other users and/or developers for accessing other
cadastral or INSPIRE resources. See \strong{Examples}.
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
# Accessing the Cadastre of Navarra
# Try also https://ropenspain.github.io/CatastRoNav/

file_local <- inspire_wfs_get(
  hostname = "inspire.navarra.es",
  path = "services/BU/wfs",
  query = list(
    service = "WFS",
    request = "getfeature",
    typenames = "BU:Building",
    bbox = "609800,4740100,611000,4741300",
    SRSNAME = "EPSG:25830"
  )
)

if (!is.null(file_local)) {
  pamp <- sf::read_sf(file_local)

  library(ggplot2)
  ggplot(pamp) +
    geom_sf()
}
\dontshow{\}) # examplesIf}
}
\seealso{
INSPIRE API functions:
\code{\link[=catr_atom_get_address]{catr_atom_get_address()}},
\code{\link[=catr_atom_get_address_db_all]{catr_atom_get_address_db_all()}},
\code{\link[=catr_atom_get_buildings]{catr_atom_get_buildings()}},
\code{\link[=catr_atom_get_buildings_db_all]{catr_atom_get_buildings_db_all()}},
\code{\link[=catr_atom_get_parcels]{catr_atom_get_parcels()}},
\code{\link[=catr_atom_get_parcels_db_all]{catr_atom_get_parcels_db_all()}},
\code{\link[=catr_wfs_get_address_bbox]{catr_wfs_get_address_bbox()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}},
\code{\link[=catr_wms_get_layer]{catr_wms_get_layer()}}

Other INSPIRE WFS services:
\code{\link{catr_srs_values}},
\code{\link[=catr_wfs_get_address_bbox]{catr_wfs_get_address_bbox()}},
\code{\link[=catr_wfs_get_buildings_bbox]{catr_wfs_get_buildings_bbox()}},
\code{\link[=catr_wfs_get_parcels_bbox]{catr_wfs_get_parcels_bbox()}}
}
\concept{INSPIRE}
\concept{WFS}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run-example.R
\encoding{UTF-8}
\name{run_example}
\alias{run_example}
\title{Decide whether an example should run}
\usage{
run_example()
}
\value{
Logical. \code{TRUE} if examples should run, \code{FALSE} otherwise.
}
\description{
Determine whether an example should run based on the current platform and
network availability.
}
\details{
Returns \code{FALSE} on CRAN, macOS, or when offline.
}
\examples{
run_example()

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache-utils.R
\encoding{UTF-8}
\name{catr_clear_cache}
\alias{catr_clear_cache}
\title{Clear your \CRANpkg{CatastRo} cache dir}
\usage{
catr_clear_cache(config = FALSE, cached_data = TRUE, verbose = FALSE)
}
\arguments{
\item{config}{If \code{TRUE}, deletes the configuration folder of
\CRANpkg{CatastRo}.}

\item{cached_data}{If \code{TRUE}, deletes your \code{cache_dir} and all its contents.}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
Invisible. This function is called for its side effects.
}
\description{
\strong{Use this function with caution}. This function clears your cached data
and configuration, specifically:
\itemize{
\item Deletes the \CRANpkg{CatastRo} config directory
(\code{tools::R_user_dir("CatastRo", "config")}).
\item Deletes the \code{cache_dir} directory.
\item Deletes the values stored on \code{Sys.getenv("CATASTROESP_CACHE_DIR")}.
}
}
\details{
This is an overkill function that is intended to reset your status
as if you had never installed and/or used \CRANpkg{CatastRo}.
}
\examples{

# Don't run this! It modifies your current state
\dontrun{
my_cache <- catr_detect_cache_dir()

# Set an example cache
ex <- file.path(tempdir(), "example", "cache")
catr_set_cache_dir(ex, verbose = FALSE)

# Restore initial cache
catr_clear_cache(verbose = TRUE)

catr_set_cache_dir(my_cache)
identical(my_cache, catr_detect_cache_dir())
}
}
\seealso{
\code{\link[tools:R_user_dir]{tools::R_user_dir()}}

Other cache utilities:
\code{\link[=catr_set_cache_dir]{catr_set_cache_dir()}}
}
\concept{cache utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ovc-rccoor-dist.R
\encoding{UTF-8}
\name{catr_ovc_get_rccoor_distancia}
\alias{catr_ovc_get_rccoor_distancia}
\title{OVCCoordenadas: Reverse geocode cadastral references on a region}
\usage{
catr_ovc_get_rccoor_distancia(lat, lon, srs = 4326, verbose = FALSE)
}
\arguments{
\item{lat}{Latitude for the query, expressed in the CRS/SRS defined by
\code{srs}.}

\item{lon}{Longitude for the query, expressed in the CRS/SRS defined by
\code{srs}.}

\item{srs}{SRS/CRS to use on the query. To see allowed values, use
\link{catr_srs_values}, specifically the \code{ovc_service} column.}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}
}
\value{
A \link[tibble:tbl_df]{tibble}. See \strong{Details}
}
\description{
Implementation of the OVCCoordenadas service
\href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccoordenadas.asmx?op=Consulta_RCCOOR_Distancia}{Consulta RCCOOR Distancia}. Return cadastral
references for coordinates. If none found, the API returns references
in a 50 square meter area around the requested coordinates.
}
\details{
When the API does not provide any result, the function returns a
\link[tibble:tbl_df]{tibble} with the input arguments only.

On a successful query, the function returns a \link[tibble:tbl_df]{tibble} with
one row by cadastral reference, including the following columns:
\itemize{
\item \code{geo.xcen}, \code{geo.ycen}, \code{geo.srs}: Input arguments of the query.
\item \code{refcat}: Cadastral reference.
\item \code{address}: Address as recorded in the Cadastre.
\item \code{cmun_ine}: Municipality code as registered on the INE (National
Statistics Institute).
\item Rest of fields: Check the API Docs.
}
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{
catr_ovc_get_rccoor_distancia(
  lat = 40.963200,
  lon = -5.671420,
  srs = 4326
)
}
\dontshow{\}) # examplesIf}
}
\references{
\href{https://ovc.catastro.meh.es/ovcservweb/ovcswlocalizacionrc/ovccoordenadas.asmx?op=Consulta_RCCOOR_Distancia}{Consulta RCCOOR Distancia}.
}
\seealso{
\link{catr_srs_values}, \code{vignette("ovcservice", package = "CatastRo")}

OVCCoordenadas API:
\code{\link[=catr_ovc_get_cpmrc]{catr_ovc_get_cpmrc()}},
\code{\link[=catr_ovc_get_rccoor]{catr_ovc_get_rccoor()}},
\code{\link{catr_srs_values}}

Other cadastral references:
\code{\link[=catr_ovc_get_cpmrc]{catr_ovc_get_cpmrc()}},
\code{\link[=catr_ovc_get_rccoor]{catr_ovc_get_rccoor()}}
}
\concept{OVCCoordenadas}
\concept{cadastral references}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catr-code-coords.R
\encoding{UTF-8}
\name{catr_get_code_from_coords}
\alias{catr_get_code_from_coords}
\title{Get the cadastral municipality code from coordinates}
\usage{
catr_get_code_from_coords(
  x,
  srs = NULL,
  verbose = FALSE,
  cache_dir = NULL,
  ...
)
}
\arguments{
\item{x}{It may be:
\itemize{
\item A pair of coordinates \code{c(x,y)}. In this case the \code{srs} of the coordinates
must be provided.
\item A \code{\link[sf:st_sf]{sf}} object. If the object has several geometries, only
the first geometry is used. The function extracts coordinates using
\code{sf::st_centroid(x, of_largest_polygon = TRUE)}.
}}

\item{srs}{SRS/CRS to use on the query. To see allowed values, use
\link{catr_srs_values}, specifically the \code{ovc_service} column.}

\item{verbose}{Logical. If \code{TRUE}, displays informational messages.}

\item{cache_dir}{Path to a cache directory. On \code{NULL}, the function
stores cached files in a temporary directory (see \code{\link[base:tempdir]{base::tempdir()}}).}

\item{...}{
  Arguments passed on to \code{\link[mapSpain:esp_get_munic_siane]{mapSpain::esp_get_munic_siane}}
  \describe{
    \item{\code{year}}{character string or number. Release year, it must be in
formats \code{YYYY} (assuming end of year) or \code{YYYY-MM-DD}. Historical
information starts as of 2005.}
    \item{\code{resolution}}{character string or number. Resolution of the geospatial
data. One of:
\itemize{
\item "10": 1:10 million.
\item "6.5": 1:6.5 million.
\item "3": 1:3 million.
}}
    \item{\code{region}}{Optional. A vector of region names, NUTS or ISO codes
(see \code{\link[mapSpain:esp_dict_region_code]{esp_dict_region_code()}}).}
    \item{\code{munic}}{character string. A name or \code{\link[base:grep]{regex}} expression
with the names of the required municipalities. \code{NULL} will return all
municipalities.}
  }}
}
\value{
A \link[tibble:tbl_df]{tibble}. See \strong{Details}
}
\description{
Get the municipality code for coordinates using a \code{\link[sf:st_sf]{sf}}
object or a pair of coordinates via \code{\link[=catr_ovc_get_cod_munic]{catr_ovc_get_cod_munic()}}.
}
\details{
On a successful query, the function returns a \link[tibble:tbl_df]{tibble}
with one row including the following columns:
\itemize{
\item \code{munic}: Name of the municipality as per the Cadastre.
\item \code{catr_to}: Cadastral territorial office code.
\item \code{catr_munic}: Municipality code as recorded on the Cadastre.
\item \code{catrcode}: Full Cadastral code for the municipality.
\item \code{cpro}: Province code as per the INE.
\item \code{cmun}: Municipality code as per the INE.
\item \code{inecode}: Full INE code for the municipality.
\item Rest of fields: Check the API Docs.
}
}
\examples{
\dontshow{if (run_example()) withAutoprint(\{ # examplesIf}
\donttest{
# Use with coords
catr_get_code_from_coords(c(-16.25462, 28.46824), srs = 4326)

# Use with sf
prov <- mapSpain::esp_get_prov("Caceres")
catr_get_code_from_coords(prov)
}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[mapSpain:esp_get_munic_siane]{mapSpain::esp_get_munic_siane()}}, \code{\link[=catr_ovc_get_cod_munic]{catr_ovc_get_cod_munic()}},
\code{\link[sf:st_centroid]{sf::st_centroid()}}.

Other search:
\code{\link[=catr_atom_search_munic]{catr_atom_search_munic()}},
\code{\link[=catr_ovc_get_cod_munic]{catr_ovc_get_cod_munic()}},
\code{\link[=catr_ovc_get_cod_provinces]{catr_ovc_get_cod_provinces()}}
}
\concept{search}
